@@ -1,4 +1,3 @@
-//  $Id: file_.cpp,v 1.20.2.2 2009/07/01 10:58:09 ccb Exp $
 //  =================================================================
 //
 //   CCP4 Coordinate Library: support of coordinate-related
@@ -6,13 +5,13 @@
 //
 //   Copyright (C) Eugene Krissinel 2000-2008.
 //
-//    This library is free software: you can redistribute it and/or 
-//    modify it under the terms of the GNU Lesser General Public 
-//    License version 3, modified in accordance with the provisions 
+//    This library is free software: you can redistribute it and/or
+//    modify it under the terms of the GNU Lesser General Public
+//    License version 3, modified in accordance with the provisions
 //    of the license to address the requirements of UK law.
 //
-//    You should have received a copy of the modified GNU Lesser 
-//    General Public License along with this library. If not, copies 
+//    You should have received a copy of the modified GNU Lesser
+//    General Public License along with this library. If not, copies
 //    may be downloaded from http://www.ccp4.ac.uk/ccp4license.php
 //
 //    This program is distributed in the hope that it will be useful,
@@ -22,7 +21,7 @@
 //
 //  =================================================================
 //
-//    08.07.08   <--  Date of Last Modification.
+//    29.01.10   <--  Date of Last Modification.
 //                   ~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 //  -----------------------------------------------------------------
 //
@@ -31,7 +30,7 @@
 //  **** Classes :  CFile  - file I/O Support.
 //       ~~~~~~~~~
 //
-//  (C) E. Krissinel 2000-2008
+//  (C) E. Krissinel 2000-2010
 //
 //  =================================================================
 //
@@ -41,7 +40,7 @@
 #include <stdlib.h>
 #endif
 
-#ifdef __WIN32__
+#ifdef  _WIN32
 # include <windows.h>
 # define sleep Sleep
 # endif
@@ -54,32 +53,36 @@
 #include <string.h>
 #endif
 
-#ifndef _MVS
+#ifndef  _WIN32
 #ifndef  __UNISTD_H
 #include <unistd.h>
 #endif
 #endif
 
-
 #ifndef  __File__
 #include "file_.h"
 #endif
 
 
+#ifdef  _WIN32
+//  for DOS/WINDOWS machines:
+#define  NEWLINE  "\r\n"
+#else
 //  for UNIX machines:
 #define  NEWLINE  "\n"
-
-//  for DOS/WINDOWS machines:
-//#define  NEWLINE  "\r\n"
+#endif
 
 
 // ===================  Auxilary Functions  =========================
 
-pstr GetFPath ( pstr FilePath, int syskey )  {
-pstr P = NULL;
-  if (syskey==syskey_unix)     
+cpstr GetFPath ( pstr FilePath, int syskey )  {
+pstr P;
+
+ P = NULL; // may be undefined if syskey is not recognized
+
+  if (syskey==syskey_unix)
     P = strrchr(FilePath,'/');
-  else if (syskey==syskey_win) 
+  else if (syskey==syskey_win)
     P = strrchr(FilePath,'\\');
   else if (syskey==syskey_all)  {
     P = strrchr(FilePath,'/');
@@ -94,10 +97,13 @@ pstr P = NULL;
 }
 
 cpstr GetFName ( cpstr FilePath, int syskey )  {
-pstr P = NULL;
-  if (syskey==syskey_unix)     
+pstr P;
+
+  P = NULL; // may be undefined if syskey is not recognized
+
+  if (syskey==syskey_unix)
     P = strrchr(FilePath,'/');
-  else if (syskey==syskey_win) 
+  else if (syskey==syskey_win)
     P = strrchr(FilePath,'\\');
   else if (syskey==syskey_all)  {
     P = strrchr(FilePath,'/');
@@ -114,7 +120,7 @@ pstr P;
      else return P;
 }
 
-pstr ChangeExt ( pstr FilePath, cpstr newExt, int syskey )  {
+cpstr ChangeExt ( pstr FilePath, cpstr newExt, int syskey )  {
 int i;
   i = strlen(FilePath)-1;
   if (syskey==syskey_unix)
@@ -191,9 +197,7 @@ void  CFile::FreeBuffer ()  {
 
 void  CFile::assign ( cpstr FileName, Boolean Text, Boolean UniB,
                       byte gzMode )  {
-#ifndef _MVS
 pstr p;
-#endif
 
   shut();
 
@@ -209,8 +213,6 @@ pstr p;
 
   UniBin = UniB;
 
-#ifndef _MVS
-
   gzipMode = gzMode;
   gzipIO   = ARCH_NONE;
   if ((gzipMode==GZM_ENFORCE) || (gzipMode==GZM_ENFORCE_GZIP))
@@ -225,17 +227,11 @@ pstr p;
     }
   }
 
-#else
-
-  gzipMode = GZM_NONE;
-  gzipIO   = ARCH_NONE;
-
-#endif
-
   memIO = False;
 
 }
 
+
 void  CFile::assign ( word poolSize, word sizeInc, pstr filePool )  {
 
   shut      ();
@@ -286,8 +282,9 @@ void  SetCompressPath ( pstr compressPath, pstr uncompressPath )  {
                   else  uncompress_path = uncompressPath;
 }
 
+
 Boolean  CFile::reset ( Boolean ReadOnly, int retry )  {
-#ifndef _MVS
+#ifndef _MSC_VER
 pstr p;
 int  i;
 #endif
@@ -319,7 +316,7 @@ int  i;
 
       StdIO = False;
       if (gzipIO==ARCH_GZIP)  {
-#ifndef _MVS
+#ifndef _MSC_VER
         p = NULL;
         CreateConcat  ( p,ungzip_path,FName );
         for (i=0;(i<=retry) && (!hFile);i++)  {
@@ -330,7 +327,7 @@ int  i;
 #endif
 
       } else if (gzipIO==ARCH_COMPRESS)  {
-#ifndef _MVS
+#ifndef _MSC_VER
         p = NULL;
         CreateConcat  ( p,uncompress_path,FName );
         for (i=0;(i<=retry) && (!hFile);i++)  {
@@ -342,18 +339,16 @@ int  i;
 
       } else  {
 
-#ifndef _MVS
+#ifndef _MSC_VER
         for (i=0;(i<=retry) && (!hFile);i++)  {
           if (i>0)  sleep ( 1 );
-#endif
           if (TextMode)  {
-            if (ReadOnly)  hFile = fopen ( FName,"rt"   );
+            if (ReadOnly)  hFile = fopen ( FName,"rt"  );
                      else  hFile = fopen ( FName,"r+t" );
           } else  {
-            if (ReadOnly)  hFile = fopen ( FName,"rb"   );
+            if (ReadOnly)  hFile = fopen ( FName,"rb"  );
                      else  hFile = fopen ( FName,"r+b" );
           }
-#ifndef _MVS
         }
 #endif
 
@@ -371,7 +366,7 @@ int  i;
         }
         IOSuccess = True;
       } else  {
-        EofFile   = True;    
+        EofFile   = True;
         IOSuccess = False;
       }
 
@@ -384,7 +379,7 @@ int  i;
 }
 
 Boolean  CFile::rewrite()  {
-#ifndef _MVS
+#ifndef _MSC_VER
 pstr p;
 #endif
 
@@ -405,13 +400,9 @@ pstr p;
     BufCnt = 0;
 
     if (gzipIO==ARCH_GZIP)  {
-#ifndef _MVS
+#ifndef _MSC_VER
       p = NULL;
-#ifdef __WIN32__
-      CreateConcat  ( p,gzip_path,pstr(" | cat > "),FName );
-#else
       CreateConcat  ( p,gzip_path,pstr(" > "),FName );
-#endif
       hFile = popen ( p,"w" );
       if (p)  delete[] p;
 #else
@@ -419,7 +410,7 @@ pstr p;
 #endif
       StdIO = False;
     } else if (gzipIO==ARCH_COMPRESS)  {
-#ifndef _MVS
+#ifndef _MSC_VER
       p = NULL;
       CreateConcat  ( p,compress_path,pstr(" > "),FName );
       hFile = popen ( p,"w" );
@@ -451,8 +442,9 @@ pstr p;
 
 }
 
+
 Boolean  CFile::append()  {
-#ifndef _MVS
+#ifndef _MSC_VER
 pstr p;
 #endif
 
@@ -473,7 +465,7 @@ pstr p;
     BufLen  = 0;
     BufCnt  = 0;
     if (gzipIO==ARCH_GZIP)  {
-#ifndef _MVS
+#ifndef _MSC_VER
       p = NULL;
       CreateConcat  ( p,gzip_path,pstr(" >> "),FName );
       hFile = popen ( p,"w" );
@@ -483,7 +475,7 @@ pstr p;
 #endif
       StdIO = False;
     } else if (gzipIO==ARCH_COMPRESS)  {
-#ifndef _MVS
+#ifndef _MSC_VER
       p = NULL;
       CreateConcat  ( p,compress_path,pstr(" >> "),FName );
       hFile = popen ( p,"w" );
@@ -556,6 +548,7 @@ Boolean  CFile::exists()  {
 }
 
 Boolean  CFile::parse ( cpstr FileName )  {
+UNUSED_ARGUMENT(FileName);
   return True;
 }
 
@@ -579,10 +572,10 @@ long  CFile::Position()  {
 
 Boolean  CFile::seek ( long Position )  {
   if (memIO)  {
-    if (Position<=BufLen)  {
+    if (Position<=(long)BufLen)  {
       BufCnt    = Position;
       IOSuccess = True;
-    } else 
+    } else
       IOSuccess = False;
     return IOSuccess;
   } else if (hFile==NULL)
@@ -596,7 +589,7 @@ Boolean  CFile::seek ( long Position )  {
 
 Boolean  CFile::FileEnd()  {
 
-  if (memIO)  return (BufCnt>=FLength);
+  if (memIO)  return ((long)BufCnt>=FLength);
 
   if (TextMode)  {
     if (EofFile || ((!hFile) && (!StdIO)))
@@ -614,7 +607,7 @@ void  CFile::shut ()  {
 
   if (hFile!=NULL)  {
     if (!StdIO)  {
-#ifndef _MVS
+#ifndef _MSC_VER
       if (gzipIO!=ARCH_NONE)  pclose ( hFile );
                         else  fclose ( hFile );
 #else
@@ -639,27 +632,27 @@ Boolean  HSuccess = IOSuccess;
   if (memIO)  {
 
     LCnt = 0;
-    while ((BufCnt<FLength)       &&
-           (LCnt<MaxLen-1)       &&
-           (IOBuf[BufCnt]!='\r') &&
+    while (((long)BufCnt<FLength) &&
+           (LCnt<MaxLen-1)        &&
+           (IOBuf[BufCnt]!='\r')  &&
            (IOBuf[BufCnt]!='\n') )
       Line[LCnt++] = IOBuf[BufCnt++];
     Line[LCnt] = char(0);
 
-    while ((BufCnt<FLength)       &&
-           (IOBuf[BufCnt]!='\r') &&
+    while (((long)BufCnt<FLength) &&
+           (IOBuf[BufCnt]!='\r')  &&
            (IOBuf[BufCnt]!='\n') )
       BufCnt++;
 
-    if (BufCnt<FLength)  {
+    if ((long)BufCnt<FLength)  {
       if (IOBuf[BufCnt]=='\r')  {
         BufCnt++;
-        if (BufCnt<FLength)  {
+        if ((long)BufCnt<FLength)  {
           if (IOBuf[BufCnt]=='\n')  BufCnt++;
         }
       } else if (IOBuf[BufCnt]=='\n')  {
         BufCnt++;
-        if (BufCnt<FLength)  {
+        if ((long)BufCnt<FLength)  {
           if (IOBuf[BufCnt]=='\r')  BufCnt++;
         }
       }
@@ -678,17 +671,16 @@ Boolean  HSuccess = IOSuccess;
     }
     if (TextMode)  {
       Line[0] = char(0);
-      if (!fgets ( Line,MaxLen,hFile )) {
-	IOSuccess = False;
-	return 0;
-      }
-      LCnt = strlen(Line);
-      while (LCnt>0)  {
-        if ((Line[LCnt-1]!='\n') &&
-            (Line[LCnt-1]!='\r'))  break;
-        Line[LCnt-1] = char(0);
-        LCnt--;
-      }
+      if (fgets(Line,MaxLen,hFile))  {
+        LCnt = strlen(Line);
+        while (LCnt>0)  {
+          if ((Line[LCnt-1]!='\n') &&
+              (Line[LCnt-1]!='\r'))  break;
+          Line[LCnt-1] = char(0);
+          LCnt--;
+        }
+      } else
+        LCnt = 0;
       return LCnt;
     } else  {
       if (IOBuf==NULL)  {
@@ -852,7 +844,7 @@ int  l=strlen(S);
 Boolean  CFile::ReadParameter  ( pstr S, realtype & X,
                                  int ParColumn )  {
   ReadLine ( S );
-  if (strlen(S)>ParColumn)  {
+  if ((int)strlen(S)>ParColumn)  {
 //    X = atof ( &(S[ParColumn]) );
     X = GetNumber ( &(S[ParColumn]) );
     return True;
@@ -866,7 +858,7 @@ Boolean  CFile::ReadParameters ( pstr S, int & n_X, rvector X,
                                  int MaxLen, int ParColumn )  {
 pstr S1,S2;
   ReadLine ( S,MaxLen );
-  if (strlen(S)>ParColumn)  {
+  if ((int)strlen(S)>ParColumn)  {
     n_X = 0;
     S2 = &(S[ParColumn]);
     S1 = S2;
@@ -963,7 +955,7 @@ Boolean    B;
   } else if (UniBin)  {
     word2UniBin ( ll,wUB );
     B = WriteFile ( wUB,sizeof(wordUniBin) );
-  } else 
+  } else
     B = WriteFile ( &ll,sizeof(ll) );
   if (B && (ll>0))  B = WriteFile ( Line,ll );
   return B;
@@ -979,7 +971,7 @@ byte       sl;
   } else if (UniBin)  {
     ReadFile    ( wUB,sizeof(wordUniBin) );
     UniBin2word ( wUB,ll );
-  } else 
+  } else
     ReadFile ( &ll,sizeof(ll) );
   if (ll>0)  ReadFile ( Line,ll );
   Line[ll] = char(0);
@@ -995,11 +987,11 @@ word  Cnt;
       BufCnt += Cnt;
     }
     IOSuccess = (Cnt==Count);
-    EofFile   = ((Cnt<Count) || (BufCnt>=FLength));
+    EofFile   = ((Cnt<Count) || ((long)BufCnt>=FLength));
     return  Cnt;
   } else if (hFile)  {
     Cnt       = (word)fread ( Buffer,1,Count,hFile );
-    EofFile   = (Cnt<Count) || 
+    EofFile   = (Cnt<Count) ||
                  ((gzipIO==ARCH_NONE) && (Position()==FLength));
     IOSuccess = (Cnt==Count);
     return  Cnt;
@@ -1206,7 +1198,7 @@ wordUniBin  wUB;
       return True;
     } else
       return False;
-  } else 
+  } else
     return ( ReadFile(W,sizeof(word))==sizeof(word) );
 }
 
