@@ -1,12 +1,12 @@
 
 //
-//    This is free software: you can redistribute it and/or 
-//    modify it under the terms of the GNU Lesser General Public 
-//    License version 3, modified in accordance with the provisions 
+//    This is free software: you can redistribute it and/or
+//    modify it under the terms of the GNU Lesser General Public
+//    License version 3, modified in accordance with the provisions
 //    of the license to address the requirements of UK law.
 //
-//    You should have received a copy of the modified GNU Lesser 
-//    General Public License along with this library. If not, copies 
+//    You should have received a copy of the modified GNU Lesser
+//    General Public License along with this library. If not, copies
 //    may be downloaded from http://www.ccp4.ac.uk/ccp4license.php
 //
 //    This program is distributed in the hope that it will be useful,
@@ -138,7 +138,7 @@ decode_pure(
     else {
       have_non_blank = 1;
       dv = digits_values[si];
-      if (dv < 0 || dv >= digits_size) {
+      if (dv < 0 || dv >= (int)digits_size) {
         *result = 0;
         return invalid_number_literal();
       }
