@@ -1,4 +1,3 @@
-//  $Id: linalg_.cpp,v 1.15 2008/08/21 09:54:05 nds65 Exp $
 //  =================================================================
 //
 //   CCP4 Coordinate Library: support of coordinate-related
@@ -6,13 +5,13 @@
 //
 //   Copyright (C) Eugene Krissinel 2000-2008.
 //
-//    This library is free software: you can redistribute it and/or 
-//    modify it under the terms of the GNU Lesser General Public 
-//    License version 3, modified in accordance with the provisions 
+//    This library is free software: you can redistribute it and/or
+//    modify it under the terms of the GNU Lesser General Public
+//    License version 3, modified in accordance with the provisions
 //    of the license to address the requirements of UK law.
 //
-//    You should have received a copy of the modified GNU Lesser 
-//    General Public License along with this library. If not, copies 
+//    You should have received a copy of the modified GNU Lesser
+//    General Public License along with this library. If not, copies
 //    may be downloaded from http://www.ccp4.ac.uk/ccp4license.php
 //
 //    This program is distributed in the hope that it will be useful,
@@ -100,7 +99,7 @@ int      i,j,k,Iter;
 
   Sigma1 = 0.0;
   OffDsq = 0.0;
-  
+
   //  Sigma1  is the Dekart measure of the diagonal elements
   //  OffDsq  is the Dekart measure of the non-diagonal elements
 
@@ -578,8 +577,9 @@ int       ItnLimit=300;
 int       i,j,k,l,i1,k1,l1,its,mn,ExitKey;
 realtype  C,G,F,X,S,H,Y,Z,Scale,ANorm,GG;
 
+  l1 = 0; // used as index but defined in conditional, may be undefined
+
   RetCode = 0;
-  l1 = 0; // otherwise l1 risks being undefined
 
   if (U!=A)  {
     if (NA==M)
@@ -842,7 +842,7 @@ realtype  C,G,F,X,S,H,Y,Z,Scale,ANorm,GG;
         RV1[l] = 0.0;
         RV1[k] = F;
         W[k]   = X;
-      
+
       } else if (Z<0.0)  {
 
         W[k] = -Z;
@@ -897,7 +897,7 @@ realtype  P;
           V[j][k] = P;
         }
     }
- 
+
   }
 
 }
