@@ -1,4 +1,3 @@
-//  $Id: mattype_.h,v 1.19.2.2 2009/09/28 13:43:34 ccb Exp $
 //  =================================================================
 //
 //   CCP4 Coordinate Library: support of coordinate-related
@@ -6,13 +5,13 @@
 //
 //   Copyright (C) Eugene Krissinel 2000-2008.
 //
-//    This library is free software: you can redistribute it and/or 
-//    modify it under the terms of the GNU Lesser General Public 
-//    License version 3, modified in accordance with the provisions 
+//    This library is free software: you can redistribute it and/or
+//    modify it under the terms of the GNU Lesser General Public
+//    License version 3, modified in accordance with the provisions
 //    of the license to address the requirements of UK law.
 //
-//    You should have received a copy of the modified GNU Lesser 
-//    General Public License along with this library. If not, copies 
+//    You should have received a copy of the modified GNU Lesser
+//    General Public License along with this library. If not, copies
 //    may be downloaded from http://www.ccp4.ac.uk/ccp4license.php
 //
 //    This program is distributed in the hope that it will be useful,
@@ -22,7 +21,7 @@
 //
 //  =================================================================
 //
-//    28.07.06   <--  Date of Last Modification.
+//    29.01.10   <--  Date of Last Modification.
 //                   ~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 //  -----------------------------------------------------------------
 //
@@ -37,7 +36,7 @@
 //               strcpy_n0  ( like strcpy_n and adds terminating 0  )
 //               PadSpaces  ( pads a string with spaces             )
 //
-//  (C) E. Krissinel 2000-2008
+//  (C) E. Krissinel 2000-2010
 //
 //  =================================================================
 //
@@ -46,6 +45,10 @@
 #ifndef  __MatType__
 #define  __MatType__
 
+#ifndef __MATH_H
+#include <math.h>
+#endif
+
 #define  UseDoubleFloat
 
 #ifndef __ClassMacros
@@ -70,6 +73,7 @@
 
 #endif
 
+#define UNUSED_ARGUMENT(x) (void)x
 
 // -----------------------------------------------------
 
@@ -91,17 +95,15 @@
 
 #endif
 
+typedef   float     shortreal;
+#define   MinShortReal  1.1755e-38
+#define   MaxShortReal  3.4020e+38
+
 #define   strrchr   LastOccurence
 #define   fstrrchr  LastOccurence
 #define   strchr    FirstOccurence
 #define   fstrchr   FirstOccurence
 
-#ifdef _MVS
-#define   strncasecmp _strnicmp
-#define   strcasecmp  _stricmp
-#define   strlen      (int)strlen
-#endif
-
 typedef   char     *         pstr;
 typedef   const char *       cpstr;
 typedef   unsigned int       word;
@@ -112,12 +114,20 @@ typedef   unsigned int       word2;
 typedef   byte *             byteptr;
 typedef   unsigned long      lword;
 
-typedef   byte intUniBin  [4];
-typedef   byte shortUniBin[2];
-typedef   byte longUniBin [4];
-typedef   byte wordUniBin [4];
-typedef   byte realUniBin [10];
-typedef   byte floatUniBin[5];
+typedef   byte intUniBin      [4];
+typedef   byte shortUniBin    [2];
+typedef   byte longUniBin     [4];
+typedef   byte wordUniBin     [4];
+typedef   byte realUniBin     [10];
+typedef   byte floatUniBin    [5];
+typedef   byte shortrealUniBin[5];
+
+#ifdef _WIN32
+#define   strncasecmp _strnicmp
+#define   strcasecmp  _stricmp
+#define   strlen      (int)strlen
+pstr strcasestr ( pstr s1, cpstr s2 );
+#endif
 
 #define   True              Boolean(1)
 #define   False             Boolean(0)
@@ -177,6 +187,8 @@ typedef   psmatrix * psmatrix3;
 extern Boolean InitMatType();
 
 // ------------------------------------------------------------
+
+/*
 extern  int       mround ( realtype X );
 extern  int       ifloor ( realtype X );
 extern  int       Abs    ( int x      );
@@ -196,6 +208,58 @@ extern  long      LMin   ( const long     x1, const long     x2 );
 extern  word      WMin   ( const word     x1, const word     x2 );
 extern  int       IMin   ( const int      x1, const int      x2 );
 extern  realtype  fsign  ( const realtype x1, const realtype x2 );
+*/
+
+inline int mround ( realtype X )  { return  (int)floor(X+0.5);   }
+inline int ifloor ( realtype X )  { return  (int)floor(X);       }
+inline int Abs    ( int x )       { return ( x >= 0 ? x : -x );  }
+
+inline void ISwap ( int & x, int & y )
+{ int  b = x;  x = y;  y = b; }
+
+inline void WSwap ( word & x, word & y )
+{ word  b = x;  x = y;  y = b; }
+
+inline void BSwap ( byte & x, byte & y )
+{ byte b = x;  x = y;  y = b; }
+
+inline void LSwap ( long & x, long & y )
+{ long b = x;  x = y;  y = b; }
+
+inline void RSwap ( realtype & x, realtype & y )
+{ realtype b = x;  x = y;  y = b; }
+
+inline realtype RMax ( const realtype x1, const realtype x2 )
+{ return ( x1 > x2 ? x1 : x2 );  }
+
+inline long LMax ( const long x1, const long x2 )
+{ return ( x1 > x2 ? x1 : x2 );  }
+
+inline word WMax ( const word x1, const word x2 )
+{ return ( x1 > x2 ? x1 : x2 );  }
+
+inline int IMax ( const int x1,  const int x2  )
+{ return ( x1 > x2 ? x1 : x2 );  }
+
+inline realtype RMin ( const realtype x1, const realtype x2 )
+{ return ( x1 < x2 ? x1 : x2 );  }
+
+inline long LMin ( const long x1, const long x2 )
+{ return ( x1 < x2 ? x1 : x2 );  }
+
+inline word WMin ( const word x1, const word x2 )
+{ return ( x1 < x2 ? x1 : x2 );  }
+
+inline int  IMin ( const int x1,  const int x2  )
+{ return ( x1 < x2 ? x1 : x2 );  }
+
+inline realtype fsign ( const realtype x1,  const realtype x2 )  {
+realtype  ax;
+  if (x1>=0.0)  ax = x1;
+          else  ax = -x1;
+  return ( x2 >= 0.0 ? ax : -ax );
+}
+
 
 // ------------------------------------------------------------
 
@@ -413,7 +477,7 @@ extern void GetString ( pstr L, cpstr S, int M );
 // or terminating null is found. If the terminating null is met among
 // the first n characters or if SMax<n, the string L will be padded
 // with spaces till the length of minimum of n and LMax and then
-// terminated with the null. 
+// terminated with the null.
 //   LMax and SMax are the buffer lengths of L and S,
 // respectively. Even if no space is found, the last character
 // in L will be the terminating null.
@@ -428,7 +492,7 @@ extern void GetStrTer ( pstr L, cpstr S, int n, int LMax,
 // terminating null is found. If the terminating null
 // is met among the first n characters or if SMax<n, the string
 // L will be padded with spaces till the length of minimum of
-// n and LMax and then terminated with the null. 
+// n and LMax and then terminated with the null.
 //   SMax are buffer lengths of L and S, respectively. The last
 // character in L will be the terminating null.
 extern void GetStrTerWin32File ( pstr L, cpstr S, int n,
@@ -537,36 +601,40 @@ extern pstr EnforceSpaces ( pstr S );
 
 // -------------------------------------------------------------
 
-extern void int2UniBin   ( int      I,  intUniBin   iUB  );
-extern void short2UniBin ( short    S,  shortUniBin sUB  );
-extern void long2UniBin  ( long     L,  longUniBin  lUB  );
-extern void word2UniBin  ( word     W,  wordUniBin  wUB  );
-extern void real2UniBin  ( realtype R,  realUniBin  rUB  );
-extern void float2UniBin ( realtype R,  floatUniBin fUB  );
-extern void UniBin2int   ( intUniBin   iUB, int      & I );
-extern void UniBin2short ( shortUniBin sUB, short    & S );
-extern void UniBin2long  ( longUniBin  lUB, long     & L );
-extern void UniBin2word  ( wordUniBin  wUB, word     & W );
-extern void UniBin2real  ( realUniBin  rUB, realtype & R );
-extern void UniBin2float ( floatUniBin fUB, realtype & R );
+extern void int2UniBin       ( int       I,  intUniBin        iUB  );
+extern void short2UniBin     ( short     S,  shortUniBin      sUB  );
+extern void long2UniBin      ( long      L,  longUniBin       lUB  );
+extern void word2UniBin      ( word      W,  wordUniBin       wUB  );
+extern void real2UniBin      ( realtype  R,  realUniBin       rUB  );
+extern void float2UniBin     ( realtype  R,  floatUniBin      fUB  );
+extern void shortreal2UniBin ( shortreal R,  shortrealUniBin  srUB );
+extern void UniBin2int       ( intUniBin        iUB, int       & I );
+extern void UniBin2short     ( shortUniBin      sUB, short     & S );
+extern void UniBin2long      ( longUniBin       lUB, long      & L );
+extern void UniBin2word      ( wordUniBin       wUB, word      & W );
+extern void UniBin2real      ( realUniBin       rUB, realtype  & R );
+extern void UniBin2shortreal ( shortrealUniBin srUB, shortreal & R );
+extern void UniBin2float     ( floatUniBin      fUB, realtype  & R );
 
-extern void mem_write ( int      I, pstr S, int & l );
-extern void mem_write ( short    I, pstr S, int & l );
-extern void mem_write ( long     I, pstr S, int & l );
-extern void mem_write ( word     W, pstr S, int & l );
-extern void mem_write ( realtype R, pstr S, int & l );
-extern void mem_write ( pstr     L, int len, pstr S, int & l );
-extern void mem_write ( pstr     L, pstr S, int & l );
-extern void mem_write ( Boolean  B, pstr S, int & l );
+extern void mem_write ( int       I, pstr S, int & l );
+extern void mem_write ( short     I, pstr S, int & l );
+extern void mem_write ( long      I, pstr S, int & l );
+extern void mem_write ( word      W, pstr S, int & l );
+extern void mem_write ( realtype  R, pstr S, int & l );
+extern void mem_write ( shortreal R, pstr S, int & l );
+extern void mem_write ( pstr      L, int len, pstr S, int & l );
+extern void mem_write ( pstr      L, pstr S, int & l );
+extern void mem_write ( Boolean   B, pstr S, int & l );
 
-extern void mem_read  ( int      & I, cpstr S, int & l );
-extern void mem_read  ( short    & I, cpstr S, int & l );
-extern void mem_read  ( long     & I, cpstr S, int & l );
-extern void mem_read  ( word     & W, cpstr S, int & l );
-extern void mem_read  ( realtype & R, cpstr S, int & l );
-extern void mem_read  ( pstr     L, int len, cpstr S, int & l );
-extern void mem_read  ( pstr     & L, cpstr S, int & l );
-extern void mem_read  ( Boolean  & B, cpstr S, int & l );
+extern void mem_read  ( int       & I, cpstr S, int & l );
+extern void mem_read  ( short     & I, cpstr S, int & l );
+extern void mem_read  ( long      & I, cpstr S, int & l );
+extern void mem_read  ( word      & W, cpstr S, int & l );
+extern void mem_read  ( realtype  & R, cpstr S, int & l );
+extern void mem_read  ( shortreal & R, cpstr S, int & l );
+extern void mem_read  ( pstr        L, int len, cpstr S, int & l );
+extern void mem_read  ( pstr      & L, cpstr S, int & l );
+extern void mem_read  ( Boolean   & B, cpstr S, int & l );
 
 #endif
 
