@@ -1,4 +1,3 @@
-//  $Id: mmdb_atom.cpp,v 1.23.2.2 2009/09/28 11:00:25 ccb Exp $
 //  =================================================================
 //
 //   CCP4 Coordinate Library: support of coordinate-related
@@ -6,13 +5,13 @@
 //
 //   Copyright (C) Eugene Krissinel 2000-2008.
 //
-//    This library is free software: you can redistribute it and/or 
-//    modify it under the terms of the GNU Lesser General Public 
-//    License version 3, modified in accordance with the provisions 
+//    This library is free software: you can redistribute it and/or
+//    modify it under the terms of the GNU Lesser General Public
+//    License version 3, modified in accordance with the provisions
 //    of the license to address the requirements of UK law.
 //
-//    You should have received a copy of the modified GNU Lesser 
-//    General Public License along with this library. If not, copies 
+//    You should have received a copy of the modified GNU Lesser
+//    General Public License along with this library. If not, copies
 //    may be downloaded from http://www.ccp4.ac.uk/ccp4license.php
 //
 //    This program is distributed in the hope that it will be useful,
@@ -29,7 +28,7 @@
 //  **** Module  :  MMDB_Atom  <implementation>
 //       ~~~~~~~~~
 //  **** Project :  MacroMolecular Data Base (MMDB)
-//       ~~~~~~~~~ 
+//       ~~~~~~~~~
 //  **** Classes :  CAtom    ( atom class    )
 //       ~~~~~~~~~  CResidue ( residue class )
 //
@@ -145,11 +144,11 @@ void  CAtom::InitAtom()  {
   u11        = 0.0;        //
   u22        = 0.0;        // anisotropic
   u33        = 0.0;        //
-  u12        = 0.0;        //    temperature 
+  u12        = 0.0;        //    temperature
   u13        = 0.0;        //
   u23        = 0.0;        //        factors
   su11       = 0.0;        //
-  su22       = 0.0;        // standard  
+  su22       = 0.0;        // standard
   su33       = 0.0;        //    deviations of
   su12       = 0.0;        //       anisotropic
   su13       = 0.0;        //          temperature
@@ -177,7 +176,7 @@ int CAtom::GetNBonds()  {
 
 void CAtom::GetBonds ( RPSAtomBond AtomBond, int & nAtomBonds )  {
 //    This GetBonds(..) returns pointer to the CAtom's
-//  internal Bond structure, IT MUST NOT BE DISPOSED. 
+//  internal Bond structure, IT MUST NOT BE DISPOSED.
   nAtomBonds = nBonds & 0x000000FF;
   AtomBond   = Bond;
 }
@@ -298,7 +297,7 @@ char N[10];
 //  S[21] = residue->chain->chainID[0];
 //  if (!S[21])  S[21] = ' ';
   if (residue->seqNum>MinInt4)  {
-    if (-999<=residue->seqNum<=9999)
+    if ((-999<=residue->seqNum) && (residue->seqNum<=9999))
       PutIntIns  ( &(S[22]),residue->seqNum,4,residue->insCode );
     else  {
       hy36encode ( 4,residue->seqNum,N );
@@ -406,7 +405,7 @@ Boolean      singleModel = True;
     Loop->AddLoopTag ( CIFTAG_CARTN_Z_ESD        ); // z-sigma
     Loop->AddLoopTag ( CIFTAG_OCCUPANCY_ESD      ); // occupancy-sigma
     Loop->AddLoopTag ( CIFTAG_B_ISO_OR_EQUIV_ESD ); // t-factor-sigma
-    Loop->AddLoopTag ( CIFTAG_PDBX_PDB_MODEL_NUM ); // model number 
+    Loop->AddLoopTag ( CIFTAG_PDBX_PDB_MODEL_NUM ); // model number
   }
 
   if (Ter)  {
@@ -430,7 +429,7 @@ Boolean      singleModel = True;
       Loop->AddNoData ( CIF_NODATA_DOT );
       Loop->AddNoData ( CIF_NODATA_DOT );
     }
-    
+
     for (i=0;i<13;i++)
       Loop->AddNoData ( CIF_NODATA_QUESTION );
 
@@ -501,7 +500,7 @@ Boolean      singleModel = True;
               else  Loop->AddInteger ( model->serNum       );
 
   if (WhatIsSet & ASET_Anis_tFac)  {
-   
+
     RC = CIF->AddLoop ( CIFCAT_ATOM_SITE_ANISOTROP,Loop );
     if (RC!=CIFRC_Ok)  {
       // the category was (re)created, provide tags
@@ -527,7 +526,7 @@ Boolean      singleModel = True;
 
     if (serNum>0)  Loop->AddInteger ( serNum );
              else  Loop->AddInteger ( index  );
-    
+
     Loop->AddReal ( u11 );
     Loop->AddReal ( u22 );
     Loop->AddReal ( u33 );
@@ -572,10 +571,10 @@ int CAtom::ConvertPDBATOM ( int ix, cpstr S )  {
   if (GetReal(tempFactor,&(S[60]),6))  WhatIsSet |= ASET_tempFactor;
 
   if (WhatIsSet & (ASET_CoordSigma | ASET_Anis_tFac |
-                   ASET_Anis_tFSigma))  
+                   ASET_Anis_tFSigma))
     // something was already submitted. check complience
     return CheckData ( S );
-  else  
+  else
     // first data submission. just take the data
     GetData ( S );
 
@@ -626,10 +625,10 @@ int CAtom::ConvertPDBSIGATM ( int ix, cpstr S )  {
   if (GetReal(sigTemp,&(S[60]),6))  WhatIsSet |= ASET_tFacSigma;
 
   if (WhatIsSet & (ASET_Coordinates | ASET_Anis_tFac |
-                   ASET_Anis_tFSigma))  
+                   ASET_Anis_tFSigma))
     // something was already submitted. check complience
     return CheckData ( S );
-  else  
+  else
     // first data submission. just take the data
     GetData ( S );
 
@@ -667,10 +666,10 @@ int CAtom::ConvertPDBANISOU ( int ix, cpstr S )  {
   WhatIsSet |= ASET_Anis_tFac;
 
   if (WhatIsSet & (ASET_Coordinates | ASET_CoordSigma |
-                   ASET_Anis_tFSigma))  
+                   ASET_Anis_tFSigma))
     // something was already submitted. check complience
     return CheckData ( S );
-  else  
+  else
     // first data submission. just take the data
     GetData ( S );
 
@@ -708,10 +707,10 @@ int CAtom::ConvertPDBSIGUIJ ( int ix, cpstr S )  {
   WhatIsSet |= ASET_Anis_tFSigma;
 
   if (WhatIsSet & (ASET_Coordinates | ASET_CoordSigma |
-                   ASET_Anis_tFac))  
+                   ASET_Anis_tFac))
     // something was already submitted. check complience
     return CheckData ( S );
-  else  
+  else
     // first data submission. just take the data
     GetData ( S );
 
@@ -728,7 +727,7 @@ int CAtom::ConvertPDBTER ( int ix, cpstr S )  {
 
   index = ix;
 
-  if (((S[6]>='0') && (S[6]<='9')) || (S[6]==' '))  { 
+  if (((S[6]>='0') && (S[6]<='9')) || (S[6]==' '))  {
     //   Although against strict PDB format, 'TER' is
     // actually allowed not to have a serial number.
     // This negative value implies that the number is
@@ -810,6 +809,12 @@ pstr  CAtom::GetInsCode()  {
           else  return pstr("");
 }
 
+int   CAtom::GetSSEType()  {
+// works only after SSE calculations
+  if (residue)  return residue->SSE;
+          else  return SSE_None;
+}
+
 pstr  CAtom::GetAtomCharge ( pstr chrg )  {
   if (WhatIsSet & ASET_Charge)  sprintf ( chrg,"%+2i",mround(charge) );
                           else  strcpy  ( chrg,"  " );
@@ -835,7 +840,7 @@ PCModel  CAtom::GetModel()  {
 int  CAtom::GetResidueNo()  {
   if (residue)  {
     if (residue->chain)
-         return  residue->chain->GetResidueNo ( 
+         return  residue->chain->GetResidueNo (
                           residue->seqNum,residue->insCode );
     else  return -2;
   } else
@@ -858,6 +863,132 @@ void  CAtom::GetStat ( realtype   v,
   v_m2 += v*v;
 }
 
+
+
+void  CAtom::GetChainCalphas ( PPCAtom & Calphas, int & nCalphas,
+                               cpstr altLoc )  {
+//   GetChainCalphas(...) is a specialized function for quick
+// access to C-alphas of chain which includes given atom.
+// This function works faster than an equivalent implementation
+// through MMDB's selection procedures.
+//    Parameters:
+//       Calphas   - array to accept pointers on C-alpha atoms
+//                  If Calphas!=NULL, then the function will
+//                  delete and re-allocate it. When the array
+//                  is no longer needed, the application MUST
+//                  delete it:  delete[] Calphas; Deleting
+//                  Calphas does not delete atoms from MMDB.
+//       nCalphas   - integer to accept number of C-alpha atoms
+//                  and the length of Calphas array.
+//       altLoc     - alternative location indicator. By default
+//                  (""), maximum-occupancy locations are taken.
+PCChain    chain;
+PPCResidue res;
+PPCAtom    atom;
+int        nResidues, nAtoms, i,j,k;
+
+  if (Calphas)  {
+    delete[] Calphas;
+    Calphas = NULL;
+  }
+  nCalphas = 0;
+
+  if (residue)  chain = residue->chain;
+          else  chain = NULL;
+
+  if (chain)  {
+
+    chain->GetResidueTable ( res,nResidues );
+
+    if (nResidues>0)  {
+
+      Calphas = new PCAtom[nResidues];
+
+      if ((!altLoc[0]) || (altLoc[0]==' '))  {  // main conformation
+
+        for (i=0;i<nResidues;i++)  {
+          nAtoms = res[i]->nAtoms;
+          atom   = res[i]->atom;
+          for (j=0;j<nAtoms;j++)
+            if (!strcmp(atom[j]->name," CA "))  {
+              Calphas[nCalphas++] = atom[j];
+              break;
+            }
+        }
+
+      } else  {  // specific conformation
+
+        for (i=0;i<nResidues;i++)  {
+          nAtoms = res[i]->nAtoms;
+          atom   = res[i]->atom;
+          k      = 0;
+          for (j=0;j<nAtoms;j++)
+            if (!strcmp(atom[j]->name," CA "))  {
+              k = 1;
+              if (!atom[j]->altLoc[0])  {
+                // take main conformation now as we do not know if
+                // the specific one is in the file
+                Calphas[nCalphas] = atom[j];
+                k = 2;
+              } else if (atom[j]->altLoc[0]==altLoc[0])  {
+                // get specific conformation and quit
+                Calphas[nCalphas] = atom[j];
+                k = 2;
+                break;
+              }
+            } else if (k)
+              break;
+          if (k==2)  nCalphas++;
+        }
+
+      }
+
+    }
+
+  } else if (residue)  {  // check just this atom's residue
+
+    Calphas = new PCAtom[1]; // can't be more than 1 C-alpha!
+
+    nAtoms = residue->nAtoms;
+    atom   = residue->atom;
+
+    if ((!altLoc[0]) || (altLoc[0]==' '))  {  // main conformation
+
+      for (j=0;j<nAtoms;j++)
+        if (!strcmp(atom[j]->name," CA "))  {
+          Calphas[nCalphas++] = atom[j];
+          break;
+        }
+
+    } else  {  // specific conformation
+
+      k = 0;
+      for (j=0;j<nAtoms;j++)
+        if (!strcmp(atom[j]->name," CA "))  {
+          k = 1;
+          if (!atom[j]->altLoc[0])  {
+            Calphas[nCalphas] = atom[j];
+            k = 2;
+          } else if (atom[j]->altLoc[0]==altLoc[0])  {
+            Calphas[nCalphas] = atom[j];
+            k = 2;
+            break;
+          }
+        } else if (k)
+          break;
+      if (k==2)  nCalphas++;
+
+    }
+
+  }
+
+  if (Calphas && (!nCalphas))  {
+    delete[] Calphas;
+    Calphas = NULL;
+  }
+
+}
+
 Boolean CAtom::isMetal()  {
   return  ::isMetal ( element );
 }
@@ -981,7 +1112,7 @@ realtype dx1,dy1,dz1, dx2,dy2,dz2,r;
 void  CAtom::MakeTer()  {
   WhatIsSet |= ASET_Coordinates;
   Het        = False;
-  Ter        = True; 
+  Ter        = True;
 }
 
 
@@ -1088,7 +1219,7 @@ char S[50];
   strcat  ( AtomID,S );
   return AtomID;
 }
-  
+
 
 
 int CAtom::ConvertPDBHETATM ( int ix, cpstr S )  {
@@ -1106,11 +1237,11 @@ int RC;
 void CAtom::GetData ( cpstr S )  {
 pstr p;
 
-  if (((S[6]>='0') && (S[6]<='9')) || (S[6]==' '))  { 
+  if (((S[6]>='0') && (S[6]<='9')) || (S[6]==' '))  {
     //   Here we forgive cards with unreadable serial numbers
     // as we always have index (ix) for the card. For the sake
     // of strict PDB syntax we would have to return
-    // Error_UnrecognizedInteger .
+    // Error_UnrecognizedInteger here.
     if (!(GetInteger(serNum,&(S[6]),5)))  serNum = -1;
   } else
     hy36decode ( 5,&(S[6]),5,&serNum );
@@ -1127,6 +1258,8 @@ pstr p;
   if ((charge!=0.0) && (p!=&(S[78])))
     WhatIsSet |= ASET_Charge;
 
+  RestoreElementName();
+
 }
 
 int  CAtom::CheckData ( cpstr S )  {
@@ -1160,7 +1293,7 @@ realtype achrg;
   // as we always have index (ix) for the card. For the sake
   // of strict PDB syntax we would have to return
   // Error_UnrecognizedInteger .
-  if ((sN!=serNum)                  || 
+  if ((sN!=serNum)                  ||
       (strcmp (altLoc ,aloc      )) ||
       (strncmp(name   ,&(S[12]),4)) ||
       (strcmp (segID  ,sID       )) ||
@@ -1170,11 +1303,11 @@ realtype achrg;
 char name1[100];
 strncpy ( name1,&(S[12]),4 );  name1[4] = char(0);
     printf ( "\n  serNum   %5i  %5i\n"
-             "  residue  '%s' '%s'\n" 
-             "  altLoc   '%s' '%s'\n" 
-             "  name     '%s' '%s'\n" 
-             "  segId    '%s' '%s'\n" 
-             "  element  '%s' '%s'\n" 
+             "  residue  '%s' '%s'\n"
+             "  altLoc   '%s' '%s'\n"
+             "  name     '%s' '%s'\n"
+             "  segId    '%s' '%s'\n"
+             "  element  '%s' '%s'\n"
              "  charge   '%s' '%s'\n",
              sN,serNum, res->name,residue->name,
              altLoc ,aloc,  name,name1,
@@ -1211,11 +1344,11 @@ int  RC;
 
   RC = CIFGetInteger1 ( serNum,Loop,CIFTAG_ID,k );
   if (RC)  {
-    if (Ter) 
+    if (Ter)
       serNum = -1;
     else if (RC==Error_NoData)
       serNum = index;
-    else 
+    else
       return RC;
   }
 
@@ -1250,6 +1383,8 @@ int  RC;
   if (RC)
     CIFGetString ( element,Loop,CIFTAG_ATOM_TYPE_SYMBOL,k,
                         sizeof(element),pstr("  ") );
+
+  RestoreElementName();
   //  MakePDBAtomName();
 
   RC = CIFGetReal1 ( sigX,Loop,CIFTAG_CARTN_X_ESD,k );
@@ -1263,7 +1398,7 @@ int  RC;
     WhatIsSet |= ASET_OccSigma;
   if (!CIFGetReal1(sigTemp,Loop,CIFTAG_B_ISO_OR_EQUIV_ESD,k))
     WhatIsSet |= ASET_tFacSigma;
-  
+
   Loop->DeleteRow ( k );
 
   if (LoopAnis)  {
@@ -1296,6 +1431,28 @@ int  RC;
 
 }
 
+Boolean CAtom::RestoreElementName()  {
+//  This function works only if element name is not given.
+  if (Ter)  {
+    name[0]    = char(0);
+    element[0] = char(0);
+    return False;
+  }
+  if ((!element[0]) ||
+      ((element[0]==' ') && ((!element[1]) || (element[1]==' '))))  {
+    if ((name[0]>='A') && (name[0]<='Z'))  {
+      element[0] = name[0];
+      element[1] = name[1];
+    } else  {
+      element[0] = ' ';
+      element[1] = name[1];
+    }
+    element[2] = char(0);
+    return False;
+  }
+  return True;
+}
+
 Boolean CAtom::MakePDBAtomName()  {
 int     i,k;
   if (Ter)  {
@@ -1607,7 +1764,7 @@ byte nb;
       f.WriteFloat ( &sigOcc );
     if ((WhatIsSet & ASET_tempFactor) &&
         (WhatIsSet & ASET_tFacSigma))
-      f.WriteFloat ( &sigTemp ); 
+      f.WriteFloat ( &sigTemp );
   }
 
   if (WhatIsSet & ASET_Anis_tFac)  {
@@ -1710,7 +1867,7 @@ byte nb,Version;
     else  sigOcc = 0.0;
     if ((WhatIsSet & ASET_tempFactor) &&
         (WhatIsSet & ASET_tFacSigma))
-          f.ReadFloat ( &sigTemp ); 
+          f.ReadFloat ( &sigTemp );
     else  sigTemp = 0.0;
   } else  {
     sigX    = 0.0;
@@ -1947,7 +2104,7 @@ void * CResidue::GetCoordHierarchy()  {
   return NULL;
 }
 
-void  CResidue::GetAltLocations ( int     & nAltLocs, 
+void  CResidue::GetAltLocations ( int     & nAltLocs,
                                   PAltLoc & aLoc,
                                   rvector & occupancy,
                                   int     & alflag )  {
@@ -1995,7 +2152,7 @@ bvector  alv;
             //   a) the data was not found wrong so far
             //   b) this atom name has not been checked before
             //   c) altcode is not the "empty"-altcode
-            if ((!(alflag & ALF_Mess)) && (!alv[i]) && 
+            if ((!(alflag & ALF_Mess)) && (!alv[i]) &&
                 (atom[i]->altLoc[0]))  {
               B    = False; // will be set True if "empty"-altcode
                             // is found for current atom name
@@ -2005,7 +2162,7 @@ bvector  alv;
                             // current atom name
               for (j=0;j<nAtoms;j++)
                 if (atom[j])  {
-                  if ((!atom[j]->Ter) && 
+                  if ((!atom[j]->Ter) &&
                       (!strcmp(atom[j]->name,atom[i]->name)))  {
                     if (atom[j]->WhatIsSet & ASET_Occupancy)
                       occ1 += atom[j]->occupancy;
@@ -2382,7 +2539,7 @@ int i;
       } else
         atom[i] = NULL;
   }
-  
+
 }
 
 
@@ -3035,7 +3192,7 @@ realtype absq,acsq,bcsq;
 
 }
 
-    
+
 void  CResidue::write ( RCFile f )  {
 int  i;
 byte Version=1;
