@@ -1,4 +1,3 @@
-//  $Id: mmdb_atom.h,v 1.22.2.2 2009/09/28 11:00:26 ccb Exp $
 //  =================================================================
 //
 //   CCP4 Coordinate Library: support of coordinate-related
@@ -6,13 +5,13 @@
 //
 //   Copyright (C) Eugene Krissinel 2000-2008.
 //
-//    This library is free software: you can redistribute it and/or 
-//    modify it under the terms of the GNU Lesser General Public 
-//    License version 3, modified in accordance with the provisions 
+//    This library is free software: you can redistribute it and/or
+//    modify it under the terms of the GNU Lesser General Public
+//    License version 3, modified in accordance with the provisions
 //    of the license to address the requirements of UK law.
 //
-//    You should have received a copy of the modified GNU Lesser 
-//    General Public License along with this library. If not, copies 
+//    You should have received a copy of the modified GNU Lesser
+//    General Public License along with this library. If not, copies
 //    may be downloaded from http://www.ccp4.ac.uk/ccp4license.php
 //
 //    This program is distributed in the hope that it will be useful,
@@ -22,7 +21,7 @@
 //
 //  =================================================================
 //
-//    04.02.09   <--  Date of Last Modification.
+//    15.07.10   <--  Date of Last Modification.
 //                   ~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 //  -----------------------------------------------------------------
 //
@@ -35,7 +34,7 @@
 //  **** Functions :  BondAngle
 //       ~~~~~~~~~~~
 //
-//  Copyright (C) E. Krissinel 2000-2009
+//  Copyright (C) E. Krissinel 2000-2010
 //
 //  =================================================================
 //
@@ -125,7 +124,7 @@ struct SAtomStat  {
 
     realtype GetMaxSize();
 
-  private : 
+  private :
     Boolean finished;
 
 };
@@ -211,7 +210,7 @@ class CAtom : public CUDData  {
     int   GetNBonds();
 
     //    This GetBonds(..) returns pointer to the CAtom's
-    //  internal Bond structure, IT MUST NOT BE DISPOSED. 
+    //  internal Bond structure, IT MUST NOT BE DISPOSED.
     void  GetBonds ( RPSAtomBond AtomBond, int & nAtomBonds );
     void  FreeBonds();
 
@@ -245,6 +244,8 @@ class CAtom : public CUDData  {
 
     int GetCIF           ( int ix, PCMMCIFLoop Loop,
                            PCMMCIFLoop LoopAnis );
+
+    Boolean RestoreElementName();
     Boolean MakePDBAtomName();
 
     void  SetAtomName    ( int            ix,      // index
@@ -276,10 +277,29 @@ class CAtom : public CUDData  {
     realtype GetOccupancy   ();
     int   GetSeqNum  ();
     pstr  GetInsCode ();
+    int   GetSSEType ();  // works only after SSE calculations
     pstr  GetAtomName   () { return name;    }
     pstr  GetElementName() { return element; }
     pstr  GetAtomCharge ( pstr chrg );
 
+    //   GetChainCalphas(...) is a specialized function for quick
+    // access to C-alphas of chain which includes given atom.
+    // This function works faster than an equivalent implementation
+    // through MMDB's selection procedures.
+    //    Parameters:
+    //       Calphas   - array to accept pointers on C-alpha atoms
+    //                  If Calphas!=NULL, then the function will
+    //                  delete and re-allocate it. When the array
+    //                  is no longer needed, the application MUST
+    //                  delete it:  delete[] Calphas; Deleting
+    //                  Calphas does not delete atoms from MMDB.
+    //       nCalphas   - integer to accept number of C-alpha atoms
+    //                  and the length of Calphas array.
+    //       altLoc     - alternative location indicator. By default
+    //                  (""), maximum-occupancy locations are taken.
+    void  GetChainCalphas ( PPCAtom & Calphas, int & nCalphas,
+                            cpstr altLoc = "" );
+
     Boolean isTer         () { return Ter; }
     Boolean isMetal       ();
     Boolean isSolvent     ();  // works only for atom in a residue!
@@ -288,9 +308,10 @@ class CAtom : public CUDData  {
     Boolean isCTerminus   ();
 
     void  CalcAtomStatistics ( RSAtomStat AS );
+
     realtype GetDist2 ( PCAtom a );
     realtype GetDist2 ( PCAtom a, mat44 & tm );  // tm applies to A
-    realtype GetDist2 ( PCAtom a, mat33 & r, vect3 & t );  // tm applies to A
+    realtype GetDist2 ( PCAtom a, mat33 & r, vect3 & t );// tm applies to A
     realtype GetDist2 ( realtype ax, realtype ay, realtype az );
 
     // GetCosine(a1,a2) calculates cosine of angle a1-this-a2,
@@ -352,7 +373,7 @@ class CAtom : public CUDData  {
     // the only parameter, the atom ID, which is of the form:
     //    {name} {[element]} {:altcode}
     // Here {} means that the item may be omitted. Any item may be
-    // represented by a wildcard '*', which means 'any value'. Just 
+    // represented by a wildcard '*', which means 'any value'. Just
     // absence of an item means 'empty', which makes sense only for
     // alternate location code. Missing name or element therefore
     // mean 'any name' or 'any element', correspondingly (same as a
@@ -457,7 +478,7 @@ class CResidue : public CUDData  {
     int      nAtoms;   // number of atoms in the residue
     PPCAtom  atom;     // array of atoms
     byte     SSE;      // SSE type
-    
+
     CResidue ();
     CResidue ( PCChain Chain_Owner );
     CResidue ( PCChain Chain_Owner, const ResName resName,
@@ -683,7 +704,7 @@ class CResidue : public CUDData  {
 
     int      AtmLen;   // length of atom array
     Boolean  Exclude;  // used internally
-    
+
     void  InitResidue  ();
     void  FreeMemory   ();
     int   _AddAtom     ( PCAtom atm );
