@@ -1,4 +1,3 @@
-//  $Id: mmdb_chain.cpp,v 1.18.2.1 2009/08/21 13:35:09 rmk65 Exp $
 //  =================================================================
 //
 //   CCP4 Coordinate Library: support of coordinate-related
@@ -6,13 +5,13 @@
 //
 //   Copyright (C) Eugene Krissinel 2000-2008.
 //
-//    This library is free software: you can redistribute it and/or 
-//    modify it under the terms of the GNU Lesser General Public 
-//    License version 3, modified in accordance with the provisions 
+//    This library is free software: you can redistribute it and/or
+//    modify it under the terms of the GNU Lesser General Public
+//    License version 3, modified in accordance with the provisions
 //    of the license to address the requirements of UK law.
 //
-//    You should have received a copy of the modified GNU Lesser 
-//    General Public License along with this library. If not, copies 
+//    You should have received a copy of the modified GNU Lesser
+//    General Public License along with this library. If not, copies
 //    may be downloaded from http://www.ccp4.ac.uk/ccp4license.php
 //
 //    This program is distributed in the hope that it will be useful,
@@ -22,14 +21,14 @@
 //
 //  =================================================================
 //
-//    08.07.08   <--  Date of Last Modification.
+//    29.01.10   <--  Date of Last Modification.
 //                   ~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 //  -----------------------------------------------------------------
 //
 //  **** Module  :  MMDB_Chain  <implementation>
 //       ~~~~~~~~~
 //  **** Project :  MacroMolecular Data Base (MMDB)
-//       ~~~~~~~~~ 
+//       ~~~~~~~~~
 //  **** Classes :  CChainContainer ( container for in-chain classes  )
 //       ~~~~~~~~~  CContainerChain ( chain containered class template)
 //                  CDBReference    ( DBREF  records                  )
@@ -39,7 +38,7 @@
 //                  CHetRec         ( HET    records                  )
 //                  CChain          ( MMDB chain class                )
 //
-//  Copyright (C) E. Krissinel 2000-2008
+//  Copyright (C) E. Krissinel 2000-2010
 //
 //  =================================================================
 //
@@ -68,6 +67,9 @@
 #include "mmdb_cifdefs.h"
 #endif
 
+//  ==================  CProModel  ======================
+
+MakeStreamFunctions(CProModel)
 
 //  ==============  CChainContainer  ====================
 
@@ -91,7 +93,7 @@ int i;
       (void)PCContainerChain(Container[i])->SetChain ( Chain );
 }
 
-pstr CChainContainer::Get1stChainID()  {
+cpstr CChainContainer::Get1stChainID()  {
 int i;
   i = 0;
   if (Container)  {
@@ -103,7 +105,7 @@ int i;
     return NULL;
 }
 
-void CChainContainer::MoveByChainID ( ChainID chainID,
+void CChainContainer::MoveByChainID ( const ChainID chainID,
                                       PCChainContainer ChainContainer )  {
 int i;
   for (i=0;i<length;i++)
@@ -182,6 +184,7 @@ void  CDBReference::InitDBReference()  {
 }
 
 void  CDBReference::PDBASCIIDump ( pstr S, int N )  {
+UNUSED_ARGUMENT(N);
 //  makes the ASCII PDB DBREF line number N
 //  from the class' data
   strcpy ( S,"DBREF" );
@@ -198,6 +201,7 @@ void  CDBReference::PDBASCIIDump ( pstr S, int N )  {
 }
 
 void  CDBReference::MakeCIF ( PCMMCIFData CIF, int N )  {
+UNUSED_ARGUMENT(N);
 PCMMCIFLoop Loop1,Loop2;
 int         RC1,RC2;
 
@@ -342,8 +346,8 @@ IDCode idCode;
   }
   GetIntIns  ( seqBeg,insBeg,&(S[14]),4  );
   GetIntIns  ( seqEnd,insEnd,&(S[20]),4  );
-  strcpy_ncs ( database     ,&(S[26]),6  );  
-  strcpy_ncs ( dbAccession  ,&(S[33]),8  ); 
+  strcpy_ncs ( database     ,&(S[26]),6  );
+  strcpy_ncs ( dbAccession  ,&(S[33]),8  );
   strcpy_ncs ( dbIdCode     ,&(S[42]),12 );
   GetIntIns  ( dbseqBeg,dbinsBeg,&(S[55]),5 );
   GetIntIns  ( dbseqEnd,dbinsEnd,&(S[62]),5 );
@@ -351,9 +355,9 @@ IDCode idCode;
 }
 
 void  CDBReference::Copy ( PCContainerClass DBRef )  {
-  
+
   CContainerChain::Copy ( DBRef );
-  
+
   seqBeg   = PCDBReference(DBRef)->seqBeg;
   seqEnd   = PCDBReference(DBRef)->seqEnd;
   dbseqBeg = PCDBReference(DBRef)->dbseqBeg;
@@ -366,8 +370,8 @@ void  CDBReference::Copy ( PCContainerClass DBRef )  {
   strcpy ( dbinsBeg   ,PCDBReference(DBRef)->dbinsBeg    );
   strcpy ( dbinsEnd   ,PCDBReference(DBRef)->dbinsEnd    );
 
-}  
-    
+}
+
 void  CDBReference::write ( RCFile f )  {
 byte Version=1;
   f.WriteByte ( &Version  );
@@ -442,6 +446,7 @@ void  CSeqAdv::InitSeqAdv()  {
 }
 
 void  CSeqAdv::PDBASCIIDump ( pstr S, int N )  {
+UNUSED_ARGUMENT(N);
 //  makes the ASCII PDB SEQADV line number N
 //  from the class' data
   strcpy     ( S,"SEQADV" );
@@ -475,10 +480,10 @@ IDCode idCode;
     GetString ( idCode,&(S[7]),4 );
     Chain->SetEntryID ( idCode );
   }
-  strcpy_ncs ( resName       ,&(S[12]),3 );  
+  strcpy_ncs ( resName       ,&(S[12]),3 );
   GetIntIns  ( seqNum,insCode,&(S[18]),4 );
-  strcpy_ncs ( database      ,&(S[24]),4 );  
-  strcpy_ncs ( dbAccession   ,&(S[29]),9 ); 
+  strcpy_ncs ( database      ,&(S[24]),4 );
+  strcpy_ncs ( dbAccession   ,&(S[29]),9 );
   strcpy_ncs ( dbRes         ,&(S[39]),3 );
   GetInteger ( dbSeq,&(S[43]),5  );
   CreateCopy ( conflict,&(S[49]) );
@@ -488,6 +493,7 @@ IDCode idCode;
 
 
 void  CSeqAdv::MakeCIF ( PCMMCIFData CIF, int N )  {
+UNUSED_ARGUMENT(N);
 PCMMCIFLoop Loop;
 int         RC;
 
@@ -592,7 +598,7 @@ void  CSeqAdv::Copy ( PCContainerClass SeqAdv )  {
   CreateCopy ( conflict,PCSeqAdv(SeqAdv)->conflict    );
 
 }
-    
+
 void  CSeqAdv::write ( RCFile f )  {
 byte Version=1;
   f.WriteByte    ( &Version );
@@ -672,8 +678,7 @@ int  i,k,sN;
     i  = 0;
     sN = 1;
     while (i<numRes)  {
-      //PutInteger ( &(S[8]),sN,2 );
-      PutInteger ( &(S[7]),sN,3 );
+      PutInteger ( &(S[8]),sN,2 );
       k = 19;
       while ((i<numRes) && (k<70))  {
         if (resName[i][0])
@@ -815,7 +820,7 @@ Boolean     isMon;
         numRes++;
       }
     }
-  }  
+  }
 
   if (numRes==0)  {
     numRes = -1;
@@ -850,7 +855,7 @@ int i;
   }
 
 }
-    
+
 void  CSeqRes::write ( RCFile f )  {
 int  i;
 byte Version=1;
@@ -920,6 +925,7 @@ void  CModRes::InitModRes()  {
 }
 
 void  CModRes::PDBASCIIDump ( pstr S, int N )  {
+UNUSED_ARGUMENT(N);
 //  makes the ASCII PDB MODRES line number N
 //  from the class' data
   strcpy     ( S,"MODRES" );
@@ -950,7 +956,7 @@ IDCode idCode;
     GetString ( idCode,&(S[7]),4 );
     Chain->SetEntryID ( idCode );
   }
-  GetString  ( resName       ,&(S[12]),3 );  
+  GetString  ( resName       ,&(S[12]),3 );
   GetIntIns  ( seqNum,insCode,&(S[18]),4 );
   GetString  ( stdRes        ,&(S[24]),3 );
   CreateCopy ( comment       ,&(S[29])   );
@@ -959,6 +965,8 @@ IDCode idCode;
 }
 
 void  CModRes::MakeCIF ( PCMMCIFData CIF, int N )  {
+UNUSED_ARGUMENT(CIF);
+UNUSED_ARGUMENT(N);
 /*  -- apparently wrong use of _struct_conn, to be revised
 PCMMCIFLoop Loop;
 int         RC;
@@ -991,6 +999,7 @@ int         RC;
 }
 
 void  CModRes::GetCIF ( PCMMCIFData CIF, int & Signal )  {
+UNUSED_ARGUMENT(CIF);
 //  GetCIF(..) must be always run without reference to Chain,
 //  see CModel::GetCIF(..).
 
@@ -1065,7 +1074,7 @@ void  CModRes::Copy ( PCContainerClass ModRes )  {
   strcpy ( stdRes ,PCModRes(ModRes)->stdRes  );
   CreateCopy ( comment,PCModRes(ModRes)->comment );
 }
-    
+
 void  CModRes::write ( RCFile f )  {
 byte Version=1;
   f.WriteByte    ( &Version );
@@ -1125,6 +1134,7 @@ void  CHetRec::InitHetRec()  {
 }
 
 void  CHetRec::PDBASCIIDump ( pstr S, int N )  {
+UNUSED_ARGUMENT(N);
 //  makes the ASCII PDB MODRES line number N
 //  from the class' data
   strcpy     ( S,"HET" );
@@ -1145,7 +1155,7 @@ int  CHetRec::ConvertPDBASCII ( cpstr S )  {
     Chain->chainID[1] = char(0);
   } else
     Chain->chainID[0] = char(0);
-  GetString  ( hetID         ,&(S[7]) ,3 );  
+  GetString  ( hetID         ,&(S[7]) ,3 );
   GetIntIns  ( seqNum,insCode,&(S[13]),4 );
   GetInteger ( numHetAtoms   ,&(S[20]),5 );
   CreateCopy ( comment       ,&(S[30])   );
@@ -1154,6 +1164,7 @@ int  CHetRec::ConvertPDBASCII ( cpstr S )  {
 }
 
 void  CHetRec::MakeCIF ( PCMMCIFData CIF, int N )  {
+UNUSED_ARGUMENT(N);
 PCMMCIFLoop Loop;
 int         RC;
 
@@ -1236,7 +1247,7 @@ void  CHetRec::Copy ( PCContainerClass Het )  {
   strcpy     ( insCode,PCHetRec(Het)->insCode );
   CreateCopy ( comment,PCHetRec(Het)->comment );
 }
-    
+
 void  CHetRec::write ( RCFile f )  {
 byte Version=1;
   f.WriteByte    ( &Version      );
@@ -1268,7 +1279,7 @@ CChain::CChain() : CUDData() {
   SetChain ( pstr("") );
 }
 
-CChain::CChain ( PCProModel Model, ChainID chID ) : CUDData() {
+CChain::CChain ( PCProModel Model, const ChainID chID ) : CUDData()  {
   InitChain();
   SetChain ( chID );
   if (Model)  Model->AddChain ( this );
@@ -1332,7 +1343,7 @@ void CChain::SetModel ( PCProModel Model )  {
   model = Model;
 }
 
-void * CChain::GetCoordHierarchy()  {
+PCMMDBManager CChain::GetCoordHierarchy()  {
   if (model)  return model->GetCoordHierarchy();
   return NULL;
 }
@@ -1357,7 +1368,7 @@ PCContainerChain ContainerChain;
   DBReference.AddData ( ContainerChain );
   return 0;
 }
-    
+
 int CChain::ConvertSEQADV ( cpstr PDBString ) {
 int              RC;
 PCContainerChain ContainerChain;
@@ -1374,7 +1385,7 @@ PCContainerChain ContainerChain;
 int CChain::ConvertSEQRES ( cpstr PDBString ) {
   return SeqRes.ConvertPDBASCII ( PDBString );
 }
-        
+
 int  CChain::ConvertMODRES ( cpstr PDBString ) {
 int              RC;
 PCContainerChain ContainerChain;
@@ -1387,7 +1398,7 @@ PCContainerChain ContainerChain;
   ModRes.AddData ( ContainerChain );
   return 0;
 }
-        
+
 int  CChain::ConvertHET ( cpstr PDBString ) {
 int              RC;
 PCContainerChain ContainerChain;
@@ -1415,7 +1426,7 @@ void  CChain::PDBASCIIDump ( RCFile f )  {
 void  CChain::PDBASCIIAtomDump ( RCFile f )  {
 int i;
   for (i=0;i<nResidues;i++)
-    if (Residue[i]) 
+    if (Residue[i])
       Residue[i]->PDBASCIIAtomDump ( f );
 }
 
@@ -1470,7 +1481,7 @@ int i;
           else if (!Enforce)
             return NULL;       // duplicate seqNum and insCode!
         }
-      } 
+      }
   }
 
   // expand the residue array, if necessary
@@ -1484,7 +1495,7 @@ int i;
   Residue[nResidues]->index = nResidues;
   nResidues++;
 
-  return Residue[nResidues-1];  
+  return Residue[nResidues-1];
 
 }
 
@@ -1850,7 +1861,7 @@ PCAtom CChain::GetAtom ( int            seqNo,
                          const Element  elmnt,
                          const AltLoc   aloc )  {
 PCResidue res;
-  res = GetResidue ( seqNo,insCode );  
+  res = GetResidue ( seqNo,insCode );
   if (res) return res->GetAtom ( aname,elmnt,aloc );
   return NULL;
 }
@@ -1858,7 +1869,7 @@ PCResidue res;
 PCAtom CChain::GetAtom ( int seqNo, const InsCode insCode,
                          int atomNo )  {
 PCResidue res;
-  res = GetResidue ( seqNo,insCode );  
+  res = GetResidue ( seqNo,insCode );
   if (res)  {
     if ((0<=atomNo) && (atomNo<res->nAtoms))
       return res->atom[atomNo];
@@ -1880,7 +1891,7 @@ PCAtom CChain::GetAtom ( int            resNo,
 PCAtom CChain::GetAtom ( int resNo, int atomNo )  {
 PCResidue res;
   if ((0<=resNo) && (resNo<nResidues))  {
-    res = Residue[resNo];  
+    res = Residue[resNo];
     if (res)  {
       if ((0<=atomNo) && (atomNo<res->nAtoms))
         return res->atom[atomNo];
@@ -1932,7 +1943,7 @@ PCResidue res;
 void CChain::GetAtomTable1 ( int resNo, PPCAtom & atomTable,
                              int & NumberOfAtoms )  {
 PCResidue res;
-  if ((0<=resNo) && (resNo<nResidues)) 
+  if ((0<=resNo) && (resNo<nResidues))
        res = Residue[resNo];
   else res = NULL;
   if (res)
@@ -1950,7 +1961,7 @@ int CChain::DeleteAtom ( int            seqNo,
                          const Element  elmnt,
                          const AltLoc   aloc )  {
 PCResidue res;
-  res = GetResidue ( seqNo,insCode );  
+  res = GetResidue ( seqNo,insCode );
   if (res) return res->DeleteAtom ( aname,elmnt,aloc );
   return 0;
 }
@@ -1958,7 +1969,7 @@ PCResidue res;
 int CChain::DeleteAtom ( int seqNo, const InsCode insCode,
                          int atomNo )  {
 PCResidue res;
-  res = GetResidue ( seqNo,insCode );  
+  res = GetResidue ( seqNo,insCode );
   if (res) return res->DeleteAtom ( atomNo );
   return 0;
 }
@@ -1985,7 +1996,7 @@ int CChain::DeleteAtom ( int resNo, int atomNo )  {
 
 int CChain::DeleteAllAtoms ( int seqNo, const InsCode insCode )  {
 PCResidue res;
-  res = GetResidue ( seqNo,insCode );  
+  res = GetResidue ( seqNo,insCode );
   if (res) return res->DeleteAllAtoms();
   return 0;
 }
@@ -2028,7 +2039,7 @@ int i,n;
 int CChain::AddAtom ( int seqNo, const InsCode insCode,
                       PCAtom atom )  {
 PCResidue res;
-  res = GetResidue ( seqNo,insCode );  
+  res = GetResidue ( seqNo,insCode );
   if (res) return res->AddAtom ( atom );
   return 0;
 }
@@ -2048,28 +2059,38 @@ int i;
 
   FreeMemory();
 
-  strcpy ( chainID    ,Chain->chainID     );
-  strcpy ( prevChainID,Chain->prevChainID );
+  if (Chain)  {
 
-  DBReference.Copy ( &(Chain->DBReference) );
-  SeqAdv     .Copy ( &(Chain->SeqAdv)      );  //  SEQADV records
-  SeqRes     .Copy ( &(Chain->SeqRes)      );  //  SEQRES data
-  ModRes     .Copy ( &(Chain->ModRes)      );  //  MODRES records
-  Het        .Copy ( &(Chain->Het)         );  //  HET    records
+    CopyAnnotations ( Chain );
 
-  nResidues = Chain->nResidues;
-  ResLen    = nResidues;
-  if (nResidues>0)  {
-    Residue = new PCResidue[nResidues];
-    for (i=0;i<nResidues;i++)  {
-      Residue[i] = newCResidue();
-      Residue[i]->SetChain ( this );
-      Residue[i]->Copy ( Chain->Residue[i] );
+    nResidues = Chain->nResidues;
+    ResLen    = nResidues;
+    if (nResidues>0)  {
+      Residue = new PCResidue[nResidues];
+      for (i=0;i<nResidues;i++)  {
+        Residue[i] = newCResidue();
+        Residue[i]->SetChain ( this );
+        Residue[i]->Copy ( Chain->Residue[i] );
+      }
     }
+
   }
 
 }
 
+void  CChain::CopyAnnotations ( PCChain Chain )  {
+  if (Chain)  {
+    strcpy ( chainID    ,Chain->chainID     );
+    strcpy ( prevChainID,Chain->prevChainID );
+    DBReference.Copy ( &(Chain->DBReference) );
+    SeqAdv     .Copy ( &(Chain->SeqAdv)      );  //  SEQADV records
+    SeqRes     .Copy ( &(Chain->SeqRes)      );  //  SEQRES data
+    ModRes     .Copy ( &(Chain->ModRes)      );  //  MODRES records
+    Het        .Copy ( &(Chain->Het)         );  //  HET    records
+  }
+}
+
+
 void  CChain::_copy ( PCChain Chain )  {
 // modify both CChain::_copy and CChain::Copy methods simultaneously!
 int i;
@@ -2157,7 +2178,7 @@ int i;
 }
 */
 
-pstr  CChain::GetEntryID()  {
+cpstr  CChain::GetEntryID()  {
   if (model)  return model->GetEntryID();
         else  return pstr("");
 }
@@ -2171,7 +2192,7 @@ int   CChain::GetModelNum()  {
   return 0;
 }
 
-pstr  CChain::GetChainID ( pstr ChID )  {
+cpstr  CChain::GetChainID ( pstr ChID )  {
   ChID[0] = char(0);
   if (model)
        sprintf ( ChID,"/%i/",model->GetSerNum() );
@@ -2359,7 +2380,7 @@ int  CChain::GetUDData ( int UDDhandle, pstr & sudd )  {
 }
 
 
-
+//  -------------------------------------------------------------------
 
 DefineClass(CSortResidues)
 
@@ -2427,7 +2448,7 @@ byte Version=1;
   f.WriteInt ( &nResidues );
   for (i=0;i<nResidues;i++)
     Residue[i]->write ( f );
-  
+
 }
 
 void  CChain::read ( RCFile f )  {
@@ -2475,7 +2496,7 @@ MakeFactoryFunctions(CChain)
 
   /*
 void  TestChain() {
-//  reads from 'in.chain', writes into 
+//  reads from 'in.chain', writes into
 //  'out.chain' and 'abin.chain'
 CFile    f;
 char     S[81];
@@ -2495,7 +2516,7 @@ PCChain  Chain;
     delete Chain;
     return;
   }
-   
+
   f.assign ( "out.chain",True );
   if (f.rewrite()) {
     Chain->PDBASCIIDump ( f );
@@ -2516,10 +2537,10 @@ PCChain  Chain;
     delete Chain;
     return;
   }
-  
+
   delete Chain;
   printf ( "   Chain deleted.\n" );
-  
+
   Chain = newCChain();
   if (f.reset()) {
     Chain->read ( f );
@@ -2529,12 +2550,12 @@ PCChain  Chain;
     delete Chain;
     return;
   }
-   
+
   f.assign ( "abin.chain",True );
   if (f.rewrite()) {
     Chain->PDBASCIIDump ( f );
     f.shut();
-  } else 
+  } else
     printf ( " Can't open output file 'abin.chain' \n" );
 
   delete Chain;
