@@ -1,4 +1,3 @@
-//  $Id: mmdb_chain.h,v 1.16 2008/08/21 09:54:02 nds65 Exp $
 //  =================================================================
 //
 //   CCP4 Coordinate Library: support of coordinate-related
@@ -6,13 +5,13 @@
 //
 //   Copyright (C) Eugene Krissinel 2000-2008.
 //
-//    This library is free software: you can redistribute it and/or 
-//    modify it under the terms of the GNU Lesser General Public 
-//    License version 3, modified in accordance with the provisions 
+//    This library is free software: you can redistribute it and/or
+//    modify it under the terms of the GNU Lesser General Public
+//    License version 3, modified in accordance with the provisions
 //    of the license to address the requirements of UK law.
 //
-//    You should have received a copy of the modified GNU Lesser 
-//    General Public License along with this library. If not, copies 
+//    You should have received a copy of the modified GNU Lesser
+//    General Public License along with this library. If not, copies
 //    may be downloaded from http://www.ccp4.ac.uk/ccp4license.php
 //
 //    This program is distributed in the hope that it will be useful,
@@ -22,7 +21,7 @@
 //
 //  =================================================================
 //
-//    18.02.04   <--  Date of Last Modification.
+//    29.01.10   <--  Date of Last Modification.
 //                   ~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 //  -----------------------------------------------------------------
 //
@@ -40,7 +39,7 @@
 //                  CHetRec         ( HET    records                  )
 //                  CChain          ( chain class                     )
 //
-//  Copyright (C) E. Krissinel 2000-2008
+//  Copyright (C) E. Krissinel 2000-2010
 //
 //  =================================================================
 //
@@ -75,6 +74,8 @@
 DefineClass(CProModel)
 DefineStreamFunctions(CProModel)
 
+DefineClass(CMMDBManager)
+
 class CProModel : public CUDData  {
 
   friend class CChain;
@@ -82,16 +83,16 @@ class CProModel : public CUDData  {
   public :
 
     CProModel  () : CUDData () {}
-    CProModel  ( RPCStream Object ) : CUDData ( Object ) {} 
+    CProModel  ( RPCStream Object ) : CUDData ( Object ) {}
     ~CProModel () {}
 
-    virtual pstr  GetEntryID () { return pstr(""); }
-    virtual void  SetEntryID ( const IDCode idCode ) {}
+    virtual cpstr GetEntryID () { return ""; }
+    virtual void  SetEntryID ( const IDCode ) {}
 
-    virtual int   AddChain ( PCChain chain ) { return 0; }
+    virtual int   AddChain ( PCChain ) { return 0; }
 
     // returns pointer to CMMDBFile
-    virtual void * GetCoordHierarchy() { return NULL; }
+    virtual PCMMDBManager GetCoordHierarchy() { return NULL; }
 
     //  GetNumberOfModels() returns TOTAL number of models
     virtual int GetNumberOfModels() { return 0;    }
@@ -105,12 +106,12 @@ class CProModel : public CUDData  {
 
     virtual int  GetSerNum       () { return 0; }
 
-    virtual void ExpandAtomArray ( int inc )  {}
-    virtual void AddAtomArray    ( int inc )  {}
+    virtual void ExpandAtomArray ( int )  {}
+    virtual void AddAtomArray    ( int )  {}
 
   protected :
 
-    virtual int  _ExcludeChain ( const ChainID chainID ) { return 0; } 
+    virtual int  _ExcludeChain ( const ChainID ) { return 0; }
 
 };
 
@@ -127,17 +128,17 @@ class CChainContainer : public CClassContainer  {
 
     CChainContainer  () : CClassContainer () {}
     CChainContainer  ( RPCStream Object )
-                        : CClassContainer ( Object ) {} 
+                        : CClassContainer ( Object ) {}
     ~CChainContainer () {}
 
     PCContainerClass MakeContainerClass ( int ClassID );
 
     void  SetChain ( PCChain Chain_Owner ); // must be set before using
                                             // the Container
-    
+
     // special functions used in CModel::GetCIF(..)
-    pstr  Get1stChainID ();
-    void  MoveByChainID ( ChainID chainID,
+    cpstr Get1stChainID ();
+    void  MoveByChainID ( const ChainID chainID,
                           PCChainContainer ChainContainer );
 
   protected :
@@ -190,7 +191,7 @@ class CDBReference : public CContainerChain  {
     InsCode  dbinsBeg;    // ins code of initial residue of the segment
     int      dbseqEnd;    // ending seq number of the database segment
     InsCode  dbinsEnd;   // ins code of the ending residue of the seg-t
-    
+
     CDBReference ();
     CDBReference ( PCChain Chain_Owner );
     CDBReference ( PCChain Chain_Owner, cpstr S );
@@ -209,7 +210,7 @@ class CDBReference : public CContainerChain  {
     void  read  ( RCFile f );
 
   protected :
-    
+
     void InitDBReference();
 
 };
@@ -223,7 +224,7 @@ DefineStreamFunctions(CSeqAdv)
 class CSeqAdv : public CContainerChain  {
 
   public :
- 
+
     ResName  resName;     // residue name in conflict
     int      seqNum;      // residue sequence number
     InsCode  insCode;     // residue insertion code
@@ -232,7 +233,7 @@ class CSeqAdv : public CContainerChain  {
     ResName  dbRes;       // sequence database residue name
     int      dbSeq;       // sequence database sequence number
     pstr     conflict;    // conflict comment
-    
+
     CSeqAdv ();
     CSeqAdv ( PCChain Chain_Owner );
     CSeqAdv ( PCChain Chain_Owner, cpstr S );
@@ -253,7 +254,7 @@ class CSeqAdv : public CContainerChain  {
     void  read  ( RCFile f );
 
   protected :
-    
+
     void InitSeqAdv();
 
 };
@@ -270,10 +271,10 @@ class CSeqRes : public CStream  {
   friend class CChain;
 
   public :
- 
+
     int       numRes;   // number of residues in the chain
     PResName  resName;  // residue names
-    
+
     CSeqRes ();
     CSeqRes ( RPCStream Object );
     ~CSeqRes();
@@ -309,13 +310,13 @@ DefineStreamFunctions(CModRes)
 class CModRes : public CContainerChain  {
 
   public :
- 
+
     ResName  resName;     // residue name used
     int      seqNum;      // residue sequence number
     InsCode  insCode;     // residue insertion code
     ResName  stdRes;      // standard residue name
     pstr     comment;     // description of the residue modification
-    
+
     CModRes ();
     CModRes ( PCChain Chain_Owner );
     CModRes ( PCChain Chain_Owner, cpstr S );
@@ -334,7 +335,7 @@ class CModRes : public CContainerChain  {
     void  read  ( RCFile f );
 
   protected :
-    
+
     void InitModRes();
 
 };
@@ -348,14 +349,14 @@ DefineStreamFunctions(CHetRec)
 class CHetRec : public CContainerChain  {
 
   public :
- 
+
     ResName  hetID;       // Het identifier (right-justified)
     int      seqNum;      // sequence number
     InsCode  insCode;     // insertion code
     int      numHetAtoms; // number of HETATM records for the
                           // group present in the entry
     pstr     comment;     // text describing Het group
-    
+
     CHetRec ();
     CHetRec ( PCChain Chain_Owner );
     CHetRec ( PCChain Chain_Owner, cpstr S );
@@ -374,7 +375,7 @@ class CHetRec : public CContainerChain  {
     void  read  ( RCFile f );
 
   protected :
-    
+
     void InitHetRec();
 
 };
@@ -409,7 +410,7 @@ class CChain : public CUDData  {
     CChainContainer Het;         // non-standard residues descriptions
 
     CChain ();  // SetModel() MUST be used after this constructor!
-    CChain ( PCProModel Model, ChainID chID );
+    CChain ( PCProModel Model, const ChainID chID );
     CChain ( RPCStream Object );
     ~CChain();
 
@@ -418,7 +419,7 @@ class CChain : public CUDData  {
     void SetModel ( PCProModel    Model );
     void SetChain ( const ChainID chID  );
 
-    void * GetCoordHierarchy();   // PCMMDBFile
+    PCMMDBManager GetCoordHierarchy();   // PCMMDBFile
 
     //   ConvertXXXXX(..) functions do not check for record name
     // and assume that PDBString is at least 81 symbols long
@@ -431,7 +432,7 @@ class CChain : public CUDData  {
 
     // This function should be used for testing purposes only.
     // A full PDB ASCII dump for all models and chains involved
-    // is done by CMMDBFile class. 
+    // is done by CMMDBFile class.
     void  PDBASCIIDump     ( RCFile f );
 
     void  PDBASCIIAtomDump ( RCFile      f   );
@@ -583,9 +584,9 @@ class CChain : public CUDData  {
 
     int   GetModelNum();
     PCModel GetModel () { return (PCModel)model; }
-    pstr  GetChainID () { return chainID; }
+    cpstr GetChainID () { return chainID; }
     void  SetChainID ( const ChainID chID );
-    pstr  GetChainID ( pstr  ChID );  // returns /m/c 
+    cpstr GetChainID ( pstr  ChID );  // returns /m/c
 
     void  GetAtomStatistics  ( RSAtomStat AS );
     void  CalcAtomStatistics ( RSAtomStat AS );
@@ -593,7 +594,7 @@ class CChain : public CUDData  {
     int   CheckID    ( const ChainID chID );
     int   CheckIDS   ( cpstr CID  );
 
-    pstr  GetEntryID ();
+    cpstr GetEntryID ();
     void  SetEntryID ( const IDCode idCode );
 
     int   GetNumberOfDBRefs ();
@@ -625,7 +626,8 @@ class CChain : public CUDData  {
     int   GetUDData ( int UDDhandle, pstr sudd, int maxLen );
     int   GetUDData ( int UDDhandle, pstr     & sudd );
 
-    void  Copy ( PCChain Chain );
+    void  Copy            ( PCChain Chain );
+    void  CopyAnnotations ( PCChain Chain );
 
     void  write ( RCFile f );    // writes header to PDB binary file
     void  read  ( RCFile f );    // reads header from PDB binary file
@@ -664,7 +666,7 @@ class CChain : public CUDData  {
 };
 
 
-extern void  TestChain();  //  reads from 'in.chain', writes into 
+extern void  TestChain();  //  reads from 'in.chain', writes into
                            //  'out.chain' and 'abin.chain'
 
 #endif
