@@ -1,4 +1,3 @@
-//  $Id: mmdb_cifdefs.h,v 1.15 2008/08/21 09:54:02 nds65 Exp $
 //  =================================================================
 //
 //   CCP4 Coordinate Library: support of coordinate-related
@@ -6,13 +5,13 @@
 //
 //   Copyright (C) Eugene Krissinel 2000-2008.
 //
-//    This library is free software: you can redistribute it and/or 
-//    modify it under the terms of the GNU Lesser General Public 
-//    License version 3, modified in accordance with the provisions 
+//    This library is free software: you can redistribute it and/or
+//    modify it under the terms of the GNU Lesser General Public
+//    License version 3, modified in accordance with the provisions
 //    of the license to address the requirements of UK law.
 //
-//    You should have received a copy of the modified GNU Lesser 
-//    General Public License along with this library. If not, copies 
+//    You should have received a copy of the modified GNU Lesser
+//    General Public License along with this library. If not, copies
 //    may be downloaded from http://www.ccp4.ac.uk/ccp4license.php
 //
 //    This program is distributed in the hope that it will be useful,
@@ -100,6 +99,7 @@ extern pstr CIFName ( int NameID, int Mode );
 #define CIFCAT_STRUCT_ASYM                 pstr("_struct_asym")
 #define CIFCAT_STRUCT_CONF                 pstr("_struct_conf")
 #define CIFCAT_STRUCT_CONN                 pstr("_struct_conn")
+#define CIFCAT_STRUCT_LINKR                cpstr("_struct_linkr")
 #define CIFCAT_STRUCT_KEYWORDS             pstr("_struct_keywords")
 #define CIFCAT_STRUCT_NCS_OPER             pstr("_struct_ncs_oper")
 #define CIFCAT_STRUCT_REF                  pstr("_struct_ref")
@@ -112,7 +112,7 @@ extern pstr CIFName ( int NameID, int Mode );
 #define CIFCAT_SYMMETRY                    pstr("_symmetry")
 #define CIFCAT_OBSLTE                      pstr("_ndb_database_pdb_obs_spr")
 
-                                                                       
+
 #define CIFTAG_ANGLE_ALPHA                    pstr("angle_alpha")
 #define CIFTAG_ANGLE_BETA                     pstr("angle_beta")
 #define CIFTAG_ANGLE_GAMMA                    pstr("angle_gamma")
@@ -309,6 +309,7 @@ extern pstr CIFName ( int NameID, int Mode );
 #define CIFTAG_CONN_PTNR1_AUTH_ASYM_ID        pstr("ptnr1_auth_asym_id")
 #define CIFTAG_CONN_PTNR1_AUTH_SEQ_ID         pstr("ptnr1_auth_seq_id")
 #define CIFTAG_CONN_PDBX_PTNR1_PDB_INS_CODE   pstr("pdbx_ptnr1_PDB_ins_code")
+#define CIFTAG_CONN_DIST                      pstr("link_dist")
 #define CIFTAG_CONN_PTNR2_AUTH_ATOM_ID        pstr("ptnr2_auth_atom_id")
 #define CIFTAG_CONN_PDBX_PTNR2_AUTH_ALT_ID    pstr("pdbx_ptnr2_auth_alt_id")
 #define CIFTAG_CONN_PTNR2_AUTH_COMP_ID        pstr("ptnr2_auth_comp_id")
@@ -317,6 +318,7 @@ extern pstr CIFName ( int NameID, int Mode );
 #define CIFTAG_CONN_PDBX_PTNR2_PDB_INS_CODE   pstr("pdbx_ptnr2_PDB_ins_code")
 #define CIFTAG_CONN_PTNR1_SYMMETRY            pstr("ptnr1_symmetry")
 #define CIFTAG_CONN_PTNR2_SYMMETRY            pstr("ptnr2_symmetry")
+#define CIFTAG_CONN_NAME                      pstr("link_name")
 
 
 #endif
