@@ -1,4 +1,3 @@
-//  $Id: mmdb_cryst.cpp,v 1.20 2008/08/21 09:54:07 nds65 Exp $
 //  =================================================================
 //
 //   CCP4 Coordinate Library: support of coordinate-related
@@ -6,13 +5,13 @@
 //
 //   Copyright (C) Eugene Krissinel 2000-2008.
 //
-//    This library is free software: you can redistribute it and/or 
-//    modify it under the terms of the GNU Lesser General Public 
-//    License version 3, modified in accordance with the provisions 
+//    This library is free software: you can redistribute it and/or
+//    modify it under the terms of the GNU Lesser General Public
+//    License version 3, modified in accordance with the provisions
 //    of the license to address the requirements of UK law.
 //
-//    You should have received a copy of the modified GNU Lesser 
-//    General Public License along with this library. If not, copies 
+//    You should have received a copy of the modified GNU Lesser
+//    General Public License along with this library. If not, copies
 //    may be downloaded from http://www.ccp4.ac.uk/ccp4license.php
 //
 //    This program is distributed in the hope that it will be useful,
@@ -22,20 +21,20 @@
 //
 //  =================================================================
 //
-//    08.07.08   <--  Date of Last Modification.
+//    29.01.10   <--  Date of Last Modification.
 //                   ~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 //  -----------------------------------------------------------------
 //
 //  **** Module  :  MMDB_Cryst  <implementation>
 //       ~~~~~~~~~
 //  **** Project :  MacroMolecular Data Base (MMDB)
-//       ~~~~~~~~~ 
+//       ~~~~~~~~~
 //  **** Classes :  CCrystContainer ( container for cryst.  data    )
 //       ~~~~~~~~~  CNCSMatrix      ( non-cryst. symm. matrix class )
 //                  CTVect          ( translational vector class    )
 //                  CMMDBCryst      ( MMDB cryst. section class     )
 //
-//  (C) E. Krissinel 2000-2008
+//  (C) E. Krissinel 2000-2010
 //
 //  =================================================================
 //
@@ -111,8 +110,8 @@ CNCSMatrix::~CNCSMatrix() {}
 
 void  CNCSMatrix::Init()  {
 int i,j;
-  serNum = -1;   
-  iGiven = -1;   
+  serNum = -1;
+  iGiven = -1;
   for (i=0;i<3;i++)  {
     for (j=0;j<3;j++)
       m[i][j] = 0.0;
@@ -128,7 +127,7 @@ Boolean  CNCSMatrix::PDBASCIIDump1 ( RCFile f )  {
 char S[100];
 int  i,j;
 
-  if ((WhatIsSet & NCSMSET_All)==NCSMSET_All)  
+  if ((WhatIsSet & NCSMSET_All)==NCSMSET_All)
     for (i=0;i<3;i++)  {
       sprintf   ( S,"MTRIX%1i %3i",i+1,serNum );
       PadSpaces ( S,80 );
@@ -285,7 +284,7 @@ char        Code[100];
     iGiven = 1;
   else
     iGiven = MinInt4;
-   
+
 
   if (CIFGetReal(m[0][0],Loop,CIFTAG_MATRIX11,Signal))  return;
   if (CIFGetReal(m[0][1],Loop,CIFTAG_MATRIX12,Signal))  return;
@@ -338,7 +337,7 @@ int i,j;
   WhatIsSet = PCNCSMatrix(NCSMatrix)->WhatIsSet;
 
 }
-    
+
 void  CNCSMatrix::write ( RCFile f )  {
 int  i,j;
 byte Version=1;
@@ -399,6 +398,7 @@ void CTVect::Init()  {
 }
 
 void CTVect::PDBASCIIDump ( pstr S, int N )  {
+UNUSED_ARGUMENT(N);
 //  makes the ASCII PDB TVECT line number N
   sprintf   ( S,"TVECT  %3i",serNum );
   PadSpaces ( S,80 );
@@ -470,7 +470,7 @@ int i;
     t[i] = PCTVect(TVect)->t[i];
   CreateCopy ( comment,PCTVect(TVect)->comment );
 }
-    
+
 void  CTVect::write ( RCFile f )  {
 int  i;
 byte Version=1;
@@ -554,7 +554,7 @@ int i,j,k;
   alphas = 90.0;
   betas  = 90.0;
   gammas = 90.0;
-  
+
   for (k=0;k<6;k++)
     AC[k] = 0.0;
 
@@ -634,7 +634,7 @@ int  CMMDBCryst::FixSpaceGroup()  {
 //            however fix is not possible.  spaceGroupFix receives
 //            a copy of spaceGroup
 //    -2    - any checks are not possible because cell parameters
-//            are not found, spaceGroupFix receives a copy of 
+//            are not found, spaceGroupFix receives a copy of
 //            spaceGroup
 //
 realtype eps,m1,m2;
@@ -660,7 +660,7 @@ char     c;
     else {
       m1 = (a+b+c)/3.0;
       m2 = (alpha+beta+gamma)/3.0;
-      if ((fabs(a-m1)<=eps) && (fabs(b-m1)<=eps) && 
+      if ((fabs(a-m1)<=eps) && (fabs(b-m1)<=eps) &&
           (fabs(c-m1)<=eps) &&
           (fabs(alpha-m2)<=eps) && (fabs(beta-m2)<=eps) &&
           (fabs(gamma-m2)<=eps))
@@ -831,7 +831,7 @@ PCTVect     tVect;
     return RC;
 
   } else if (!strncmp(PDBString,"TVECT ",6))  {
-    
+
     tVect = new CTVect();
     RC = tVect->ConvertPDBASCII(PDBString);
     if (RC==0)  TVect.AddData ( tVect );
@@ -869,7 +869,7 @@ char S[100];
     f.WriteLine ( S );
   }
 
-  if ((WhatIsSet & CSET_OrigMatrix)==CSET_OrigMatrix)  
+  if ((WhatIsSet & CSET_OrigMatrix)==CSET_OrigMatrix)
     for (i=0;i<3;i++)  {
       sprintf   ( S,"ORIGX%1i",i+1);
       PadSpaces ( S,80 );
@@ -879,7 +879,7 @@ char S[100];
       f.WriteLine ( S );
     }
 
-  if ((WhatIsSet & CSET_ScaleMatrix)==CSET_ScaleMatrix) 
+  if ((WhatIsSet & CSET_ScaleMatrix)==CSET_ScaleMatrix)
     for (i=0;i<3;i++)  {
       sprintf   ( S,"SCALE%1i",i+1);
       PadSpaces ( S,80 );
@@ -922,7 +922,7 @@ int RC;
     if (!RC) WhatIsSet |= CSET_ZValue;
 
   }
- 
+
   Struct = CIF->GetStructure ( CIFCAT_SYMMETRY );
   if (Struct)  {
     CIFGetString ( spaceGroup,Struct,CIFTAG_SPACE_GROUP_NAME_H_M,
@@ -942,7 +942,7 @@ int RC;
       WhatIsSet |= CSET_SpaceGroup;
   }
 
-  if ((a*b*c*alpha*beta*gamma==0.0) || 
+  if ((a*b*c*alpha*beta*gamma==0.0) ||
       ((a==1.0)      && (b==1.0)     && (c==1.0)      &&
        (alpha==90.0) && (beta==90.0) && (gamma==90.0) &&
        (!strcmp(spaceGroup,"P 1"))))  {
@@ -1095,7 +1095,7 @@ int      i,j,k;
   if ((WhatIsSet & CSET_CellParams)==CSET_CellParams)  {
     //   The 'cryst1' card was supplied. Calculate
     // standard orthogonalizations.
-  
+
     CalcOrthMatrices();
     if (NCode<0)  NCode = 0;
 
@@ -1153,8 +1153,8 @@ int      i,j,k;
             }
           }
       }
-    
-      //   Correct inaccuracy of SCALEi input due to FORMAT, 
+
+      //   Correct inaccuracy of SCALEi input due to FORMAT,
       // replace RF,RO with RR[NCode][][] if possible.
 
       if (NCode>=0)  {
@@ -1184,7 +1184,7 @@ int      i,j,k;
     RFU[3][3] = 1.0;
     Mat4Inverse ( RFU,ROU );
 
-  } else  
+  } else
     CellCheck |= CCHK_NoCell;
 
 }
@@ -1211,7 +1211,7 @@ int i,j;
   } else
     printf ( "\n  $WARNING: NO CRYST CARDS READ$\n" );
 
-  if ((WhatIsSet & CSET_ScaleMatrix)!=CSET_ScaleMatrix) 
+  if ((WhatIsSet & CSET_ScaleMatrix)!=CSET_ScaleMatrix)
     printf ( "\n  $WARNING: NO SCALE CARDS READ$\n" );
 
 }
@@ -1241,7 +1241,7 @@ int      i,j,k;
   Vol  = 2.0*a*b*c*V;
 
   //  Precaution measure for erratic use of the library
-  if ((fabs(Alph)<1.0e-6) || (fabs(Bet)<1.0e-6) || 
+  if ((fabs(Alph)<1.0e-6) || (fabs(Bet)<1.0e-6) ||
                              (fabs(Gamm)<1.0e-6))  {
     as     = 0.0;
     bs     = 0.0;
@@ -1259,7 +1259,7 @@ int      i,j,k;
     }
     return;
   }
-  
+
   sinA   = sin(Alph);
   cosA   = cos(Alph);
   sinB   = sin(Bet);
@@ -1463,7 +1463,7 @@ int   i,j,k;
     return k;
   }
 
-  // transformation back to orthogonal coordinates    
+  // transformation back to orthogonal coordinates
   for (i=0;i<3;i++)  {
     for (j=0;j<4;j++)  {
       TMatrix[i][j] = 0.0;
@@ -1639,8 +1639,9 @@ mat33    A,AT,Tmp,TmpMat;
 realtype BB;
 int      i,j,k;
 
-  if (areMatrices())  {
+ if (areMatrices())  { // if orth2frac and frac2orth matrices are defined
 
+    // U is in upper triangular form
     Tmp[0][0] = U[0];
     Tmp[1][1] = U[1];
     Tmp[2][2] = U[2];
@@ -1651,26 +1652,27 @@ int      i,j,k;
     Tmp[1][2] = U[5];
     Tmp[2][1] = U[5];
 
-    for (i=0;i<=3;i++)
-      for (j=0;j<=3;j++)  {
+    // ROU is a 4x4 matrix, but we only use the upper 3x3 submatrix
+    for (i=0;i<3;i++)
+      for (j=0;j<3;j++)  {
         A [j][i] = ROU[j][i];
         AT[i][j] = ROU[j][i];
       }
 
     //  TmpMat = Tmp*AT
-    for (i=0;i<=3;i++)
-      for (j=0;j<=3;j++)  {
+    for (i=0;i<3;i++)
+      for (j=0;j<3;j++)  {
         BB = 0.0;
-        for (k=0;k<=3;k++)
+        for (k=0;k<3;k++)
           BB += Tmp[i][k]*AT[k][j];
         TmpMat[i][j] = BB;
       }
 
     //  Tmp = A*TmpMat
-    for (i=0;i<=3;i++)
-      for (j=0;j<=3;j++)  {
+    for (i=0;i<3;i++)
+      for (j=0;j<3;j++)  {
         BB = 0.0;
-        for (k=0;k<=3;k++)
+        for (k=0;k<3;k++)
           BB += A[i][k]*TmpMat[k][j];
         Tmp[i][j] = BB;
       }
@@ -1708,26 +1710,26 @@ int      i,j,k;
     Tmp[1][2] = U[5];
     Tmp[2][1] = U[5];
 
-    for (i=0;i<=3;i++)
-      for (j=0;j<=3;j++)  {
+    for (i=0;i< 3;i++)
+      for (j=0;j< 3;j++)  {
         A [j][i] = RFU[j][i];
         AT[i][j] = RFU[j][i];
       }
 
     //  TmpMat = Tmp*AT
-    for (i=0;i<=3;i++)
-      for (j=0;j<=3;j++)  {
+    for (i=0;i< 3;i++)
+      for (j=0;j< 3;j++)  {
         BB = 0.0;
-        for (k=0;k<=3;k++)
+        for (k=0;k< 3;k++)
           BB += Tmp[i][k]*AT[k][j];
         TmpMat[i][j] = BB;
       }
 
     //  Tmp = A*TmpMat
-    for (i=0;i<=3;i++)
-      for (j=0;j<=3;j++)  {
+    for (i=0;i< 3;i++)
+      for (j=0;j< 3;j++)  {
         BB = 0.0;
-        for (k=0;k<=3;k++)
+        for (k=0;k< 3;k++)
           BB += A[i][k]*TmpMat[k][j];
         Tmp[i][j] = BB;
       }
@@ -1876,7 +1878,7 @@ int i,j;
     RO[3][3] = 1.0;
 
     Mat4Inverse ( RO,RF );
-  
+
     WhatIsSet |= CSET_Transforms;
 
   } else
@@ -2217,7 +2219,7 @@ MakeStreamFunctions(CMMDBCryst)
 
 
 void  TestCryst() {
-//  reads from 'in.cryst', writes into 
+//  reads from 'in.cryst', writes into
 //  'out.cryst' and 'abin.cryst'
 CFile       f;
 char        S[81];
@@ -2237,7 +2239,7 @@ PCMMDBCryst Cryst;
     delete Cryst;
     return;
   }
-   
+
   f.assign ( pstr("out.cryst"),True );
   if (f.rewrite()) {
     Cryst->PDBASCIIDump ( f );
@@ -2258,10 +2260,10 @@ PCMMDBCryst Cryst;
     delete Cryst;
     return;
   }
-  
+
   delete Cryst;
   printf ( "   Cryst deleted.\n" );
-  
+
   Cryst = new CMMDBCryst();
   if (f.reset()) {
     Cryst->read ( f );
@@ -2271,12 +2273,12 @@ PCMMDBCryst Cryst;
     delete Cryst;
     return;
   }
-   
+
   f.assign ( pstr("abin.cryst"),True );
   if (f.rewrite()) {
     Cryst->PDBASCIIDump ( f );
     f.shut();
-  } else 
+  } else
     printf ( " Can't open output file 'abin.cryst' \n" );
 
   delete Cryst;
