@@ -1,4 +1,3 @@
-//  $Id: mmdb_defs.h,v 1.19.2.2 2009/09/28 11:00:26 ccb Exp $
 //  =================================================================
 //
 //   CCP4 Coordinate Library: support of coordinate-related
@@ -6,13 +5,13 @@
 //
 //   Copyright (C) Eugene Krissinel 2000-2008.
 //
-//    This library is free software: you can redistribute it and/or 
-//    modify it under the terms of the GNU Lesser General Public 
-//    License version 3, modified in accordance with the provisions 
+//    This library is free software: you can redistribute it and/or
+//    modify it under the terms of the GNU Lesser General Public
+//    License version 3, modified in accordance with the provisions
 //    of the license to address the requirements of UK law.
 //
-//    You should have received a copy of the modified GNU Lesser 
-//    General Public License along with this library. If not, copies 
+//    You should have received a copy of the modified GNU Lesser
+//    General Public License along with this library. If not, copies
 //    may be downloaded from http://www.ccp4.ac.uk/ccp4license.php
 //
 //    This program is distributed in the hope that it will be useful,
@@ -22,7 +21,7 @@
 //
 //  =================================================================
 //
-//    10.09.07   <--  Date of Last Modification.
+//    29.01.10   <--  Date of Last Modification.
 //                   ~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 //  -----------------------------------------------------------------
 //
@@ -33,7 +32,7 @@
 //
 //      Definition of types, constants and important classes.
 //
-//  (C) E. Krissinel 2000-2008
+//  (C) E. Krissinel 2000-2010
 //
 //  =================================================================
 //
@@ -45,12 +44,16 @@
 #include "mattype_.h"
 #endif
 
+#define MMDB_MAJOR_VERSION    (1)
+#define MMDB_MINOR_VERSION    (23)
+#define MMDB_MICRO_VERSION    (2)
+
 
 //  =======================  types  =================================
 
 typedef  char         IDCode  [16];   // ID code of the entry
-typedef  IDCode *     PIDCode;        // pointer to ID code 
-typedef  PIDCode &    RPIDCode;       // ref-ce to pointer to ID code 
+typedef  IDCode *     PIDCode;        // pointer to ID code
+typedef  PIDCode &    RPIDCode;       // ref-ce to pointer to ID code
 typedef  char         Date    [12];   // date DD-MMM-YYYY
 typedef  char         RecName [7];    // name of PDB record
 
@@ -69,6 +72,7 @@ typedef  char         HelixID [20];   // helix ID
 typedef  char         StrandID[20];   // strand ID
 typedef  char         SheetID [20];   // sheet ID
 typedef  char         TurnID  [20];   // turn ID
+typedef  char         LinkRID [20];   // Refmac link ID
 
 typedef  char         SymGroup[100];  // group of space symmetry
 typedef  realtype     vect3   [3];    // vector of 3 real numbers
@@ -126,7 +130,7 @@ typedef  char     maxMMDBName[MaxMMDBNameLength];
 
 //  Error_SEQRES_serNum is returned by CSeqRes::ConvertPDBASCII() if
 //  serial numbers of SEQRES records do not increment by 1
-#define  Error_SEQRES_serNum         4 
+#define  Error_SEQRES_serNum         4
 
 //  Error_SEQRES_numRes is returned by CSeqRes::ConvertPDBASCII() if
 //  SEQRES records show different number of residues
@@ -234,7 +238,8 @@ typedef  char     maxMMDBName[MaxMMDBNameLength];
 #define  ClassID_Helix             19
 #define  ClassID_Turn              20
 #define  ClassID_Link              21
-#define  ClassID_CisPep            22
+#define  ClassID_LinkR             22
+#define  ClassID_CisPep            23
 
 
 //  =====================  classes  ===============================
