@@ -1,4 +1,3 @@
-//  $Id: mmdb_file.cpp,v 1.25.2.1 2009/09/28 11:00:26 ccb Exp $
 //  =================================================================
 //
 //   CCP4 Coordinate Library: support of coordinate-related
@@ -6,13 +5,13 @@
 //
 //   Copyright (C) Eugene Krissinel 2000-2008.
 //
-//    This library is free software: you can redistribute it and/or 
-//    modify it under the terms of the GNU Lesser General Public 
-//    License version 3, modified in accordance with the provisions 
+//    This library is free software: you can redistribute it and/or
+//    modify it under the terms of the GNU Lesser General Public
+//    License version 3, modified in accordance with the provisions
 //    of the license to address the requirements of UK law.
 //
-//    You should have received a copy of the modified GNU Lesser 
-//    General Public License along with this library. If not, copies 
+//    You should have received a copy of the modified GNU Lesser
+//    General Public License along with this library. If not, copies
 //    may be downloaded from http://www.ccp4.ac.uk/ccp4license.php
 //
 //    This program is distributed in the hope that it will be useful,
@@ -22,7 +21,7 @@
 //
 //  =================================================================
 //
-//    24.03.09   <--  Date of Last Modification.
+//    29.01.10   <--  Date of Last Modification.
 //                   ~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 //  -----------------------------------------------------------------
 //
@@ -33,7 +32,7 @@
 //  **** Classes :  CMMDBFile  ( macromolecular data file class )
 //       ~~~~~~~~~
 //
-//  (C) E. Krissinel 2000-2009
+//  (C) E. Krissinel 2000-2010
 //
 //  =================================================================
 //
@@ -100,9 +99,10 @@ void  CMMDBFile::InitMMDBFile()  {
   Flags   = 0x00000000;           // no special effects
   FType   = MMDB_FILE_Undefined;  // undefined file operation
   Exclude = True;
-  ignoreRemarks = False;  // used temporarily
-  allowDuplChID = False;  // used temporarily
-  modelCnt      = 0;      // used only at reading files
+  ignoreRemarks     = False;  // used temporarily
+  allowDuplChID     = False;  // used temporarily
+  enforceUniqueChID = False;  // used temporarily
+  modelCnt          = 0;      // used only at reading files
 }
 
 
@@ -171,22 +171,24 @@ void CMMDBFile::ResetManager() {
 
 void CMMDBFile::SetFlag ( word Flag )  {
   Flags |= Flag;
-  ignoreSegID   = (Flags & MMDBF_IgnoreSegID  )!=0;
-  ignoreElement = (Flags & MMDBF_IgnoreElement)!=0;
-  ignoreCharge  = (Flags & MMDBF_IgnoreCharge )!=0;
-  ignoreNonCoorPDBErrors = (Flags & MMDBF_IgnoreNonCoorPDBErrors)!=0;
-  ignoreUnmatch = (Flags & MMDBF_IgnoreUnmatch)!=0;
-  allowDuplChID = (Flags & MMDBF_AllowDuplChainID )!=0;
-} 
+  ignoreSegID            = (Flags & MMDBF_IgnoreSegID           ) != 0;
+  ignoreElement          = (Flags & MMDBF_IgnoreElement         ) != 0;
+  ignoreCharge           = (Flags & MMDBF_IgnoreCharge          ) != 0;
+  ignoreNonCoorPDBErrors = (Flags & MMDBF_IgnoreNonCoorPDBErrors) != 0;
+  ignoreUnmatch          = (Flags & MMDBF_IgnoreUnmatch         ) != 0;
+  allowDuplChID          = (Flags & MMDBF_AllowDuplChainID      ) != 0;
+  enforceUniqueChID      = (Flags & MMDBF_EnforceUniqueChainID  ) != 0;
+}
 
 void CMMDBFile::RemoveFlag ( word Flag )  {
   Flags &= ~Flag;
-  ignoreSegID   = (Flags & MMDBF_IgnoreSegID  )!=0;
-  ignoreElement = (Flags & MMDBF_IgnoreElement)!=0;
-  ignoreCharge  = (Flags & MMDBF_IgnoreCharge )!=0;
-  ignoreNonCoorPDBErrors = (Flags & MMDBF_IgnoreNonCoorPDBErrors)!=0;
-  ignoreUnmatch = (Flags & MMDBF_IgnoreUnmatch)!=0;
-  allowDuplChID = (Flags & MMDBF_AllowDuplChainID )!=0;
+  ignoreSegID            = (Flags & MMDBF_IgnoreSegID           ) != 0;
+  ignoreElement          = (Flags & MMDBF_IgnoreElement         ) != 0;
+  ignoreCharge           = (Flags & MMDBF_IgnoreCharge          ) != 0;
+  ignoreNonCoorPDBErrors = (Flags & MMDBF_IgnoreNonCoorPDBErrors) != 0;
+  ignoreUnmatch          = (Flags & MMDBF_IgnoreUnmatch         ) != 0;
+  allowDuplChID          = (Flags & MMDBF_AllowDuplChainID      ) != 0;
+  enforceUniqueChID      = (Flags & MMDBF_EnforceUniqueChainID  ) != 0;
 }
 
 
@@ -258,14 +260,15 @@ Boolean      fixSpaceGroup,fend;
   FreeFileMemory();
 
   FType = MMDB_FILE_PDB;
-  ignoreSegID   = (Flags & MMDBF_IgnoreSegID  )!=0;
-  ignoreElement = (Flags & MMDBF_IgnoreElement)!=0;
-  ignoreCharge  = (Flags & MMDBF_IgnoreCharge )!=0;
-  ignoreNonCoorPDBErrors = (Flags & MMDBF_IgnoreNonCoorPDBErrors)!=0;
-  ignoreUnmatch = (Flags & MMDBF_IgnoreUnmatch)!=0;
-  ignoreRemarks = False;
-  allowDuplChID = (Flags & MMDBF_AllowDuplChainID)!=0;
-  fixSpaceGroup = (Flags & MMDBF_FixSpaceGroup   )!=0;
+  ignoreSegID            = (Flags & MMDBF_IgnoreSegID           ) != 0;
+  ignoreElement          = (Flags & MMDBF_IgnoreElement         ) != 0;
+  ignoreCharge           = (Flags & MMDBF_IgnoreCharge          ) != 0;
+  ignoreNonCoorPDBErrors = (Flags & MMDBF_IgnoreNonCoorPDBErrors) != 0;
+  ignoreUnmatch          = (Flags & MMDBF_IgnoreUnmatch         ) != 0;
+  ignoreRemarks          = False;
+  allowDuplChID          = (Flags & MMDBF_AllowDuplChainID      ) != 0;
+  enforceUniqueChID      = (Flags & MMDBF_EnforceUniqueChainID  ) != 0;
+  fixSpaceGroup          = (Flags & MMDBF_FixSpaceGroup         ) != 0;
 
   if (f.FileEnd())  return Error_EmptyFile;
 
@@ -282,7 +285,7 @@ Boolean      fixSpaceGroup,fend;
     } else  {
       RC = Title.ConvertPDBString(S);
       if ((RC!=Error_WrongSection) && ignoreNonCoorPDBErrors)
-        RC = 0; 
+        RC = 0;
       if (RC)  break;
     }
     fend = f.FileEnd();
@@ -304,9 +307,9 @@ Boolean      fixSpaceGroup,fend;
       ContString = new CContString(S);
       Footnote.AddData ( ContString );
     } else  {
-      RC = crModel->ConvertPDBString(S);
+      RC = crModel->ConvertPDBString ( S );
       if ((RC!=Error_WrongSection) && ignoreNonCoorPDBErrors)
-        RC = 0; 
+        RC = 0;
       if (RC)  break;
     }
     fend = f.FileEnd();
@@ -321,7 +324,7 @@ Boolean      fixSpaceGroup,fend;
 
   // temporary solution: the rest of file is stored
   // in the form of strings
-  while (!f.FileEnd()          && 
+  while (!f.FileEnd()          &&
          strncmp(S,"CRYST" ,5) &&
          strncmp(S,"ORIGX" ,5) &&
          strncmp(S,"SCALE" ,5) &&
@@ -336,12 +339,12 @@ Boolean      fixSpaceGroup,fend;
          strncmp(S,"HETATM",6) &&
          strncmp(S,"ENDMDL",6))  {
     if (!strncmp(S,"LINK  ",6))
-      crModel->ConvertPDBString(S);
+      crModel->ConvertPDBString ( S );
     else if (!strncmp(S,"CISPEP",6)) {
       GetInteger ( modNum,&(S[43]),3 );
       if (modNum<=0)  modNum = 1;
       if (modNum!=1)  SwitchModel ( modNum );
-      crModel->ConvertPDBString(S);
+      crModel->ConvertPDBString ( S );
       if (modNum!=1)  SwitchModel ( 1 );
     } else  {
       ContString = new CContString(S);
@@ -356,7 +359,7 @@ Boolean      fixSpaceGroup,fend;
   do {
     RC = Cryst.ConvertPDBString(S,fixSpaceGroup);
     if ((RC!=Error_WrongSection) && ignoreNonCoorPDBErrors)
-      RC = 0; 
+      RC = 0;
     if (RC)  break;
     fend = f.FileEnd();
     if (!fend)  {
@@ -369,7 +372,7 @@ Boolean      fixSpaceGroup,fend;
   if (!RC)  {
     RC = Cryst.ConvertPDBString(S,fixSpaceGroup);
     if ((RC!=Error_WrongSection) && ignoreNonCoorPDBErrors)
-      RC = Error_WrongSection; 
+      RC = Error_WrongSection;
   }
 
   Cryst.CalcCoordTransforms();
@@ -380,7 +383,7 @@ Boolean      fixSpaceGroup,fend;
 
   // temporary solution: the rest of file is stored
   // in the form of strings
-  while (!f.FileEnd()          && 
+  while (!f.FileEnd()          &&
          strncmp(S,"MODEL ",6) &&
          strncmp(S,"ATOM  ",6) &&
          strncmp(S,"SIGATM",6) &&
@@ -460,14 +463,15 @@ Boolean fixSpaceGroup;
   ResetManager  ();
   FreeFileMemory();
   FType = MMDB_FILE_CIF;
-  ignoreSegID   = (Flags & MMDBF_IgnoreSegID  )!=0;
-  ignoreElement = (Flags & MMDBF_IgnoreElement)!=0;
-  ignoreCharge  = (Flags & MMDBF_IgnoreCharge )!=0;
-  ignoreNonCoorPDBErrors = (Flags & MMDBF_IgnoreNonCoorPDBErrors)!=0;
-  ignoreUnmatch = (Flags & MMDBF_IgnoreUnmatch)!=0;
-  allowDuplChID = (Flags & MMDBF_AllowDuplChainID)!=0;
-  fixSpaceGroup = (Flags & MMDBF_FixSpaceGroup   )!=0;
-  
+  ignoreSegID            = (Flags & MMDBF_IgnoreSegID           ) != 0;
+  ignoreElement          = (Flags & MMDBF_IgnoreElement         ) != 0;
+  ignoreCharge           = (Flags & MMDBF_IgnoreCharge          ) != 0;
+  ignoreNonCoorPDBErrors = (Flags & MMDBF_IgnoreNonCoorPDBErrors) != 0;
+  ignoreUnmatch          = (Flags & MMDBF_IgnoreUnmatch         ) != 0;
+  allowDuplChID          = (Flags & MMDBF_AllowDuplChainID      ) != 0;
+  enforceUniqueChID      = (Flags & MMDBF_EnforceUniqueChainID  ) != 0;
+  fixSpaceGroup          = (Flags & MMDBF_FixSpaceGroup         ) != 0;
+
   //  open the file as ASCII for reading
   //  opening it in pseudo-binary mode helps reading various
   //  line terminators for files coming from different platforms
@@ -671,6 +675,8 @@ pstr      chID;
 ChainID   chainID;
 Boolean   NewChain,Done,Solvent;
 
+  k = 0; // else may be used uninitialized
+
   RC = 0;
   if (nAtoms<=0)  return RC;
 
@@ -761,7 +767,7 @@ Boolean   NewChain,Done,Solvent;
               }
             }
           }
-        }  
+        }
         c = 'A';
         if (CleanKey & PDBCLEAN_CHAIN_STRONG)  {
           // rename all chains through from A to Z
@@ -825,7 +831,7 @@ Boolean   NewChain,Done,Solvent;
               nal++;
             }
           }
-        }  
+        }
       }
     c = 'A';
     if (CleanKey & PDBCLEAN_ALTCODE_STRONG)
@@ -1221,7 +1227,7 @@ int  i,k;
 }
 
 
-void  CMMDBFile::FinishStructEdit()  {
+int  CMMDBFile::FinishStructEdit()  {
 // Makes a new atom index after insertion or deletion of atoms.
 // This function may change atoms' positions in the index and
 // correspondingly the CAtom::index field.
@@ -1278,10 +1284,11 @@ int       i,j,k,l,n,index,nAtoms1;
     }
   }
 
-  if (n!=nAtoms1)  {
-    printf ( " ***** PROGRAM ERROR IN CMMDBFile::FinishStructEdit\n" );
-    exit ( 1 );
-  }
+//  if (n!=nAtoms1)  {
+//    printf ( " **** PROGRAM ERROR IN CMMDBFile::FinishStructEdit\n" );
+//    exit ( 1 );
+//  }
+
 
   // check if there are dead atoms in the old index
   for (i=0;i<AtmLen;i++)
@@ -1291,8 +1298,11 @@ int       i,j,k,l,n,index,nAtoms1;
   if (Atom)  delete[] Atom;
 
   Atom   = Atom1;
-  AtmLen = nAtoms1;
-  nAtoms = nAtoms1;
+  AtmLen = n;
+  nAtoms = n;
+
+  if (n==nAtoms1)  return 0;  // Ok
+             else  return 1;  // not Ok; should never happen
 
 }
 
@@ -1481,7 +1491,7 @@ int k;
 
   if (!(Cryst.WhatIsSet & CSET_Transforms))
     return CRRDY_NoTransfMatrices;
- 
+
   if ((Cryst.WhatIsSet & CSET_CellParams)!=CSET_CellParams)
     return CRRDY_NoCell;
 
@@ -1643,7 +1653,7 @@ int         RC,i,index,nATS;
     if (RC && (RC!=Error_CIF_EmptyRow))  return RC;
   }
   if (Flags & MMDBF_AutoSerials)
-    PDBCleanup ( PDBCLEAN_SERIAL ); 
+    PDBCleanup ( PDBCLEAN_SERIAL );
 
   return 0;
 
@@ -1691,7 +1701,7 @@ int  CMMDBFile::PutAtom ( int            index,
 
 int i,kndex,RC;
 
-  kndex = index; 
+  kndex = index;
 
   if (kndex<0)  {  // the new atom is to be inserted
 
@@ -1702,7 +1712,7 @@ int i,kndex,RC;
     if (Atom[kndex-1]!=NULL)  { // the position is occupied
 
       // expand the array if necessary
-      if (nAtoms>=AtmLen)  
+      if (nAtoms>=AtmLen)
         ExpandAtomArray ( IMax(kndex,nAtoms)+1000-AtmLen );
 
       // now shift all atoms from (kndex-1)th to the end of array.
@@ -1712,7 +1722,7 @@ int i,kndex,RC;
         Atom[i] = Atom[i-1];
         Atom[i]->index = i+1;  // this is Ok because residues keep
                                // POINTERS rather than indices!
-      }  
+      }
       Atom[kndex-1] = NULL;
       nAtoms++;
 
@@ -1721,6 +1731,9 @@ int i,kndex,RC;
   }
 
   if (kndex==0)  kndex = nAtoms+1;
+	
+  if (!crModel)  SwitchModel ( 1 );
+
 
   RC = AllocateAtom ( kndex,chainID,resName,seqNum,insCode,True );
   if (!RC)
@@ -1744,7 +1757,7 @@ int i,kndex,RC,sn;
 
   if (!A)  return -1;
 
-  kndex = index; 
+  kndex = index;
 
   if (kndex<0)  {  // the new atom is to be inserted
 
@@ -1756,7 +1769,7 @@ int i,kndex,RC,sn;
     if (Atom[kndex-1]!=NULL)  { // the position is occupied
 
       // expand the array if necessary
-      if (nAtoms>=AtmLen)  
+      if (nAtoms>=AtmLen)
         ExpandAtomArray ( IMax(kndex,nAtoms)+1000-AtmLen );
       // now shift all atoms from (kndex-1)th to the end of array.
       // note that this does not affect residues as they keep only
@@ -1800,7 +1813,7 @@ int i,kndex;
 
   if (!A)  return -1;
 
-  kndex = index; 
+  kndex = index;
 
   if (kndex<0)  {  // the new atom is to be inserted
 
@@ -1812,7 +1825,7 @@ int i,kndex;
     if (Atom[kndex-1]!=NULL)  { // the position is occupied
 
       // expand the array if necessary
-      if (nAtoms>=AtmLen)  
+      if (nAtoms>=AtmLen)
         ExpandAtomArray ( IMax(kndex,nAtoms)+1000-AtmLen );
       // now shift all atoms from (kndex-1)th to the end of array.
       // note that this does not affect residues as they keep only
@@ -1850,24 +1863,30 @@ int CMMDBFile::CheckInAtoms ( int index, // same meaning as above
 PPCAtom A1;
 int     i,j,k,k1,kndex;
 
+  A1 = (PCAtom *) NULL;  // else may be used uninitialized 
+
   if (!A)  return -1;
 
-  kndex = index; 
+  kndex = index;
 
   if (kndex<0)  {  // the new atoms are to be inserted
 
     kndex = -kndex;
 
-    if (nAtoms+natms>=AtmLen)  
+    if (nAtoms+natms>=AtmLen)
       ExpandAtomArray ( IMax(kndex,nAtoms)+1000+natms-AtmLen );
 
     if (kndex<nAtoms)
-    A1 = new PCAtom[natms];
+      A1 = new PCAtom[natms];
+
+    // A1 may be undefined at this point, yet it is used in the
+    // next loop!
+
     k = kndex-1;
     j = 0;
     for (i=0;i<natms;i++)
       if (A[i])  {
-        if (Atom[k])  A1[j++] = Atom[k];
+        if (Atom[k])  A1[j++] = Atom[k]; 
         Atom[k] = A[i];
         Atom[k]->index = k+1;
         k++;
@@ -2013,7 +2032,7 @@ InsCode  insCode;
 
   return AllocateAtom ( index ,chainID,resName,
                         seqNum,insCode,False );
- 
+
 }
 
 int CMMDBFile::CheckAtomPlace ( int index, PCMMCIFLoop Loop )  {
@@ -2061,7 +2080,7 @@ pstr     F;
 
   return AllocateAtom ( index ,chainID,resName,
                         seqNum,insCode,False );
- 
+
 }
 
 
@@ -2092,13 +2111,23 @@ int  CMMDBFile::AllocateAtom ( int           index,
     //   All this does not work if insCode[0] is set to 1
     // which indicates a special case of 'TER' card without
     // parameters.
-    if (strcmp(chainID,crChain->chainID))
+    if (enforceUniqueChID)  {
+      // enforcing unique chain IDs should be used only in case
+      // of multi-chain complexes where 1-letter chain IDs are
+      // not enough to accomodate all chains. Then chains are
+      // dynamically renamed like A0,A1,A2,.. etc. Therefore, we
+      // check only first symbol here.
+      if (chainID[0]!=crChain->chainID[0])
+        crChain = NULL;  // the chain has to be changed
+    } else if (strcmp(chainID,crChain->chainID))
       crChain = NULL;  // the chain has to be changed
   }
   if (!crChain) {
     // either the model or chain was changed  -- get a new chain
-    if (allowDuplChID)  crChain = crModel->CreateChain    ( chainID );
-                  else  crChain = crModel->GetChainCreate ( chainID );
+    if (allowDuplChID)
+          crChain = crModel->CreateChain    ( chainID );
+    else  crChain = crModel->GetChainCreate ( chainID,
+                                              enforceUniqueChID );
     crRes = NULL;  // new chain - new residue
   }
 
@@ -2149,7 +2178,7 @@ PPCAtom Atom1;
 int     i;
   AtmLen += inc;
   Atom1   = new PCAtom[AtmLen];
-  for (i=0;i<nAtoms;i++)      
+  for (i=0;i<nAtoms;i++)
     Atom1[i] = Atom[i];
   for (i=nAtoms;i<AtmLen;i++)
     Atom1[i] = NULL;
@@ -2167,7 +2196,7 @@ int     i;
   if (nAtoms+inc>AtmLen)  {
     AtmLen = nAtoms+inc;
     Atom1  = new PCAtom[AtmLen];
-    for (i=0;i<nAtoms;i++)      
+    for (i=0;i<nAtoms;i++)
       Atom1[i] = Atom[i];
     for (i=nAtoms;i<AtmLen;i++)
       Atom1[i] = NULL;
@@ -2592,6 +2621,7 @@ Boolean CMMDBFile::GetNewChainID ( int modelNo, ChainID chID,
 //  -------------------------------------------------------------
 
 PCMask CMMDBFile::GetSelMask ( int selHnd )  {
+UNUSED_ARGUMENT(selHnd);
   return NULL;
 }
 
@@ -2646,9 +2676,9 @@ int i;
         Model[i]->_copy ( MMDBFile->Model[i] );
       } else
         Model[i] = NULL;
-    } 
+    }
   }
- 
+
   SA      .Copy ( &MMDBFile->SA       );
   Footnote.Copy ( &MMDBFile->Footnote );
   SB      .Copy ( &MMDBFile->SB       );
@@ -2792,7 +2822,7 @@ byte Version;
         // model->chain below
       } else
         Atom[i] = NULL;
-    } 
+    }
   }
 
   f.ReadInt ( &nModels );
@@ -2806,7 +2836,7 @@ byte Version;
         Model[i]->read ( f );
       } else
         Model[i] = NULL;
-    } 
+    }
   }
 
   SA      .read ( f );
