@@ -1,4 +1,3 @@
-//  $Id: mmdb_file.h,v 1.22.2.1 2009/09/28 11:00:26 ccb Exp $
 //  =================================================================
 //
 //   CCP4 Coordinate Library: support of coordinate-related
@@ -6,13 +5,13 @@
 //
 //   Copyright (C) Eugene Krissinel 2000-2008.
 //
-//    This library is free software: you can redistribute it and/or 
-//    modify it under the terms of the GNU Lesser General Public 
-//    License version 3, modified in accordance with the provisions 
+//    This library is free software: you can redistribute it and/or
+//    modify it under the terms of the GNU Lesser General Public
+//    License version 3, modified in accordance with the provisions
 //    of the license to address the requirements of UK law.
 //
-//    You should have received a copy of the modified GNU Lesser 
-//    General Public License along with this library. If not, copies 
+//    You should have received a copy of the modified GNU Lesser
+//    General Public License along with this library. If not, copies
 //    may be downloaded from http://www.ccp4.ac.uk/ccp4license.php
 //
 //    This program is distributed in the hope that it will be useful,
@@ -93,6 +92,21 @@
 #define MMDBF_AllowDuplChainID        0x00004000
 #define MMDBF_FixSpaceGroup           0x00008000
 #define MMDBF_EnforceAtomNames        0x00010000
+#define MMDBF_EnforceUniqueChainID    0x00020000
+
+// MMDBF_EnforceUniqueChainID   will make MMDB to rename chains on
+//         reading a file such as to maintain chains uniquesness. This
+//         is supposed to work only with 1-letter chain IDs and only
+//         if chain names are interchanged in the file. For example,
+//         if file contains a sequence of chains named
+//
+//              A,B, A,B, A,B, A,B, A,B
+//
+//         and this flag is set on, the resulting chain names in
+//         MMDB will be:
+//
+//              A,B, A0,B0, A1,B1, A2,B2, A3,B3
+//
 
 // file types:
 #define MMDB_FILE_Undefined     -1
@@ -189,7 +203,7 @@ class CMMDBFile : public CUDData  {
                            byte gzipMode=GZM_CHECK );
     int   WriteCIFASCII1 ( cpstr CIFLFName,
                            byte gzipMode=GZM_CHECK );
-    
+
     int   ReadMMDBF      ( cpstr MMDBFileName,
                            byte gzipMode=GZM_CHECK );
     int   ReadMMDBF1     ( cpstr MMDBLFName,
@@ -320,7 +334,7 @@ class CMMDBFile : public CUDData  {
     // model.
     void  RemoveAtom ( int index );
 
-    void  FinishStructEdit();
+    int   FinishStructEdit();
 
     void  TrimModelTable();
 
@@ -556,9 +570,10 @@ class CMMDBFile : public CUDData  {
     PCChain     crChain; // current chain, used at reading a PDB file
     PCResidue   crRes;   // current residue, used at reading a PDB file
 
-    Boolean     Exclude; // used internally
-    Boolean     ignoreRemarks;  // used temporarily
-    Boolean     allowDuplChID;  // used temporarily
+    Boolean     Exclude;            // used internally
+    Boolean     ignoreRemarks;      // used temporarily
+    Boolean     allowDuplChID;      // used temporarily
+    Boolean     enforceUniqueChID;  // used temporarily
 
     void  InitMMDBFile    ();
     void  FreeCoordMemory ();
