@@ -1,4 +1,3 @@
-//  $Id: mmdb_manager.cpp,v 1.17.2.3 2009/09/28 13:43:35 ccb Exp $
 //  =================================================================
 //
 //   CCP4 Coordinate Library: support of coordinate-related
@@ -6,13 +5,13 @@
 //
 //   Copyright (C) Eugene Krissinel 2000-2008.
 //
-//    This library is free software: you can redistribute it and/or 
-//    modify it under the terms of the GNU Lesser General Public 
-//    License version 3, modified in accordance with the provisions 
+//    This library is free software: you can redistribute it and/or
+//    modify it under the terms of the GNU Lesser General Public
+//    License version 3, modified in accordance with the provisions
 //    of the license to address the requirements of UK law.
 //
-//    You should have received a copy of the modified GNU Lesser 
-//    General Public License along with this library. If not, copies 
+//    You should have received a copy of the modified GNU Lesser
+//    General Public License along with this library. If not, copies
 //    may be downloaded from http://www.ccp4.ac.uk/ccp4license.php
 //
 //    This program is distributed in the hope that it will be useful,
@@ -60,7 +59,11 @@ CMMDBManager::CMMDBManager ( RPCStream Object )
 CMMDBManager::~CMMDBManager()  {}
 
 void  CMMDBManager::Copy ( PCMMDBManager MMDB, word CopyMask )  {
-int i;
+PCModel  model;
+PPCChain chain;
+PCChain  ch;
+ChainID  chID;
+int      i,j, nchains;
 
   if (CopyMask & MMDBFCM_Flags)  Flags = MMDB->Flags;
 
@@ -86,7 +89,7 @@ int i;
           Atom[i]->SetAtomIndex ( i+1 );
         } else
           Atom[i] = NULL;
-      } 
+      }
     }
 
     nModels = MMDB->nModels;
@@ -99,7 +102,7 @@ int i;
           Model[i]->_copy ( MMDB->Model[i] );
         } else
           Model[i] = NULL;
-      } 
+      }
     }
 
     crModel = NULL;
@@ -162,8 +165,49 @@ int i;
     }
     */
 
+  } else if (CopyMask & (MMDBFCM_HetInfo | MMDBFCM_SecStruct |
+                          MMDBFCM_Links | MMDBFCM_CisPeps |
+                          MMDBFCM_ChainAnnot))  {
+
+    for (i=0;i<MMDB->nModels;i++)
+      if (MMDB->Model[i])  {
+
+        model = GetModel ( i+1 );
+        if (!model)  {
+          model = new CModel( NULL,i+1 );
+          AddModel ( model );
+        }
+
+        if (CopyMask & MMDBFCM_HetInfo)
+          model->CopyHets ( MMDB->Model[i] );
+        if (CopyMask & MMDBFCM_SecStruct)
+          model->CopySecStructure ( MMDB->Model[i] );
+        if (CopyMask & MMDBFCM_Links)  {
+          model->CopyLinks  ( MMDB->Model[i] );
+          model->CopyLinkRs ( MMDB->Model[i] );
+        }
+        if (CopyMask & MMDBFCM_CisPeps)
+          model->CopyCisPeps ( MMDB->Model[i] );
+        if (CopyMask & MMDBFCM_ChainAnnot)  {
+          MMDB->GetChainTable ( i+1,chain,nchains );
+          for (j=0;j<nchains;j++)
+            if (chain[j])  {
+              chain[j]->GetChainID ( chID );
+              ch = model->GetChain ( chID );
+              if (!ch)  {
+                ch = new CChain();
+                ch->SetChainID ( chID );
+                model->AddChain ( ch );
+              }
+              ch->CopyAnnotations ( chain[j] );
+            }
+
+        }
+
+      }
+
   }
- 
+
   if (CopyMask & MMDBFCM_SA)  SA.Copy ( &(MMDB->SA) );
   if (CopyMask & MMDBFCM_SB)  SB.Copy ( &(MMDB->SB) );
   if (CopyMask & MMDBFCM_SC)  SC.Copy ( &(MMDB->SC) );
@@ -175,7 +219,7 @@ int i;
     strncpy ( S,MMDB->S,sizeof(S) );
   }
 
-} 
+}
 
 void  CMMDBManager::Delete ( word DelMask )  {
 PPCModel model;
@@ -213,8 +257,10 @@ int      i,j,nm, nchains;
     GetModelTable ( model,nm );
     if (model)
       for (i=0;i<nm;i++)
-        if (model[i])
-          model[i]->RemoveLinks();
+        if (model[i])  {
+          model[i]->RemoveLinks ();
+          model[i]->RemoveLinkRs();
+        }
   }
 
   if (DelMask & MMDBFCM_CisPeps)  {
@@ -235,7 +281,7 @@ int      i,j,nm, nchains;
             chain[j]->FreeAnnotations();
     }
   }
- 
+
   if (DelMask & MMDBFCM_SA)        SA.FreeContainer();
   if (DelMask & MMDBFCM_SB)        SB.FreeContainer();
   if (DelMask & MMDBFCM_SC)        SC.FreeContainer();
