@@ -1,4 +1,3 @@
-//  $Id: mmdb_mmcif.cpp,v 1.16.2.1 2009/09/28 11:00:26 ccb Exp $
 //  =================================================================
 //
 //   CCP4 Coordinate Library: support of coordinate-related
@@ -6,13 +5,13 @@
 //
 //   Copyright (C) Eugene Krissinel 2000-2008.
 //
-//    This library is free software: you can redistribute it and/or 
-//    modify it under the terms of the GNU Lesser General Public 
-//    License version 3, modified in accordance with the provisions 
+//    This library is free software: you can redistribute it and/or
+//    modify it under the terms of the GNU Lesser General Public
+//    License version 3, modified in accordance with the provisions
 //    of the license to address the requirements of UK law.
 //
-//    You should have received a copy of the modified GNU Lesser 
-//    General Public License along with this library. If not, copies 
+//    You should have received a copy of the modified GNU Lesser
+//    General Public License along with this library. If not, copies
 //    may be downloaded from http://www.ccp4.ac.uk/ccp4license.php
 //
 //    This program is distributed in the hope that it will be useful,
@@ -22,21 +21,21 @@
 //
 //  =================================================================
 //
-//    08.07.08   <--  Date of Last Modification.
+//    29.01.10   <--  Date of Last Modification.
 //                   ~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 //  -----------------------------------------------------------------
 //
 //  **** Module  :  MMDB_MMCIF  <implementation>
 //       ~~~~~~~~~
 //  **** Project :  MacroMolecular Data Base (MMDB)
-//       ~~~~~~~~~ 
+//       ~~~~~~~~~
 //  **** Classes :  CMMCIFCategory ( mmCIF category    )
 //       ~~~~~~~~~  CMMCIFStruct   ( mmCIF structure   )
 //                  CMMCIFLoop     ( mmCIF loop        )
 //                  CMMCIFData     ( mmCIF data block  )
 //                  CMMCIFFile     ( mmCIF file        )
 //
-//  (C) E. Krissinel 2000-2008
+//  (C) E. Krissinel 2000-2010
 //
 //  =================================================================
 //
@@ -100,7 +99,7 @@ void  SortTags ( psvector tag, int len, ivector index )  {
 //   The idea is, the tags are created already sorted and
 // SortTags(..) should serve only in special cases, occasionally.
 // The bubble sorting should perform best in this situation
-// becouse of its N-complexity for sorted arrays.
+// because of its N-complexity for sorted arrays.
 //   Note: index must contain valid indices on input.
 int     i,i1,i2;
 Boolean Done;
@@ -310,7 +309,7 @@ void CMMCIFCategory::Sort()  {
 int i,k;
   if (nAllocTags>0)  {
     k = 0;
-    if (!index) 
+    if (!index)
       GetVectorMemory ( index,nAllocTags,0 );
     for (i=0;i<nTags;i++)
       if (tag[i])  {
@@ -370,7 +369,7 @@ int l1,l2,l,k;
     }
   }
   if (k==0)  return index[l];    // is at RCth position
-  k = strcasecmp ( ttag,tag[index[l1]] );   
+  k = strcasecmp ( ttag,tag[index[l1]] );
   if (k==0)  return index[l1];   // is at RCth position
   if (k<0)   return -1;          // would be at (-RC-1)th position
   if (l2!=l1)  {
@@ -393,7 +392,7 @@ int  i1,i;
     nTags  = 1;
     return -nTags;  // the tag has been added on the top of array
   }
-  i1 = GetTagNo ( ttag ); 
+  i1 = GetTagNo ( ttag );
   if (i1>=0)  return i1;  // non-negative returns mean that
                           // the tag is already in the array
   i1 = -i1-1;  // otherwise the tag has to be added and indexed at here
@@ -507,7 +506,7 @@ CMMCIFStruct::~CMMCIFStruct()  {
 
 void CMMCIFStruct::FreeMemory()  {
 int i;
-  for (i=0;i<nAllocTags;i++) 
+  for (i=0;i<nAllocTags;i++)
     if (field[i]) delete[] field[i];
   FreeVectorMemory ( field,0 );
   CMMCIFCategory::FreeMemory();
@@ -694,7 +693,7 @@ pstr p;
   if (!S)  PutNoData ( CIF_NODATA_QUESTION,T );
   else  {
     p = pstr(S);
-    if (NonBlankOnly) 
+    if (NonBlankOnly)
       while (*p==' ')  p++;
     if (!(*p))  PutNoData ( CIF_NODATA_DOT,T );
           else  AddField  ( S,T,False );
@@ -830,7 +829,7 @@ int i;
 void CMMCIFStruct::write ( RCFile f )  {
 int i;
   CMMCIFCategory::write ( f );
-  for (i=0;i<nTags;i++)  
+  for (i=0;i<nTags;i++)
     f.CreateWrite ( field[i] );
 }
 
@@ -994,10 +993,10 @@ int       i,j,nT1;
         } else
           field[i] = NULL;
       delete[] f1;
-    } else 
+    } else
       // The tag was already in the category. Just restore fields.
       field = f1;
-  }              
+  }
 }
 
 
@@ -1686,9 +1685,9 @@ int i,lc,lt;
   FreeWrongFields();
   if ((!cats) || (!tags))  return;
   lc = 0;
-  while (cats[lc]) lc++; 
+  while (cats[lc]) lc++;
   lt = 0;
-  while (tags[lt]) lt++; 
+  while (tags[lt]) lt++;
   nWrongFields = IMax(lc,lt);
   if (nWrongFields>0)  {
     WrongCat = new pstr[nWrongFields];
@@ -1828,7 +1827,7 @@ pstr L;
   Optimize();  // get rid of over-allocated fields.
 
   return Warning;
-  
+
 }
 
 
@@ -1840,7 +1839,7 @@ int           RC,i;
 
   i = 0;
   while ((*p) && (*p!=' ') && (*p!=char(9)) && (*p!='.'))  {
-    if (i<sizeof(T)-1)  T[i++] = *p;
+    if (i<(int)sizeof(T)-1)  T[i++] = *p;
     p++;
   }
   T[i] = char(0);
@@ -1943,7 +1942,7 @@ int         RC,i,nC;
       // get category name
       i = 0;
       while ((*p) && (*p!=' ') && (*p!=char(9)) && (*p!='.'))  {
-        if (i<sizeof(T)-1)  T[i++] = *p;
+        if (i<(int)sizeof(T)-1)  T[i++] = *p;
         p++;
       }
       T[i] = char(0);
@@ -2005,7 +2004,7 @@ int         RC,i,nC;
       }
 */
       if (Loop)  {
-  
+
         if (*p=='.')  {  // get item name
           i = 0;
           p++;  // skip period
@@ -2020,7 +2019,7 @@ int         RC,i,nC;
         if (nWrongFields>0)
               WrongField = CheckWrongField ( Loop->name,T );
         else  WrongField = False;
- 
+
         if (!WrongField)  {
           if (Loop->AddTag(T)>=0)  {
             if (flags & CIFFL_SuggestTags)  {
@@ -2156,7 +2155,7 @@ char    c;
           delete[] L;
           L = L1;
         }
-        strcat ( L,"\n" );  
+        strcat ( L,"\n" );
         strcat ( L,S );
       }
     }
@@ -2165,7 +2164,7 @@ char    c;
     p = &(S[strlen(S)]);
 
   } else  {
-  
+
     i = 0;
     if (*p!='_')  {
       if ((*p=='\'') || (*p=='"'))  {
@@ -2258,6 +2257,8 @@ int  CMMCIFData::GetCategoryNo ( cpstr cname )  {
 //          (-RC-1)th element, where RC is the return value
 int l1,l2,l,k;
 
+  l = 0; // may become undefined in not initialized
+
   if (!Category) return -1;
   if (!index)    Sort();
   if (cname[0])  {
@@ -2310,7 +2311,7 @@ ivector          index1;
     nCategories  = 1;
     return -nCategories;  // the category has been added on the top of array
   }
-  l1 = GetCategoryNo ( cname ); 
+  l1 = GetCategoryNo ( cname );
   if (l1>=0)  return l1;  // non-negative returns mean that
                           // the category is already in the array
   l1 = -l1-1;  // otherwise the category has to be added and indexed at here
@@ -2355,7 +2356,7 @@ int  i;
     f.WriteLine ( pstr("\ndata_") );
 
   for (i=0;i<nCategories;i++)
-    if (Category[i]) 
+    if (Category[i])
       Category[i]->WriteMMCIF ( f );
 
 }
@@ -2469,7 +2470,7 @@ int  CMMCIFData::CheckData ( cpstr CName, cpstr TName )  {
 //                        tag TName
 //   CIFRC_NoCategory      category CName is not present.
 // If TName is set to NULL then only the CName is checked and
-// possible returns are CIFRC_Structure, CIFRC_Loop and 
+// possible returns are CIFRC_Structure, CIFRC_Loop and
 // CIFRC_NoCategory.
 int i,k;
   i = GetCategoryNo ( CName );
@@ -2491,13 +2492,13 @@ void CMMCIFData::Optimize()  {
 int              i,k;
 PPCMMCIFCategory C1;
   k = 0;
-  for (i=0;i<nCategories;i++) 
+  for (i=0;i<nCategories;i++)
     if (Category[i])  {
       Category[i]->Optimize();
       if (Category[i]->nTags<=0)  {
         delete Category[i];
         Category[i] = NULL;
-      } else 
+      } else
         k++;
     }
   if (k>0)  {
@@ -2577,7 +2578,7 @@ int  CMMCIFData::GetInteger ( int & I, cpstr CName,
 int j = GetCategoryNo ( CName );
   if (j<0)  return CIFRC_NoCategory;
   if (Category[j]->GetCategoryID()!=MMCIF_Struct)
-            return CIFRC_NotAStructure; 
+            return CIFRC_NotAStructure;
   return PCMMCIFStruct(Category[j])->GetInteger ( I,TName,Remove );
 }
 
@@ -2638,7 +2639,7 @@ int  CMMCIFData::GetLoopReal ( realtype & R, cpstr CName,
 int i = GetCategoryNo ( CName );
   if (i<0)  return CIFRC_NoCategory;
   if (Category[i]->GetCategoryID()!=MMCIF_Loop)
-            return CIFRC_NotALoop; 
+            return CIFRC_NotALoop;
   return PCMMCIFLoop(Category[i])->GetReal ( R,TName,nrow,Remove );
 }
 
@@ -2648,7 +2649,7 @@ int  CMMCIFData::GetLoopInteger ( int & I, cpstr CName,
 int j = GetCategoryNo ( CName );
   if (j<0)  return CIFRC_NoCategory;
   if (Category[j]->GetCategoryID()!=MMCIF_Loop)
-            return CIFRC_NotALoop; 
+            return CIFRC_NotALoop;
   return PCMMCIFLoop(Category[j])->GetInteger ( I,TName,nrow,Remove );
 }
 
@@ -3033,7 +3034,7 @@ int         i,RC;
 
 }
 
-int  CMMCIFData::PutLoopIVector ( ivector I, cpstr CName, 
+int  CMMCIFData::PutLoopIVector ( ivector I, cpstr CName,
                                   cpstr TName,
                                   int i1, int i2 )  {
 PCMMCIFLoop Loop;
@@ -3152,7 +3153,7 @@ void CMMCIFData::PrintCategories()  {
 int i;
   printf ( " Total %i categories:\n",nCategories );
   for (i=0;i<nCategories;i++)
-    if (Category[i])  {  
+    if (Category[i])  {
       printf ( " %5i. ",i+1 );
       if (Category[i]->GetCategoryID()==MMCIF_Loop)
             printf ( "Loop      %s\n",Category[i]->name );
@@ -3360,7 +3361,7 @@ int l1,l2,l,k;
     }
   }
   if (k==0)  return index[l];    // is at RCth position
-  k = strcasecmp ( DName,data[index[l1]]->name );   
+  k = strcasecmp ( DName,data[index[l1]]->name );
   if (k==0)  return index[l1];   // is at RCth position
   if (k<0)   return -1;          // would be at (-RC-1)th position
   if (l2!=l1)  {
@@ -3421,7 +3422,7 @@ int  i1,i;
     return -nData;     // the CIF data structure has been added
                        // "on the top" of array
   }
-  i1 = GetCIFDataNo ( DName ); 
+  i1 = GetCIFDataNo ( DName );
   if (i1>=0)  return i1;  // non-negative returns mean that the CIF
                           // data structure is already in the array
   i1 = -i1-1;  // otherwise the data has to be added and indexed at here
