@@ -1,4 +1,3 @@
-//  $Id: mmdb_mmcif.h,v 1.15.2.1 2009/09/28 11:00:26 ccb Exp $
 //  =================================================================
 //
 //   CCP4 Coordinate Library: support of coordinate-related
@@ -6,13 +5,13 @@
 //
 //   Copyright (C) Eugene Krissinel 2000-2008.
 //
-//    This library is free software: you can redistribute it and/or 
-//    modify it under the terms of the GNU Lesser General Public 
-//    License version 3, modified in accordance with the provisions 
+//    This library is free software: you can redistribute it and/or
+//    modify it under the terms of the GNU Lesser General Public
+//    License version 3, modified in accordance with the provisions
 //    of the license to address the requirements of UK law.
 //
-//    You should have received a copy of the modified GNU Lesser 
-//    General Public License along with this library. If not, copies 
+//    You should have received a copy of the modified GNU Lesser
+//    General Public License along with this library. If not, copies
 //    may be downloaded from http://www.ccp4.ac.uk/ccp4license.php
 //
 //    This program is distributed in the hope that it will be useful,
@@ -22,7 +21,7 @@
 //
 //  =================================================================
 //
-//    08.07.08   <--  Date of Last Modification.
+//    29.01.10   <--  Date of Last Modification.
 //                   ~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 //  -----------------------------------------------------------------
 //
@@ -36,7 +35,7 @@
 //                  CMMCIFData     ( mmCIF data block  )
 //                  CMMCIFFile     ( mmCIF file        )
 //
-//  (C) E. Krissinel 2000-2008
+//  (C) E. Krissinel 2000-2010
 //
 //  =================================================================
 //
@@ -74,7 +73,7 @@ class CMMCIFCategory : public CStream  {
 
     pstr   GetCategoryName    () { return name;           }
     virtual int  GetCategoryID() { return MMCIF_Category; }
-    virtual void WriteMMCIF   ( RCFile f ) {}
+    virtual void WriteMMCIF   ( RCFile ) {}
     virtual void Optimize     ();
 
     void  Sort     ();
@@ -412,7 +411,7 @@ class CMMCIFData : public CStream  {
     //                        tag TName
     //   CIFRC_NoCategory      category CName is not present.
     // If TName is set to NULL then only the CName is checked and
-    // possible returns are CIFRC_Structure, CIFRC_Loop and 
+    // possible returns are CIFRC_Structure, CIFRC_Loop and
     // CIFRC_NoCategory.
     int  CheckData       ( cpstr CName, cpstr TName );
 
@@ -561,7 +560,7 @@ class CMMCIFData : public CStream  {
     // for nrow in the loop and move down to 0. The least efficient way
     // is to start with nrow=0 and move up.
     //   These functions allow to form loops in arbitrary way.
-    //   The functions may return CIFRC_Ok or CIFRC_NotALoop. 
+    //   The functions may return CIFRC_Ok or CIFRC_NotALoop.
     int  PutLoopNoData  ( int NoDataType, cpstr CName,
                                           cpstr TName, int nrow );
     int  PutLoopString  ( cpstr S,   cpstr CName,
