@@ -1,4 +1,3 @@
-//  $Id: mmdb_model.h,v 1.18 2008/08/21 09:54:03 nds65 Exp $
 //  =================================================================
 //
 //   CCP4 Coordinate Library: support of coordinate-related
@@ -6,13 +5,13 @@
 //
 //   Copyright (C) Eugene Krissinel 2000-2008.
 //
-//    This library is free software: you can redistribute it and/or 
-//    modify it under the terms of the GNU Lesser General Public 
-//    License version 3, modified in accordance with the provisions 
+//    This library is free software: you can redistribute it and/or
+//    modify it under the terms of the GNU Lesser General Public
+//    License version 3, modified in accordance with the provisions
 //    of the license to address the requirements of UK law.
 //
-//    You should have received a copy of the modified GNU Lesser 
-//    General Public License along with this library. If not, copies 
+//    You should have received a copy of the modified GNU Lesser
+//    General Public License along with this library. If not, copies
 //    may be downloaded from http://www.ccp4.ac.uk/ccp4license.php
 //
 //    This program is distributed in the hope that it will be useful,
@@ -22,7 +21,7 @@
 //
 //  =================================================================
 //
-//    08.07.08   <--  Date of Last Modification.
+//    30.04.10   <--  Date of Last Modification.
 //                   ~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 //  -----------------------------------------------------------------
 //
@@ -40,11 +39,13 @@
 //                  CTurn         ( turn info                       )
 //                  CLinkContainer   ( container for link data      )
 //                  CLink            ( link data                    )
+//                  CLinkRContainer  ( container for refmac link    )
+//                  CLinkR           ( link data                    )
 //                  CCisPepContainer ( container for CisPep data    )
 //                  CCisPep          ( CisPep data                  )
 //                  CModel        ( PDB model                       )
 //
-//  Copyright (C) E. Krissinel 2000-2008
+//  Copyright (C) E. Krissinel 2000-2010
 //
 //  =================================================================
 //
@@ -126,7 +127,7 @@ class CSSContainer : public CClassContainer  {
 
     CSSContainer  () : CClassContainer() {}
     CSSContainer  ( RPCStream Object )
-                     : CClassContainer ( Object ) {} 
+                     : CClassContainer ( Object ) {}
     ~CSSContainer () {}
 
     PCContainerClass MakeContainerClass ( int ClassID );
@@ -173,7 +174,7 @@ class CHelix : public CContainerClass  {
     void  read  ( RCFile f );
 
   protected :
-    
+
     void InitHelix();
 
 };
@@ -231,7 +232,7 @@ class CStrand : public CStream  {
     void  read  ( RCFile f );
 
   protected :
-    
+
     void InitStrand();
 
 };
@@ -346,7 +347,7 @@ class CTurn : public CContainerClass  {
     void  read  ( RCFile f );
 
   protected :
-    
+
     void InitTurn();
 
 };
@@ -404,7 +405,7 @@ class CLinkContainer : public CClassContainer  {
 
     CLinkContainer  () : CClassContainer() {}
     CLinkContainer  ( RPCStream Object )
-                     : CClassContainer ( Object ) {} 
+                     : CClassContainer ( Object ) {}
     ~CLinkContainer () {}
 
     PCContainerClass MakeContainerClass ( int ClassID );
@@ -434,7 +435,7 @@ class CLink : public CContainerClass  {
     InsCode  insCode2;  // insertion code of 2nd linked atom
     int      s1,i1,j1,k1;  // sym id of 1st atom
     int      s2,i2,j2,k2;  // sym id of 2nd atom
-    
+
     CLink ();
     CLink ( cpstr S );
     CLink ( RPCStream Object );
@@ -452,11 +453,105 @@ class CLink : public CContainerClass  {
     void  read  ( RCFile f );
 
   protected :
-    
+
     void InitLink();
 
 };
 
+//  ===================  CLinkRContainer  ====================
+
+DefineClass(CLinkRContainer)
+DefineStreamFunctions(CLinkRContainer)
+
+class CLinkRContainer : public CClassContainer  {
+
+  public :
+
+    CLinkRContainer  () : CClassContainer() {}
+    CLinkRContainer  ( RPCStream Object )
+                     : CClassContainer ( Object ) {}
+    ~CLinkRContainer () {}
+
+    PCContainerClass MakeContainerClass ( int ClassID );
+
+};
+
+
+//  ====================  CLinkR  ============================
+
+DefineClass(CLinkR)
+DefineStreamFunctions(CLinkR)
+
+
+/*
+
+Garib's
+LINK             LYS A  27                     PLP A 255                PLPLYS
+LINK             MAN S   3                     MAN S   4                BETA1-4
+LINK        C6  BBEN B   1                O1  BMAF S   2                BEN-MAF
+LINK        OE2 AGLU A 320                C1  AMAF S   2                GLU-MAF
+LINK        OE2  GLU A  67        1.895   ZN   ZN  R   5                GLU-ZN
+LINK        NE2  HIS A  71        2.055   ZN   ZN  R   5                HIS-ZN
+LINK        O    ARG A  69        2.240   NA   NA  R   9                ARG-NA
+
+Coot's
+LINKR        O   VAL C 103                NA    NA C 401                VAL-NA
+LINKR        OD1 ASP D  58                NA    NA D 401                ASP-NA
+LINKR        O   ALA D  97                NA    NA D 401                ALA-NA
+LINKR        OG1 THR D  99                NA    NA D 401                THR-NA
+LINKR        O   SER D 101                NA    NA D 401                SER-NA
+LINKR        O   VAL D 103                NA    NA D 401                VAL-NA
+
+PDB's
+LINK         O   GLY A  49                NA    NA A6001     1555   1555  2.98
+LINK         OG1 THR A  51                NA    NA A6001     1555   1555  2.72
+LINK         OD2 ASP A  66                NA    NA A6001     1555   1555  2.72
+LINK         NE  ARG A  68                NA    NA A6001     1555   1555  2.93
+
+LINK         NE  ARG A  68                NA    NA A6001     1555   1555  2.93
+LINK         C21 2EG A   7                 C22 2EG B  19     1555   1555  1.56
+*/
+
+class CLinkR : public CContainerClass  {
+
+  public :
+    LinkRID  linkRID;   // link name
+    AtomName atName1;   // name of 1st linked atom
+    AltLoc   aloc1;     // alternative location of 1st linked atom
+    ResName  resName1;  // residue name of 1st linked atom
+    ChainID  chainID1;  // chain ID of 1st linked atom
+    int      seqNum1;   // sequence number of 1st linked atom
+    InsCode  insCode1;  // insertion code of 1st linked atom
+    AtomName atName2;   // name of 2nd linked atom
+    AltLoc   aloc2;     // alternative location of 2nd linked atom
+    ResName  resName2;  // residue name of 2nd linked atom
+    ChainID  chainID2;  // chain ID of 2nd linked atom
+    int      seqNum2;   // sequence number of 2nd linked atom
+    InsCode  insCode2;  // insertion code of 2nd linked atom
+    realtype dist;      // link distance
+
+    CLinkR ();
+    CLinkR ( cpstr S );
+    CLinkR ( RPCStream Object );
+    ~CLinkR();
+
+    void  PDBASCIIDump    ( pstr S, int N   );
+    void  MakeCIF         ( PCMMCIFData CIF, int N );
+    int   ConvertPDBASCII ( cpstr S );
+    void  GetCIF          ( PCMMCIFData CIF, int & Signal );
+    int   GetClassID      () { return ClassID_LinkR; }
+
+    void  Copy  ( PCContainerClass LinkR );
+
+    void  write ( RCFile f );
+    void  read  ( RCFile f );
+
+  protected :
+
+    void InitLinkR();
+
+};
+
 
 
 //  ===================  CCisPepContainer  =====================
@@ -470,7 +565,7 @@ class CCisPepContainer : public CClassContainer  {
 
     CCisPepContainer  () : CClassContainer() {}
     CCisPepContainer  ( RPCStream Object )
-                     : CClassContainer ( Object ) {} 
+                     : CClassContainer ( Object ) {}
     ~CCisPepContainer () {}
 
     PCContainerClass MakeContainerClass ( int ClassID );
@@ -531,6 +626,9 @@ class CCisPep : public CContainerClass  {
 #define  SSERC_noAminoacids  2
 #define  SSERC_noSSE         3
 
+#define  SORT_CHAIN_ChainID_Asc    0
+#define  SORT_CHAIN_ChainID_Desc   1
+
 DefineFactoryFunctions(CModel)
 
 class CModel : public CProModel  {
@@ -552,12 +650,17 @@ class CModel : public CProModel  {
     ~CModel();
 
     void   SetMMDBManager   ( PCMMDBManager MMDBM, int serialNum );
-    void * GetCoordHierarchy() { return manager; }    
-    
+    PCMMDBManager GetCoordHierarchy() { return manager; }
+
     //   GetChainCreate() returns pointer on chain, whose identifier
     // is given in chID. If such a chain is absent in the model,
-    // it is created.
-    PCChain GetChainCreate ( const ChainID chID );
+    // it is created. If enforceUniqueChainID is True and chain with
+    // the same first letter in chain ID already exists in the model,
+    // then the new chain ID will be appended with a serial number
+    // in order to keep it unique. The model will contain chains like
+    // A, A0, A1, A2, ... in such cases.
+    PCChain GetChainCreate ( const ChainID chID,
+                             Boolean enforceUniqueChainID );
 
     //   CreateChain() creates a new chain with chain ID regardless
     // the presence of same-ID chains in the model. This function
@@ -566,12 +669,12 @@ class CModel : public CProModel  {
     // strictly discouraged.
     PCChain CreateChain    ( const ChainID chID );
 
-    pstr   GetEntryID ();
+    cpstr  GetEntryID ();
     void   SetEntryID ( const IDCode idCode );
 
     int    GetSerNum  (); // returns the model's serial number
 
-    pstr   GetModelID ( pstr modelID );  // returns "/mdl"
+    cpstr  GetModelID ( pstr modelID );  // returns "/mdl"
 
     int    GetNumberOfModels  (); // returns TOTAL number of models
     int    GetNumberOfAtoms   ( Boolean countTers ); // returns number
@@ -601,11 +704,15 @@ class CModel : public CProModel  {
     int  DeleteAllChains    ();
     int  DeleteSolventChains();
     void TrimChainTable     ();
-    
+
     //  -------------------  Adding chains  ---------------------------
 
     int  AddChain ( PCChain chain );
 
+    //  --------------------  Sort chains  ----------------------------
+
+    void SortChains ( int sortKey ); // SORT_CHAIN_XXXX
+
     //  ----------------  Extracting residues  ------------------------
 
     int GetNumberOfResidues ( const ChainID chainID );
@@ -880,6 +987,15 @@ class CModel : public CProModel  {
     void   RemoveLinks();
     void   AddLink    ( PCLink Link );
 
+    //  ----  Working Refmac Links
+
+    int    GetNumberOfLinkRs ();
+    PCLinkR          GetLinkR ( int serialNum ); // 1<=serNum<=NofLinks
+    PCLinkRContainer GetLinkRs() { return &LinkRs; }
+
+    void   RemoveLinkRs();
+    void   AddLinkR   ( PCLinkR LinkR );
+
 
     //  ----  Working CisPeps
 
@@ -892,7 +1008,7 @@ class CModel : public CProModel  {
 
 
 
-    void  ApplyTransform    ( mat44 & TMatrix );  // transforms all
+    void  ApplyTransform ( mat44 & TMatrix );  // transforms all
                                       // coordinates by multiplying
                                       // with matrix TMatrix
 
@@ -910,7 +1026,12 @@ class CModel : public CProModel  {
     int   GetUDData ( int UDDhandle, pstr     & sudd );
 
 
-    void  Copy ( PCModel Model );
+    void  Copy             ( PCModel Model );
+    void  CopyHets         ( PCModel Model );
+    void  CopySecStructure ( PCModel Model );
+    void  CopyLinks        ( PCModel Model );
+    void  CopyLinkRs       ( PCModel Model );
+    void  CopyCisPeps      ( PCModel Model );
 
     void  write ( RCFile f );
     void  read  ( RCFile f );
@@ -925,6 +1046,7 @@ class CModel : public CProModel  {
     CSheets          Sheets;       // information on sheets
     CSSContainer     Turns;        // information on turns
     CLinkContainer   Links;        // information on links
+    CLinkRContainer  LinkRs;       // information on refmac links
     CCisPepContainer CisPeps;      // information on cispeps
 
     int              nChains;      // number of chains
