@@ -1,4 +1,3 @@
-//  $Id: mmdb_sbase0.cpp,v 1.12.2.1 2009/07/01 10:52:19 ccb Exp $
 //  =================================================================
 //
 //   CCP4 Coordinate Library: support of coordinate-related
@@ -6,13 +5,13 @@
 //
 //   Copyright (C) Eugene Krissinel 2000-2008.
 //
-//    This library is free software: you can redistribute it and/or 
-//    modify it under the terms of the GNU Lesser General Public 
-//    License version 3, modified in accordance with the provisions 
+//    This library is free software: you can redistribute it and/or
+//    modify it under the terms of the GNU Lesser General Public
+//    License version 3, modified in accordance with the provisions
 //    of the license to address the requirements of UK law.
 //
-//    You should have received a copy of the modified GNU Lesser 
-//    General Public License along with this library. If not, copies 
+//    You should have received a copy of the modified GNU Lesser
+//    General Public License along with this library. If not, copies
 //    may be downloaded from http://www.ccp4.ac.uk/ccp4license.php
 //
 //    This program is distributed in the hope that it will be useful,
@@ -22,7 +21,7 @@
 //
 //  =================================================================
 //
-//    08.07.08   <--  Date of Last Modification.
+//    29.01.10   <--  Date of Last Modification.
 //                   ~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 //  -----------------------------------------------------------------
 //
@@ -34,7 +33,7 @@
 //                  CSBStructure ( SB structure (monomer) class )
 //                  CSBIndex     ( SB index class               )
 //
-//   (C) E. Krissinel 2000-2008
+//   (C) E. Krissinel 2000-2010
 //
 //  =================================================================
 //
@@ -683,7 +682,7 @@ Boolean done;
                     k    = j;
                     done = True;
                   }
-                } 
+                }
               }
             }
             j++;
@@ -738,7 +737,7 @@ int  superpose_atoms ( mat44 & T, PPCSBAtom A1, PPCAtom A2, int nAtoms,
 // superposes exactly the atoms A1[0] and A2[0].
 realtype det,B;
 vect3    vc1,vc2;
- int      i,j,k;
+int      i,j,k;
 
   //  1.  Calculate the correlation matrix. The rotation will be
   //      done around
@@ -761,7 +760,7 @@ vect3    vc1,vc2;
 
   //  2. Calculate transformation matrix (to be applied to A1)
 
-  det = A[1][1]*A[2][2]*A[3][3] + 
+  det = A[1][1]*A[2][2]*A[3][3] +
         A[1][2]*A[2][3]*A[3][1] +
         A[2][1]*A[3][2]*A[1][3] -
         A[1][3]*A[2][2]*A[3][1] -
@@ -1320,11 +1319,7 @@ PPCSBIndex Index1;
     dirpath = new char[i+10];
     strcpy ( dirpath,path );
     if (i>0)  {
-#if defined(_MSC_VER) || defined (WIN32)
-      if (dirpath[i-1]!='\\')  strcat ( dirpath,"\\" );
-#else
       if (dirpath[i-1]!='/')  strcat ( dirpath,"/" );
-#endif
     }
   }
 
@@ -1402,7 +1397,7 @@ int            structNo,i,k,RC;
       } else
         RC = SBASE_FileNotFound;
     }
-  } else 
+  } else
     RC = SBASE_StructNotFound;
 
   if (RC==SBASE_Ok)  {
@@ -1486,13 +1481,23 @@ int MakeElementType ( int ElType, char chirality, Boolean Cflag )  {
   return ElType;
 }
 
-
 int CSBase0::GetStructNo ( cpstr compoundID )  {
 int  l1,l2,l,k;
 char id[20];
 
-  l = k = 0; // otherwise l and k might be undefined
+  l = k = 0; // initialize, else may become undefined
+
   strcpy_css ( id,compoundID );
+
+  if (nStructures<=3)  {
+    k = -1;
+    for (l=0;(l<nStructures) && (k<0);l++)
+      if (!strcasecmp(Index[l]->compoundID,id))
+        k = l;
+    if (k>=0)  return k;
+    return SBASE_StructNotFound;
+  }
+
   l1 = 0;
   l2 = nStructures-1;
   while (l1<l2-1)  {
@@ -1505,13 +1510,16 @@ char id[20];
       l2 = l;
     }
   }
+
   if (k==0)  return l;
   else if (l==l1)  {
     if (!strcasecmp(Index[l2]->compoundID,id))  return l2;
   } else if (l==l2)  {
     if (!strcasecmp(Index[l1]->compoundID,id))  return l1;
   }
+
   return SBASE_StructNotFound;
+
 }
 
 PCFile CSBase0::GetStructFile()  {
@@ -1625,7 +1633,7 @@ int           structNo;
 }
 
 
-PCResidue CSBase0::makeCResidue ( cpstr compoundID, 
+PCResidue CSBase0::makeCResidue ( cpstr compoundID,
                                   PCFile     structFile,
                                   Boolean    includeHydrogens,
                                   Boolean    makeTer )  {
@@ -1640,7 +1648,7 @@ PCResidue     Res;
     return NULL;
 }
 
-PCResidue CSBase0::makeCResidue ( int     structNo, 
+PCResidue CSBase0::makeCResidue ( int     structNo,
                                   PCFile  structFile,
                                   Boolean includeHydrogens,
                                   Boolean makeTer )  {
@@ -2158,7 +2166,7 @@ int           i,j,rc,natoms;
     } else
       rc =  SBASE_NoAtomsFound;
     delete SBS;
-  } else  
+  } else
     rc = SBASE_StructNotFound;
 
   return rc;
@@ -2210,7 +2218,7 @@ int           i,j,k,n,natoms;
                     }
                 }
             }
-          } 
+          }
         }
         j++;
         while (j<nRes)
@@ -2250,7 +2258,7 @@ PPCResidue    Res;
 PPCChain      chain;
 PCMMDBManager MMDB;
 int           rc,selHnd,i,nRes,nChains;
- 
+
   rc   = SBASE_Ok;
   MMDB = PCMMDBManager(model->GetCoordHierarchy());
 
@@ -2573,7 +2581,7 @@ Boolean       complH,complNH,complXT;
           aname = A1[i]->GetAtomName();
           if (strcmp(A1[i]->GetElementName(),Hydrogen))  {
             //  a non-hydrogen atom, check if it should be added
-            if (complNH &&  
+            if (complNH &&
                      (complXT || (strcmp(aname," OXT"))))
               A3[k] = A1[i];
           } else if (complH)  {
@@ -2886,7 +2894,7 @@ PCFile        structFile;
     SBS = ldStructure[Index[structNo]->loadPos];
 
   } else  {
-  
+
     structFile = GetStructFile();
     if (!structFile)  return SBASE_FileNotFound;
 
@@ -2946,7 +2954,7 @@ PCFile        structFile;
     }
     j = 0;
     while ((j<nHAtoms) && (Hconnect[j]>=0)) j++;
-    if (j<nHAtoms) 
+    if (j<nHAtoms)
       rc = SBASE_ConnectivityError;
   }
 
@@ -3015,7 +3023,7 @@ int     structNo,i, a1,a2;
       }
     }
   }
-  
+
   if (Index[structNo]->loadPos<0)  delete G;
 
   return SBASE_Ok;
@@ -3094,3 +3102,4 @@ int           i,structNo;
   return SBASE_Ok;
 
 }
+
