@@ -1,4 +1,3 @@
-//  $Id: mmdb_symop.cpp,v 1.18 2008/08/21 09:54:09 nds65 Exp $
 //  =================================================================
 //
 //   CCP4 Coordinate Library: support of coordinate-related
@@ -6,13 +5,13 @@
 //
 //   Copyright (C) Eugene Krissinel 2000-2008.
 //
-//    This library is free software: you can redistribute it and/or 
-//    modify it under the terms of the GNU Lesser General Public 
-//    License version 3, modified in accordance with the provisions 
+//    This library is free software: you can redistribute it and/or
+//    modify it under the terms of the GNU Lesser General Public
+//    License version 3, modified in accordance with the provisions
 //    of the license to address the requirements of UK law.
 //
-//    You should have received a copy of the modified GNU Lesser 
-//    General Public License along with this library. If not, copies 
+//    You should have received a copy of the modified GNU Lesser
+//    General Public License along with this library. If not, copies
 //    may be downloaded from http://www.ccp4.ac.uk/ccp4license.php
 //
 //    This program is distributed in the hope that it will be useful,
@@ -22,7 +21,7 @@
 //
 //  =================================================================
 //
-//    08.07.08   <--  Date of Last Modification.
+//    29.01.10   <--  Date of Last Modification.
 //                   ~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 //  -----------------------------------------------------------------
 //
@@ -32,9 +31,9 @@
 //       ~~~~~~~~~
 //
 //  **** Classes :   CSymOp  ( symmetry operator )
-//       ~~~~~~~~~   
+//       ~~~~~~~~~
 //
-//   (C) E. Krissinel 2000-2008
+//   (C) E. Krissinel 2000-2010
 //
 //  =================================================================
 //
@@ -200,7 +199,7 @@ realtype V;
   } while (p1);
 
   DelSpaces ( L );
-  if (strlen(L)>=len)  return SYMOP_NotAnOperation;
+  if ((int)strlen(L)>=len)  return SYMOP_NotAnOperation;
 
   // translational part
   p1 = L;
@@ -338,7 +337,7 @@ realtype AV;
                                                       10,pstr("*") );
 
   strcat ( S,Axis );
-    
+
 }
 
 */
@@ -440,7 +439,7 @@ byte Version=1;
     for (j=0;j<4;j++)
       f.WriteReal ( &(T[i][j]) );
 }
-   
+
 void  CSymOp::read ( RCFile f )  {
 int  i,j;
 byte Version;
@@ -557,7 +556,7 @@ int   i,RC;
 
 #define  syminfo_file  cpstr("syminfo.lib")
 
-int  CSymOps::SetGroup ( cpstr SpaceGroup, 
+int  CSymOps::SetGroup ( cpstr SpaceGroup,
                          cpstr syminfo_lib )  {
 CFile f;
 pstr  p;
@@ -569,6 +568,8 @@ int   i,j,k,l,m,RC;
 int   npops,ncops;
 long  symop_start;
 
+  symop_start = 0;  // else may be used uninitialized 
+
   FreeMemory();
 
   npops = 0;
@@ -777,7 +778,7 @@ byte Version=1;
   for (i=0;i<Nops;i++)
     StreamWrite ( f,SymOp[i] );
 }
-   
+
 void  CSymOps::read ( RCFile f )  {
 int  i;
 byte Version;
