@@ -1,4 +1,3 @@
-//  $Id: mmdb_tables.h,v 1.19.2.1 2009/09/28 11:00:27 ccb Exp $
 //  =================================================================
 //
 //   CCP4 Coordinate Library: support of coordinate-related
@@ -6,13 +5,13 @@
 //
 //   Copyright (C) Eugene Krissinel 2000-2008.
 //
-//    This library is free software: you can redistribute it and/or 
-//    modify it under the terms of the GNU Lesser General Public 
-//    License version 3, modified in accordance with the provisions 
+//    This library is free software: you can redistribute it and/or
+//    modify it under the terms of the GNU Lesser General Public
+//    License version 3, modified in accordance with the provisions
 //    of the license to address the requirements of UK law.
 //
-//    You should have received a copy of the modified GNU Lesser 
-//    General Public License along with this library. If not, copies 
+//    You should have received a copy of the modified GNU Lesser
+//    General Public License along with this library. If not, copies
 //    may be downloaded from http://www.ccp4.ac.uk/ccp4license.php
 //
 //    This program is distributed in the hope that it will be useful,
@@ -31,8 +30,8 @@
 //  **** Project :   MacroMolecular Data Base (MMDB)
 //       ~~~~~~~~~
 //
-//  **** Functions : 
-//       ~~~~~~~~~~~ 
+//  **** Functions :
+//       ~~~~~~~~~~~
 //
 //  **** Constants : AName  ( array of 2-character atom names       )
 //       ~~~~~~~~~~~ HAName ( array of 2=character heteroatom names )
@@ -102,7 +101,9 @@ struct SAAProperty  {
 };
 
 extern SAAProperty const AAProperty[nAminoacidNames];
+extern int const AASimilarity[nAminoacidNames][nAminoacidNames];
 
+extern int      GetAAPIndex     ( cpstr resName );  // 0..nAminoacidNames-1
 extern realtype GetAAHydropathy ( cpstr resName );  // -4.5...+4.5
 extern realtype GetAACharge     ( cpstr resName );
 extern realtype GetAASolvationEnergy ( cpstr resName );
