@@ -1,4 +1,3 @@
-//  $Id: mmdb_title.cpp,v 1.16.2.3 2010/01/16 13:30:53 rmk65 Exp $
 //  =================================================================
 //
 //   CCP4 Coordinate Library: support of coordinate-related
@@ -6,13 +5,13 @@
 //
 //   Copyright (C) Eugene Krissinel 2000-2008.
 //
-//    This library is free software: you can redistribute it and/or 
-//    modify it under the terms of the GNU Lesser General Public 
-//    License version 3, modified in accordance with the provisions 
+//    This library is free software: you can redistribute it and/or
+//    modify it under the terms of the GNU Lesser General Public
+//    License version 3, modified in accordance with the provisions
 //    of the license to address the requirements of UK law.
 //
-//    You should have received a copy of the modified GNU Lesser 
-//    General Public License along with this library. If not, copies 
+//    You should have received a copy of the modified GNU Lesser
+//    General Public License along with this library. If not, copies
 //    may be downloaded from http://www.ccp4.ac.uk/ccp4license.php
 //
 //    This program is distributed in the hope that it will be useful,
@@ -22,14 +21,14 @@
 //
 //  =================================================================
 //
-//    17.03.09   <--  Date of Last Modification.
+//    29.01.10   <--  Date of Last Modification.
 //                   ~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 //  -----------------------------------------------------------------
 //
 //  **** Module  :  MMDB_Title  <implementation>
 //       ~~~~~~~~~
 //  **** Project :  MacroMolecular Data Base (MMDB)
-//       ~~~~~~~~~ 
+//       ~~~~~~~~~
 //  **** Classes :  CTitleContainer  (container for title classes)
 //       ~~~~~~~~~  CObsLine
 //                  CTitleLine
@@ -46,7 +45,7 @@
 //                  CBiomolecule
 //                  CMMDBTitle  ( MMDB title section )
 //
-//  (C) E. Krissinel 2000-2009
+//  (C) E. Krissinel 2000-2010
 //
 //  =================================================================
 //
@@ -75,10 +74,10 @@ PCContainerClass CTitleContainer::MakeContainerClass ( int ClassID )  {
     default :
     case ClassID_Template  : return
                          CClassContainer::MakeContainerClass(ClassID);
-    case ClassID_ObsLine   : return new CObsLine  ();      
-    case ClassID_CAVEAT    : return new CCaveat   ();      
-    case ClassID_TitleLine : return new CTitleLine();     
-    case ClassID_Compound  : return new CCompound ();      
+    case ClassID_ObsLine   : return new CObsLine  ();
+    case ClassID_CAVEAT    : return new CCaveat   ();
+    case ClassID_TitleLine : return new CTitleLine();
+    case ClassID_Compound  : return new CCompound ();
     case ClassID_Source    : return new CSource   ();
     case ClassID_ExpData   : return new CExpData  ();
     case ClassID_Author    : return new CAuthor   ();
@@ -130,7 +129,7 @@ int i;
     strncpy ( &(S[31+5*i]),rIdCode[i],4 );
 }
 
-void  CObsLine::MakeCIF ( PCMMCIFData CIF, int N )  {
+void  CObsLine::MakeCIF ( PCMMCIFData CIF, int )  {
 PCMMCIFLoop Loop;
 int         RC,i,j;
 char        DateCIF[20];
@@ -213,7 +212,7 @@ IDCode      idCode1;
         Loop->DeleteField ( CIFTAG_REPLACE_PDB_ID,Signal );
         Loop->DeleteField ( CIFTAG_PDB_ID        ,Signal );
         i++;
-      } 
+      }
     }
     Signal++;
   } while (i<8);
@@ -226,7 +225,7 @@ int i;
   for (i=0;i<8;i++)
     strcpy ( rIdCode[i],PCObsLine(ObsLine)->rIdCode[i] );
 }
-    
+
 void  CObsLine::write ( RCFile f )  {
 int  i;
 byte Version=1;
@@ -329,7 +328,7 @@ int  CCaveat::ConvertPDBASCII ( cpstr S )  {
   if (strlen(S)>12)  {
     strncpy ( idCode,&(S[11]),4 );
     idCode[4] = char(0);
-    if (strlen(S)>19) 
+    if (strlen(S)>19)
           CreateCopy ( Line,&(S[19])  );
     else  CreateCopy ( Line,pstr(" ") );
   } else
@@ -558,7 +557,7 @@ pstr * KW;
   for (i=nKeyWords;i<k;i++)
     KW[i] = NULL;       // null new pointers
   KeyWord = KW;
-  
+
   i = 10;
   if (Cont)  nKeyWords--;
   while (S[i] && (i<70))  {
@@ -595,7 +594,7 @@ char c;
   if (KeyWord)  {
     N = 0;
     i = 0;
-    while (i<nKeyWords)  {  
+    while (i<nKeyWords)  {
       if (N==0)  strcpy  ( S,"KEYWDS    " );
            else  sprintf ( S,"KEYWDS  %2i ",N+1 );
       do  {
@@ -649,7 +648,7 @@ char S[500];
       k = strlen(KeyWord[i]);
       if (strlen(S)+k>70)  {
         CIF->PutString ( S,CIFCAT_STRUCT_KEYWORDS,CIFTAG_TEXT,True );
-        if (k>sizeof(S))  {
+        if (k>(int)sizeof(S))  {
           CIF->PutString ( KeyWord[i],CIFCAT_STRUCT_KEYWORDS,
                            CIFTAG_TEXT,True );
           k = 0;
@@ -730,7 +729,7 @@ byte Version=1;
   f.WriteByte ( &Version   );
   f.WriteInt  ( &nKeyWords );
   for (i=0;i<nKeyWords;i++)
-    f.CreateWrite ( KeyWord[i] );  
+    f.CreateWrite ( KeyWord[i] );
 }
 
 void  CKeyWords::read ( RCFile f )  {
@@ -840,13 +839,13 @@ void  CAuthor::PDBASCIIDump ( pstr S, int N )  {
 void  CAuthor::write ( RCFile f )  {
 byte Version=1;
   f.WriteByte ( &Version );
-  CContString::write ( f );  
+  CContString::write ( f );
 }
 
 void  CAuthor::read ( RCFile f )  {
 byte Version;
   f.ReadByte ( &Version );
-  CContString::read ( f );  
+  CContString::read ( f );
 }
 
 MakeStreamFunctions(CAuthor)
@@ -873,7 +872,7 @@ CRevData::~CRevData() {}
 void  CRevData::InitRevData()  {
 int i;
   modNum  = 0;
-  strcpy ( modDate,"DD-MMM-YYYY" ); 
+  strcpy ( modDate,"DD-MMM-YYYY" );
   strcpy ( modId  , "----" );
   modType = -1;
   for (i=0;i<4;i++)
@@ -971,7 +970,7 @@ pstr        F;
   if ((!RC) && F)  strcpy ( record[2],F );
   F = Loop->GetString ( CIFTAG_RCSB_RECORD_REVISED_4,Signal,RC );
   if ((!RC) && F)  strcpy ( record[3],F );
-  
+
   Loop->DeleteField ( CIFTAG_DATE                 ,Signal );
   Loop->DeleteField ( CIFTAG_REPLACES             ,Signal );
   Loop->DeleteField ( CIFTAG_RCSB_RECORD_REVISED_1,Signal );
@@ -1013,7 +1012,7 @@ int i;
   for (i=0;i<4;i++)
     strcpy ( record[i],PCRevData(RevData)->record[i] );
 }
-    
+
 void  CRevData::write ( RCFile f )  {
 int  i;
 byte Version=1;
@@ -1087,7 +1086,7 @@ int i;
     strncpy ( &(S[31+5*i]),sIdCode[i],4 );
 }
 
-void  CSupersede::MakeCIF ( PCMMCIFData CIF, int N )  {
+void  CSupersede::MakeCIF ( PCMMCIFData CIF, int )  {
 PCMMCIFLoop Loop;
 int         RC,i,j;
 char        DateCIF[20];
@@ -1173,7 +1172,7 @@ IDCode      idCode1;
         Loop->DeleteField ( CIFTAG_REPLACE_PDB_ID,Signal );
         Loop->DeleteField ( CIFTAG_PDB_ID        ,Signal );
         i++;
-      } 
+      }
     }
     Signal++;
   } while (i<8);
@@ -1186,7 +1185,7 @@ int i;
   for (i=0;i<8;i++)
     strcpy ( sIdCode[i],PCSupersede(Supersede)->sIdCode[i] );
 }
-    
+
 void  CSupersede::write ( RCFile f )  {
 int  i;
 byte Version=1;
@@ -1239,7 +1238,7 @@ int  CJournal::ConvertPDBASCII ( cpstr S )  {
   return 0;
 }
 
-void  CJournal::PDBASCIIDump ( pstr S, int N )  {
+void  CJournal::PDBASCIIDump ( pstr S, int )  {
   strcpy ( S,"JRNL      " );
   strcat ( S,Line         );
 }
@@ -1296,14 +1295,14 @@ int i;
   return 0;
 }
 
-void  CRemark::PDBASCIIDump ( pstr S, int N )  {
+void  CRemark::PDBASCIIDump ( pstr S, int )  {
   if (remarkNum==MinInt4)
     strcpy ( S,Remark );
   else  {
     strcpy     ( S,"REMARK" );
     PadSpaces  ( S,80 );
     PutInteger ( &(S[7]) ,remarkNum,3 );
-    strncpy    ( &(S[11]),Remark,IMin(68,strlen(Remark)) ); 
+    strncpy    ( &(S[11]),Remark,IMin(68,strlen(Remark)) );
   }
 }
 
@@ -1381,10 +1380,6 @@ MakeStreamFunctions(CRemark)
 #define  R350_CHAINS         2
 #define  R350_BIOMT          3
 
-#ifdef _WIN32
-#define  strcasestr(s, t) strstr(strupr(s), t)
-#endif
-
 void getRemarkKey ( RPCRemark rem, int & lkey )  {
   if (rem)  {
     if (rem->remarkNum!=350)  lkey = R350_END;
@@ -1469,7 +1464,7 @@ int      l,lkey,nAdd,j;
       while (*p==' ')  p++;
       if ((p[1]!=',') && (p[1]!=' '))  p = NULL;
     }
- 
+
     if (p)  {
       nAdd  = strlen(p)/2 + 3;
       ch1   = chain;
@@ -1561,7 +1556,7 @@ int  l,j,lkey;
 int  CBMApply::addMatrices ( int & i, RPCRemark rem,
                              RCTitleContainer Remark )  {
 pmat44 tm1;
-int    l,lkey,nAdd,j,k1,k2,nAlloc;
+int    l,lkey,j,k1,k2,nAlloc;
 
   l      = Remark.Length();
   lkey   = R350_BIOMT;
@@ -1741,7 +1736,7 @@ Boolean cmp;
 
 void  CBiomolecule::Copy ( PCBiomolecule B )  {
 // if B is NULL, then empties the class
-int  i,j,k;
+int  i;
 
   FreeMemory();
 
@@ -1796,7 +1791,7 @@ CMMDBTitle::CMMDBTitle ( RPCStream Object )  : CStream(Object)  {
 }
 
 void  CMMDBTitle::Init()  {
-  
+
   //  Header data
   classification = NULL;
   depDate[0]     = char(0);
@@ -1861,7 +1856,7 @@ void  CMMDBTitle::SetHeader ( cpstr Classification,
   depDate[sizeof(depDate)-1] = char(0);
   idCode [sizeof(idCode) -1] = char(0);
 }
-    
+
 int  CMMDBTitle::ConvertPDBString ( pstr PDBString ) {
 // Interprets the ASCII PDB line belonging to the title section
 // and fills the corresponding fields.
@@ -1894,7 +1889,7 @@ PCContainerClass ContainerClass;
 
     ContainerClass = new CObsLine(PDBString);
     ObsData.AddData ( ContainerClass );
-    
+
   } else if (!strncmp(PDBString,"TITLE ",6))  {
 
     ContainerClass = new CTitleLine(PDBString);
@@ -1904,7 +1899,7 @@ PCContainerClass ContainerClass;
 
     ContainerClass = new CCaveat(PDBString);
     CAVEAT.AddData ( ContainerClass );
-    
+
   } else if (!strncmp(PDBString,"COMPND",6))  {
 
     ContainerClass = new CCompound(PDBString);
@@ -1914,47 +1909,51 @@ PCContainerClass ContainerClass;
 
     ContainerClass = new CSource(PDBString);
     Source.AddData ( ContainerClass );
-  
+
   } else if (!strncmp(PDBString,"KEYWDS",6))  {
-   
+
     KeyWords.ConvertPDBASCII ( PDBString );
 
   } else if (!strncmp(PDBString,"EXPDTA",6))  {
 
     ContainerClass = new CExpData(PDBString);
     ExpData.AddData ( ContainerClass );
-  
+
   } else if (!strncmp(PDBString,"AUTHOR",6))  {
 
     ContainerClass = new CAuthor(PDBString);
     Author.AddData ( ContainerClass );
-  
+
   } else if (!strncmp(PDBString,"REVDAT",6))  {
 
     ContainerClass = new CRevData(PDBString);
     RevData.AddData ( ContainerClass );
-  
+
   } else if (!strncmp(PDBString,"SPRSDE",6))  {
 
     ContainerClass = new CSupersede(PDBString);
     Supersede.AddData ( ContainerClass );
-  
+
   } else if (!strncmp(PDBString,"JRNL  ",6))  {
 
     ContainerClass = new CJournal(PDBString);
     Journal.AddData ( ContainerClass );
-  
+
   } else if (!strncmp(PDBString,"REMARK",6))  {
 
     ContainerClass = new CRemark(PDBString);
     Remark.AddData ( ContainerClass );
-  
+
   } else
     return Error_WrongSection;
 
   //  check for ID code in columns 73-80
-  if ((!col73) && (!strncasecmp(idCode,&(PDBString[72]),4)))
-    col73 = True;
+  if (!col73)  {
+    if (('0'<=idCode[0]) && (idCode[0]<='9'))  {
+      if (!strncasecmp(idCode,&(PDBString[72]),4))
+        col73 = True;
+    }
+  }
 
   return  0;
 
@@ -2010,7 +2009,6 @@ int CMMDBTitle::ParseBiomolecules()  {
 PCRemark       rem;
 PCBiomolecule  BMol;
 PCBMApply      BMA;
-pstr           p,eptr;
 int            i,l, lkey;
 
   FreeBiomolecules();
@@ -2137,13 +2135,13 @@ int i;
     strcpy ( PDBString,"HEADER    " );
     strcat ( PDBString,classification );
     i = strlen(PDBString);
-    while (i<80) 
+    while (i<80)
       PDBString[i++] = ' ';
     PDBString[IMin(i,80)] = char(0);
     Date11to9 ( depDate,&(PDBString[50]) );
     strncpy   ( &(PDBString[62]),idCode,4 );
 
-  } else  
+  } else
     strcpy ( PDBString,
       "HEADER    XXXXXXXXXXXXXXXXXXXXXXXXXXXX            XX-XXX-XX   ----" );
 
@@ -2307,7 +2305,7 @@ char DateCIF[20];
     CIF->PutString ( DateCIF,CIFCAT_DATABASE,CIFTAG_DATE_ORIGINAL );
   } else
     CIF->PutString ( NULL,CIFCAT_DATABASE,CIFTAG_DATE_ORIGINAL );
-  
+
   ObsData  .MakeCIF ( CIF );
   Title    .MakeCIF ( CIF );
   CAVEAT   .MakeCIF ( CIF );
@@ -2415,7 +2413,7 @@ byte Version=2;
   f.WriteInt ( &nBiomolecules );
   for (i=0;i<nBiomolecules;i++)
     StreamWrite ( f,Biomolecule[i] );
-  
+
 }
 
 void  CMMDBTitle::read ( RCFile f )  {
@@ -2528,7 +2526,7 @@ PCMMDBTitle  Title;
     delete Title;
     return;
   }
-   
+
   f.assign ( pstr("out.title"),True );
   if (f.rewrite()) {
     Title->PDBASCIIDump ( f );
@@ -2550,10 +2548,10 @@ PCMMDBTitle  Title;
     delete Title;
     return;
   }
-  
+
   delete Title;
   printf ( "   Title deleted.\n" );
-  
+
   Title = new CMMDBTitle();
   if (f.reset()) {
     Title->read ( f );
@@ -2563,7 +2561,7 @@ PCMMDBTitle  Title;
     delete Title;
     return;
   }
-   
+
   f.assign ( pstr("abin.title"),True );
   if (f.rewrite()) {
     Title->PDBASCIIDump ( f );
