@@ -1,4 +1,3 @@
-//  $Id: mmdb_title.h,v 1.15.2.1 2009/09/28 11:00:27 ccb Exp $
 //  =================================================================
 //
 //   CCP4 Coordinate Library: support of coordinate-related
@@ -6,13 +5,13 @@
 //
 //   Copyright (C) Eugene Krissinel 2000-2008.
 //
-//    This library is free software: you can redistribute it and/or 
-//    modify it under the terms of the GNU Lesser General Public 
-//    License version 3, modified in accordance with the provisions 
+//    This library is free software: you can redistribute it and/or
+//    modify it under the terms of the GNU Lesser General Public
+//    License version 3, modified in accordance with the provisions
 //    of the license to address the requirements of UK law.
 //
-//    You should have received a copy of the modified GNU Lesser 
-//    General Public License along with this library. If not, copies 
+//    You should have received a copy of the modified GNU Lesser
+//    General Public License along with this library. If not, copies
 //    may be downloaded from http://www.ccp4.ac.uk/ccp4license.php
 //
 //    This program is distributed in the hope that it will be useful,
@@ -22,7 +21,7 @@
 //
 //  =================================================================
 //
-//    17.03.09   <--  Date of Last Modification.
+//    29.01.10   <--  Date of Last Modification.
 //                   ~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 //  -----------------------------------------------------------------
 //
@@ -46,7 +45,7 @@
 //                  CBiomolecule
 //                  CMMDBTitle       ( MMDB title section )
 //
-//   (C) E. Krissinel 2000-2009
+//   (C) E. Krissinel 2000-2010
 //
 //  =================================================================
 //
@@ -83,7 +82,7 @@ class CTitleContainer : public CClassContainer  {
 
     CTitleContainer  () : CClassContainer() {}
     CTitleContainer  ( RPCStream Object )
-                        : CClassContainer ( Object ) {} 
+                        : CClassContainer ( Object ) {}
     ~CTitleContainer () {}
 
     PCContainerClass MakeContainerClass ( int ClassID );
@@ -121,7 +120,7 @@ class CObsLine : public CContainerClass  {
     void  read  ( RCFile f );
 
   protected :
-    
+
     void InitObsLine();
 
 };
@@ -143,7 +142,7 @@ class CTitleLine : public CContString  {
 
     int   ConvertPDBASCII ( cpstr S );
     void  PDBASCIIDump    ( pstr S, int N );
-    Boolean PDBASCIIDump1 ( RCFile f ) { return False; }
+    Boolean PDBASCIIDump1 ( RCFile ) { return False; }
     int   GetClassID      () { return ClassID_TitleLine; }
 
 //    void  GetCIF          ( PCMMCIFData CIF, int & Signal );
@@ -177,7 +176,7 @@ class CCaveat : public CContString  {
     ~CCaveat();
 
     void  PDBASCIIDump    ( pstr S, int N );
-    Boolean PDBASCIIDump1 ( RCFile f ) { return False; }
+    Boolean PDBASCIIDump1 ( RCFile ) { return False; }
     void  MakeCIF         ( PCMMCIFData CIF, int N );
     int   ConvertPDBASCII ( cpstr S );
     void  GetCIF          ( PCMMCIFData CIF, int & Signal );
@@ -210,7 +209,7 @@ class CCompound : public CContString  {
     ~CCompound();
 
     void  PDBASCIIDump    ( pstr S, int N );
-    Boolean PDBASCIIDump1 ( RCFile f ) { return False; }
+    Boolean PDBASCIIDump1 ( RCFile  ) { return False; }
     int   ConvertPDBASCII ( cpstr S );
     int   GetClassID      () { return ClassID_Compound; }
 
@@ -222,7 +221,7 @@ class CCompound : public CContString  {
     void  read  ( RCFile f );
 
   protected :
-    
+
     void InitCompound();
 
 };
@@ -243,7 +242,7 @@ class CSource : public CContString  {
     ~CSource();
 
     void  PDBASCIIDump    ( pstr S, int N );
-    Boolean PDBASCIIDump1 ( RCFile f ) { return False; }
+    Boolean PDBASCIIDump1 ( RCFile  ) { return False; }
     int   ConvertPDBASCII ( cpstr S );
     int   GetClassID      () { return ClassID_Source; }
 
@@ -315,7 +314,7 @@ class CExpData : public CContString  {
     ~CExpData();
 
     void  PDBASCIIDump    ( pstr S, int N );
-    Boolean PDBASCIIDump1 ( RCFile f ) { return False; }
+    Boolean PDBASCIIDump1 ( RCFile ) { return False; }
 
     int   ConvertPDBASCII ( cpstr S );
     int   GetClassID      () { return ClassID_ExpData; }
@@ -328,7 +327,7 @@ class CExpData : public CContString  {
     void  read  ( RCFile f );
 
   protected :
-    
+
     void InitExpData();
 
 };
@@ -349,7 +348,7 @@ class CAuthor : public CContString  {
     ~CAuthor();
 
     void  PDBASCIIDump    ( pstr S, int N   );
-    Boolean PDBASCIIDump1 ( RCFile f ) { return False; }
+    Boolean PDBASCIIDump1 ( RCFile ) { return False; }
 
     int   ConvertPDBASCII ( cpstr S );
     int   GetClassID      () { return ClassID_Author; }
@@ -444,7 +443,7 @@ class CSupersede : public CContainerClass  {
     void  read  ( RCFile f );
 
   protected :
-    
+
     void InitSupersede();
 
 };
@@ -465,7 +464,7 @@ class CJournal : public CContString  {
     ~CJournal();
 
     void  PDBASCIIDump    ( pstr S, int N );
-    Boolean PDBASCIIDump1 ( RCFile f ) { return False; }
+    Boolean PDBASCIIDump1 ( RCFile ) { return False; }
 
     int   ConvertPDBASCII ( cpstr S );
     int   GetClassID      () { return ClassID_Journal; }
