@@ -1,4 +1,3 @@
-//  $Id: mmdb_utils.cpp,v 1.26 2008/08/21 09:54:10 nds65 Exp $
 //  =================================================================
 //
 //   CCP4 Coordinate Library: support of coordinate-related
@@ -6,13 +5,13 @@
 //
 //   Copyright (C) Eugene Krissinel 2000-2008.
 //
-//    This library is free software: you can redistribute it and/or 
-//    modify it under the terms of the GNU Lesser General Public 
-//    License version 3, modified in accordance with the provisions 
+//    This library is free software: you can redistribute it and/or
+//    modify it under the terms of the GNU Lesser General Public
+//    License version 3, modified in accordance with the provisions
 //    of the license to address the requirements of UK law.
 //
-//    You should have received a copy of the modified GNU Lesser 
-//    General Public License along with this library. If not, copies 
+//    You should have received a copy of the modified GNU Lesser
+//    General Public License along with this library. If not, copies
 //    may be downloaded from http://www.ccp4.ac.uk/ccp4license.php
 //
 //    This program is distributed in the hope that it will be useful,
@@ -22,7 +21,7 @@
 //
 //  =================================================================
 //
-//    08.07.06   <--  Date of Last Modification.
+//    29.01.10   <--  Date of Last Modification.
 //                   ~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 //  -----------------------------------------------------------------
 //
@@ -54,13 +53,13 @@
 //                   CIFGetString   (reads and deletes string from CIF)
 //                   CIFGetInteger1 (reads and del-s int from CIF loop)
 //                   CIFGetReal1    (reads and del-s int from CIF loop)
-//                   Mat4Inverse    ( inversion of 4x4 matrices      ) 
+//                   Mat4Inverse    ( inversion of 4x4 matrices      )
 //                   GetErrorDescription (ascii line to an Error_XXXXX)
 //                   ParseAtomID    ( parses atom ID line            )
 //                   ParseResID     ( parses residue ID line         )
 //                   ParseAtomPath  ( parses full atom path          )
 //
-//   (C) E. Krissinel  2000-2008
+//   (C) E. Krissinel  2000-2010
 //
 //  =================================================================
 //
@@ -306,7 +305,7 @@ char L[50];
   if (N==MinInt4)
     for (i=0;i<M;i++)
       S[i] = ' ';
-  else  { 
+  else  {
     sprintf ( L,"%*i",M,N );
     strncpy ( S,L,M );
   }
@@ -324,7 +323,7 @@ char N[50];
   if (R==-MaxReal)
     for (i=0;i<M;i++)
       S[i] = ' ';
-  else  { 
+  else  {
     sprintf ( N,"%*.*f",M,L,R );
     strncpy ( S,N,M );
   }
@@ -375,7 +374,7 @@ pstr F;
                      Loop->GetCategoryName(),Tag,nrow );
     return Error_UnrecognizedInteger;
   }
-  if (RC==CIFRC_WrongIndex) 
+  if (RC==CIFRC_WrongIndex)
     return Error_NoData;
   if (RC)  {
     F = Loop->GetString ( Tag,nrow,RC );
@@ -593,7 +592,7 @@ int      ii,jj,i,i1,j,j1;
 
 // ---- Get inverse matrix
 
-  if (s!=0.0) 
+  if (s!=0.0)
     for (i=0;i<4;i++)
       for (j=0;j<4;j++)
         AI[i][j] = c[j][i]/s;
@@ -633,7 +632,7 @@ int      ii,jj,i,i1,j,j1;
 
   // Get inverse matrix
 
-  if (s!=0.0) 
+  if (s!=0.0)
     for (i=0;i<3;i++)
       for (j=0;j<3;j++)
         AI[i][j] = c[j][i]/s;
@@ -694,7 +693,7 @@ int i,j;
 
 realtype Mat4RotDet ( mat44 & T )  {
 //  returns determinant of the rotation part
-  return T[0][0]*T[1][1]*T[2][2] + 
+  return T[0][0]*T[1][1]*T[2][2] +
          T[0][1]*T[1][2]*T[2][0] +
          T[1][0]*T[2][1]*T[0][2] -
          T[0][2]*T[1][1]*T[2][0] -
@@ -862,9 +861,9 @@ static cpstr msWrongSheetOrder     = "Incomplete _struct_sheet_order.";
 static cpstr msHBondInconsistency  =
   "Inconsistency in _struct_sheet_hbond.";
 
-static cpstr msEmptyResidueName    = 
+static cpstr msEmptyResidueName    =
         "No residue name on PDB ATOM or TER card.";
-static cpstr msDuplicateSeqNum     = 
+static cpstr msDuplicateSeqNum     =
         "Duplicate sequence number and insertion code.";
 
 static cpstr msEmptyFile           = "Non-existent or empty file.";
@@ -880,7 +879,7 @@ cpstr  GetErrorDescription ( int ErrorCode )  {
 
     case Error_WrongSection         :  return msWrongSection;
     case Error_WrongChainID         :  return msWrongChainID;
-    case Error_WrongEntryID         :  return msWrongEntryID; 
+    case Error_WrongEntryID         :  return msWrongEntryID;
 
     case Error_SEQRES_serNum        :  return msSEQRES_serNum;
     case Error_SEQRES_numRes        :  return msSEQRES_numRes;
@@ -985,7 +984,7 @@ int  CContString::ConvertPDBASCII ( cpstr S )  {
   return 0;
 }
 
-void  CContString::PDBASCIIDump ( pstr S, int N )  {
+void  CContString::PDBASCIIDump ( pstr S, int )  {
   if (Line)  strcpy ( S,Line );
        else  strcpy ( S,""   );
 }
@@ -1009,7 +1008,7 @@ char c;
     Signal = -1;
     return;
   }
-  if (Signal>=strlen(F))  {
+  if (Signal>=(int)strlen(F))  {
     CIF->DeleteField ( CIFCategory,CIFTag );
     Signal = -1;
     return;
@@ -1064,9 +1063,9 @@ void  CContString::Copy ( PCContainerClass CString )  {
 void  CContString::write ( RCFile f )  {
 byte Version=1;
   f.WriteByte   ( &Version    );
-  f.CreateWrite ( Line        );  
-  f.CreateWrite ( CIFCategory );  
-  f.CreateWrite ( CIFTag      );  
+  f.CreateWrite ( Line        );
+  f.CreateWrite ( CIFCategory );
+  f.CreateWrite ( CIFTag      );
 }
 
 void  CContString::read ( RCFile f )  {
@@ -1107,7 +1106,7 @@ void  CClassContainer::FreeContainer()  {
 int i;
   if (Container)  {
     for (i=0;i<length;i++)
-      if (Container[i]) 
+      if (Container[i])
         delete Container[i];
     delete[] Container;
   }
@@ -1234,7 +1233,7 @@ byte Version;
       f.ReadInt ( &ClassID );
       if (ClassID>=0)  {
         Container[i] = MakeContainerClass ( ClassID );
-        Container[i]->read ( f ); 
+        Container[i]->read ( f );
       } else
         Container[i] = NULL;
     }
@@ -1353,7 +1352,7 @@ int  i,k;
 
   if (((*p>='0') && (*p<='9')) || (*p=='-') ||
        (*p=='(') || (*p=='.'))  {
-    //  sequence number, residue name and insertion code 
+    //  sequence number, residue name and insertion code
     i = 0;
     while ((*p) && (*p!='/'))  {
       if (*p!=' ')  N[i++] = *p;
@@ -1369,7 +1368,7 @@ int  i,k;
   }
 
   if (strchr(p,':') || strchr(p,'['))  {
-    // atom name, chemical element and alternative location  
+    // atom name, chemical element and alternative location
     i = 0;
     while (*p)  {
       if (*p!=' ')  N[i++] = *p;
@@ -1466,7 +1465,7 @@ int rm = right+1;
     do
       rm--;
     while ((rm>0) && (Compare(rm,lv)>0));
-    do 
+    do
       lm++;
     while ((lm<dlen) && (Compare(lm,lv)<0));
     if (lm<rm)  {
@@ -1500,7 +1499,7 @@ void  takeWord ( pstr & p, pstr wrd, cpstr ter, int l )  {
 pstr p1;
 int  i;
   p1 = strpbrk ( p,ter );
-  if (!p1) 
+  if (!p1)
     p1 = p + strlen(p);
   i = 0;
   while ((p!=p1) && (i<l))  {
@@ -1524,7 +1523,7 @@ pstr p;
   strcpy ( elname,"*" );
   if (*p)  aloc[0] = char(0);
      else  strcpy ( aloc,"*" );
-  
+
   takeWord ( p,aname,pstr("[: "),sizeof(AtomName) );
 
   if (*p=='[')  {
@@ -1596,7 +1595,7 @@ int ParseAtomPath ( cpstr      ID,
 char    N[100];
 pstr    p,p1;
 int     i,RC;
-Boolean wasRes;
+Boolean wasRes = False;
 
 
   RC = 0;
@@ -1728,7 +1727,7 @@ Boolean wasModel,wasChain,wasRes,haveNeg;
     p++;
   }
   *p1 = char(0);
-  
+
   p = ID;
 
   iModel = 0;
