@@ -1,4 +1,3 @@
-//  $Id: mmdb_utils.h,v 1.19 2008/08/21 09:54:05 nds65 Exp $
 //  =================================================================
 //
 //   CCP4 Coordinate Library: support of coordinate-related
@@ -6,13 +5,13 @@
 //
 //   Copyright (C) Eugene Krissinel 2000-2008.
 //
-//    This library is free software: you can redistribute it and/or 
-//    modify it under the terms of the GNU Lesser General Public 
-//    License version 3, modified in accordance with the provisions 
+//    This library is free software: you can redistribute it and/or
+//    modify it under the terms of the GNU Lesser General Public
+//    License version 3, modified in accordance with the provisions
 //    of the license to address the requirements of UK law.
 //
-//    You should have received a copy of the modified GNU Lesser 
-//    General Public License along with this library. If not, copies 
+//    You should have received a copy of the modified GNU Lesser
+//    General Public License along with this library. If not, copies
 //    may be downloaded from http://www.ccp4.ac.uk/ccp4license.php
 //
 //    This program is distributed in the hope that it will be useful,
@@ -22,7 +21,7 @@
 //
 //  =================================================================
 //
-//    08.07.08   <--  Date of Last Modification.
+//    29.01.10   <--  Date of Last Modification.
 //                   ~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 //  -----------------------------------------------------------------
 //
@@ -47,7 +46,7 @@
 //                   GetReal    ( reads real from a string            )
 //                   GetIntIns  ( reads integer and insert code       )
 //                   PutInteger ( writes integer into a string        )
-//                   PutRealF   ( writes real in F-foram into a string)
+//                   PutRealF   ( writes real in F-form into a string )
 //                   PutIntIns  ( writes integer and insert code      )
 //                   CIFGetInteger ( reads and deletes int from CIF   )
 //                   CIFGetReal    ( reads and deletes real from CIF  )
@@ -60,7 +59,7 @@
 //                   ParseResID     ( parses residue ID line          )
 //                   ParseAtomPath  ( parses full atom path           )
 //
-//   (C) E. Krissinel  2000-2008
+//   (C) E. Krissinel  2000-2010
 //
 //  =================================================================
 //
@@ -126,7 +125,7 @@ extern Boolean GetReal ( realtype & R, cpstr S, int M );
 //   The return is False if no integer number may be
 // recognized. In this case, N is assigned MinInt4 value,
 // "ins" just returns (M+1)th symbol of S (+terminating 0).
-extern Boolean  GetIntIns ( int & N, pstr ins, cpstr S, int M ); 
+extern Boolean  GetIntIns ( int & N, pstr ins, cpstr S, int M );
 
 //  Integer N is converted into ASCII string of length M
 // and pasted onto first M characters of string S. No
@@ -220,7 +219,7 @@ extern realtype TransformZ ( mat44 & T,
 extern char CIFErrorLocation[200];
 
 //  Returns ASCII string explaining the nature of
-// Error_xxxx error code. 
+// Error_xxxx error code.
 extern cpstr  GetErrorDescription ( int ErrorCode );
 
 
@@ -242,10 +241,10 @@ class CContainerClass : public CStream  {
 
     //    ConvertPDBASCII(..) will return one of the Error_XXXXX
     // constants, see <mmdb_defs.h>
-    virtual int     ConvertPDBASCII ( cpstr S  ) { return 0;     }
-    virtual void    PDBASCIIDump    ( pstr S, int N ) {}
-    virtual Boolean PDBASCIIDump1   ( RCFile f )      { return False; }
-    virtual void    MakeCIF         ( PCMMCIFData CIF, int N ) {}
+    virtual int     ConvertPDBASCII ( cpstr     ) { return 0;     }
+    virtual void    PDBASCIIDump    ( pstr, int ) {}
+    virtual Boolean PDBASCIIDump1   ( RCFile    ) { return False; }
+    virtual void    MakeCIF         ( PCMMCIFData, int ) {}
 
     //   Append(..) should return True if CC is appended to this class.
     // If this is not the case, CC is merely put on the top of
@@ -274,14 +273,14 @@ class CContainerClass : public CStream  {
     //                  in <mmdb_defs.h>. This instance of container
     //                  class should be deleted and the whole input
     //                  should be stopped.
-    virtual void GetCIF ( PCMMCIFData CIF, int & Signal )
+    virtual void GetCIF ( PCMMCIFData, int & Signal )
                                { Signal = -1;  }
     virtual int  GetClassID () { return ClassID_Template; }
 
-    virtual void Copy ( PCContainerClass CClass ) {}
+    virtual void Copy ( PCContainerClass ) {}
 
-    void write ( RCFile f ) {}
-    void read  ( RCFile f ) {}
+    void write ( RCFile ) {}
+    void read  ( RCFile ) {}
 
   protected :
     int  ContinuationNo;
@@ -337,7 +336,7 @@ class CClassContainer : public CStream  {
   public :
 
     CClassContainer  ();
-    CClassContainer  ( RPCStream Object ); 
+    CClassContainer  ( RPCStream Object );
     ~CClassContainer ();
 
     void    FreeContainer     ();
@@ -441,7 +440,7 @@ class CAtomPath : public CStream  {
     //     :A      an atom with insertion code A
     //     5       residue number 5
     //     (GLU)   residue GLU
-    // 
+    //
     //   All spaces are ignored. SetPath(..) sets bit of isSet
     // for each element present. Any element may be a wildcard
     // '*'. Wildcard for model will set modelNo=0, for sequence
