@@ -1,4 +1,3 @@
-//  $Id: mmdb_xml.cpp,v 1.19 2008/08/21 09:54:10 nds65 Exp $
 //  =================================================================
 //
 //   CCP4 Coordinate Library: support of coordinate-related
@@ -6,13 +5,13 @@
 //
 //   Copyright (C) Eugene Krissinel 2000-2008.
 //
-//    This library is free software: you can redistribute it and/or 
-//    modify it under the terms of the GNU Lesser General Public 
-//    License version 3, modified in accordance with the provisions 
+//    This library is free software: you can redistribute it and/or
+//    modify it under the terms of the GNU Lesser General Public
+//    License version 3, modified in accordance with the provisions
 //    of the license to address the requirements of UK law.
 //
-//    You should have received a copy of the modified GNU Lesser 
-//    General Public License along with this library. If not, copies 
+//    You should have received a copy of the modified GNU Lesser
+//    General Public License along with this library. If not, copies
 //    may be downloaded from http://www.ccp4.ac.uk/ccp4license.php
 //
 //    This program is distributed in the hope that it will be useful,
@@ -22,7 +21,7 @@
 //
 //  =================================================================
 //
-//    08.07.08   <--  Date of Last Modification.
+//    29.01.10   <--  Date of Last Modification.
 //                   ~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 //  -----------------------------------------------------------------
 //
@@ -33,7 +32,7 @@
 //  **** Classes :  CXMLObject
 //       ~~~~~~~~~
 //
-//   (C) E. Krissinel 2000-2008
+//   (C) E. Krissinel 2000-2010
 //
 //  =================================================================
 //
@@ -282,7 +281,7 @@ int         nTags,i,k;
       k++;
     }
   }
-  
+
   if (SName)  {
     if (SName[0]!=char(1))
       AddObject ( XMLObject1 );
@@ -330,7 +329,7 @@ int         nTags,nRows,i,j,k;
     }
     XMLObject1->AddObject ( XMLObject2 );
   }
-  
+
   if (SName)  {
     if (SName[0]!=char(1))
       AddObject ( XMLObject1 );
@@ -526,7 +525,7 @@ Boolean sngline;
       rm = 72;               // right margin
       lm = IMin ( pos1,36 ); // left margin
       tl = strlen(objTag);
-      if ((pos+tl+2+strlen(objData)<rm-tl-2) &&
+      if ((pos+tl+2+(int)strlen(objData)<rm-tl-2) &&
           (nObjects<=0))  {
         // single-line output
         sngline = True;
@@ -608,6 +607,8 @@ pstr        S1;
 int         k,k1,k2,rc;
 Boolean     Done;
 
+  k2 = 0; // else may become undefined
+
   FreeMemory();
 
   rc = XMLR_Ok;
@@ -716,7 +717,7 @@ Boolean     Done;
 
   if (rc!=XMLR_Ok)  FreeMemory();
   return rc;
-   
+
 }
 
 
